package com.gx.obe.server.common.cache;

import java.io.Serializable;

import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.NoArgsConstructor;

/** 
 * @Description: 缓存对象
 * @author mazc 
 */
@Data
@NoArgsConstructor
@AllArgsConstructor
public class CacheEntity implements Serializable {

	private static final long serialVersionUID = 2082223810638865724L;

	/**
	 *  key
	 */
	private String key;

	/**
	 *  数据
	 */
	private Object value;

	/**
	 * 缓存的时候存的时间戳 
	 */
	private Long timestamp;// 

	/**
	 * 默认长期有效
	 */
	private int seconds = 0; 

	/**
	 *  组容器
	 */
	private Group group;


	/**
	 * 获取剩余时间
	 * 
	 * @return
	 */
	public int ttl() {

		if (this.seconds == 0) {
			return this.seconds;
		}
		return this.seconds - getTime();
	}
	
	/**
	 * 获取当前时间和元素的相差时间
	 * @return
	 */
	private int getTime() {

		if (this.seconds == 0) {
			return this.seconds;
		}
		Long current = System.currentTimeMillis();
		Long value = current - this.timestamp;
		return (int) (value / 1000) + 1;
	}

	/**
	 * 是否到期
	 * 
	 * @return
	 */
	public boolean isExpire() {

		if (this.seconds == 0) {
			return true;
		}
		if (getTime() > this.seconds) {
			// 失效了就移除
			group.delete(key);
			return false;
		}
		return true;
	}
}
