package com.gx.obe.server.im.serverthread;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.IOException;
import java.io.Serializable;
import java.net.Socket;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import com.gx.obe.server.common.utils.IDUtils;
import com.gx.obe.server.im.ChatConstants;
import com.gx.obe.server.im.util.BeanContext;
import com.gx.obe.server.management.evaluation.service.impl.BidPriceResultServiceImpl;
import com.gx.obe.server.management.im.entity.ChatRecord;
import com.gx.obe.server.management.im.entity.Message;
import com.gx.obe.server.management.im.service.ChatRecordService;
import com.gx.obe.server.management.im.service.GroupItemService;
import com.gx.obe.server.management.im.service.MessageService;
 
/**<pre>
 * @Description: 每打开一个聊天窗口，就会打开一个聊天监听线程
 * @Description:
 * @author guoyr
 * </pre>
 */
class ClientChatThread implements Runnable, Serializable {
	
	private static final long serialVersionUID = -9194096328955415477L;
	private Socket clientChatWindowSocket;
	private String fromuserid;
	private String touserid;
	private boolean isGroupChat = false;
	private BufferedInputStream bis;
	private BufferedOutputStream bos;
	private boolean cont = false;
	private String groupId;
	// private String[] groupUsersId = new String[1];
//	private List<Object> groupUsersId = new ArrayList<Object>();
	private int byteCountTotal;
	private List<byte[]> byteList;
	int styleEndFlagIndex = -1;// 一条完整消息结束的标记位
	private static final org.slf4j.Logger LOG = org.slf4j.LoggerFactory.getLogger(BidPriceResultServiceImpl.class);
 	public MessageService messageService = BeanContext.getApplicationContext().getBean(MessageService.class);
 	public ChatRecordService recordService = BeanContext.getApplicationContext().getBean(ChatRecordService.class);
 	public GroupItemService groupItemService = BeanContext.getApplicationContext().getBean(GroupItemService.class);
	 
	private Thread thread;
	private ChatListener listener;
	 
	public ClientChatThread(Socket chatWindowSocket, ChatListener listener) {
		this.clientChatWindowSocket = chatWindowSocket;
		this.listener = listener;
		try {
			bis = new BufferedInputStream(clientChatWindowSocket.getInputStream());
			bos = new BufferedOutputStream(clientChatWindowSocket.getOutputStream());
			cont = true;
		} catch (IOException ex) {
			LOG.error(ex.getMessage(), ex);
 		}
	}
	
	public void start(){
		thread = new Thread(this);
		thread.setName("聊天窗口监听");
		thread.start();
	}
	public void stop(){
		listener.close(this);
		thread = null;
		cont = false;
		if(null != bis){
			try {
				bis.close();
			} catch (IOException e) {
			}
			bis = null;
		}
		if(null != bos){
			try {
				bos.close();
			} catch (IOException e) {
			}
			bos = null;
		}
		if (clientChatWindowSocket != null) {
			try {
				clientChatWindowSocket.close();
			} catch (IOException e) {
			}
			clientChatWindowSocket = null;
		}
	}
	
	
	public void run() {
		try {
			// 处理发送者socket中发来的消息
			byteList = new ArrayList<byte[]>();
			int len = 0;
			int messageBytesLength = 0;
			boolean isLoginOrExitFlag = false;
			boolean hasBegin = false;
			
			while (cont) {
				byte[] bs = new byte[ChatConstants.MAX_LENGTH];
				len = bis.read(bs);
				if (len == -1)
					break;
				byte[] tb = new byte[len];
				for (int i = 0; i < len; i++) {
					tb[i] = bs[i];
				}
				
				//保存本次接收的字节内容到集合
				byteList.add(tb);
				
				/**
				  :T52_]宋体-14-宋体-16-0,0,255-0,0,0-]郭永荣  2014-10-23 16:15:34
					中华人民共和国
				 * **/
				
				// 统计收到的消息样式和消息内容字节的总长度
				int styleAndMessateTotalByteSize = getStyleAndMessageByteLength();
				
				// 如果未获得到消息结束标志，则说明未获得到一条完整的消息，就继续收数据，直到找到 ChatMain.END_TAG为止
				if (styleEndFlagIndex == -1){
					
					// 获得消息样式结尾标记的位置
					styleEndFlagIndex = getStyleEndFlagIndex(styleAndMessateTotalByteSize);
				}
				
				if (styleEndFlagIndex == -1) {
					continue;
				}
				
				if (!hasBegin) {
					String styleAndMessage = getStyleAndMessageStr(styleAndMessateTotalByteSize);// 获得当前获得的所有消息内容
					
					// 如果是刚打开聊天窗口
					if (styleAndMessage.indexOf(ChatConstants.FIRST_LOGIN) == 0 || styleAndMessage.indexOf(ChatConstants.CLIENT_EXIT) == 0) {
						messageBytesLength = 0;
						isLoginOrExitFlag = true;
					} else if (styleAndMessage.indexOf(ChatConstants.TEXT_MESS_START) == 0) {// 如果发送的是纯文字消息
						// 计算发送的消息的长度如52
						messageBytesLength = Integer.parseInt(styleAndMessage.substring(ChatConstants.TEXT_MESS_START.length(), styleAndMessage.indexOf(ChatConstants.END_ATTR)));
					} else {
						messageBytesLength = Integer.parseInt(styleAndMessage.substring(0, styleAndMessage.indexOf(":")));
					}
					// 消息内容的总长度＝消息样式结束标记位置（即消息样式的长度）+消息内容的长度
					byteCountTotal = styleEndFlagIndex + messageBytesLength;
					hasBegin = true;
				}
				// 判断获得的消息样式和消息内容和总长度是否＝消息样式的长度+消息内容的长度
				if (styleAndMessateTotalByteSize < byteCountTotal) {
					continue;
				}
				
				// 如果是第一次打开聊天窗口，判断是私聊还是群聊，并确定对方的id或群id
				if (isLoginOrExitFlag) {
					String str = getStyleAndMessageStr(styleAndMessateTotalByteSize);
					str = str.substring(0, str.indexOf(ChatConstants.END_TAG));
					if (str.indexOf(ChatConstants.CLIENT_EXIT) == 0) {
						break;
					} else if (str.indexOf(ChatConstants.FIRST_LOGIN) == 0) {// 如果是第一次打开聊天窗口
						// :L:2c95650a13e5450c9ee1b7669ce713c7-e719ef5dff2b4205adc1a60ffd138546=0-]
						// 获得第一个id为发送者id
						fromuserid = str.substring(str.lastIndexOf(":") + 1, str.lastIndexOf("-"));
//						fromuserid = id;
						// 获得第二个id为目标接收者id（有可能是群id或私聊对象id）
						String targetId = str.substring(str.lastIndexOf("-") + 1, str.lastIndexOf("="));
						// 获得=后面的参数（即打开的是群消息窗口，还是私聊消息窗口）
						String chatFlag = str.substring(str.lastIndexOf("=") + 1);
						
						// 如果是群聊
						if (Integer.parseInt(chatFlag) == ChatConstants.GROUP_FLAG) {
							groupId = targetId;
							isGroupChat = true;
							// 获得群成员
//							QueryWrapper queryWrapper = new QueryWrapper(GroupItem.class);
//							queryWrapper.setQueryParm("userId");
//							queryWrapper.setWhereParam("groupId", groupId);
//							groupUsersId = groupItemService.queryColumnList(queryWrapper);
//							if (null == groupUsersId) {
//								groupUsersId = new ArrayList<Object>();
//							}
						} else {// 如果是私聊（即一对一聊）
							touserid = targetId;
							isGroupChat = false;
						}
					}
				} else {
					byte[] all = getAllBytes();
					// 如果是私聊
					if (isGroupChat) {
						listener.sendGroupChatMessage(this, all, fromuserid, groupId);
					}else {
						listener.sendChatMessage(this, all, fromuserid, touserid);
					}
				}
				byteList.clear();
				styleEndFlagIndex = -1;
				hasBegin = false;
				isLoginOrExitFlag = false;
			}
		} catch (Exception ex) {
			ex.printStackTrace();
			LOG.error(ex.getMessage(), ex);
		} finally {
			stop();
		}
	}
	
	public int send(byte[] bs, int len) {
		try {
			bos.write(bs, 0, len);
			bos.flush();
		} catch (IOException e) {
			LOG.error(e.getMessage(), e);
			listener.close(this);
			return 0;
		}
		return 1;
	}
	
	
	
	/**
	 * @Description:  保存离线聊天信息
	 * @author guoyr
	 * @param bs
	 * @param sendTime
	 */
	public void saveOfflineMessage(byte[] bs, Date sendTime) {
		try {
			Message message = new Message();
			message.setSendId(fromuserid);
			message.setReceiveId(touserid);
			message.setFirstLength(styleEndFlagIndex);
			message.setMsgContent(bs);
			message.setSendTime(sendTime);
			messageService.saveOrUpdate(message);
		} catch (Exception ex) {
			LOG.error(ex.getMessage(), ex);
		}
	}
	
	
	/**
	 * @Description:  保存群组离线聊天信息
	 * @author guoyr
	 * @param bs
	 * @param groupId
	 * @param sendTime
	 * @return
	 */
	public String saveGroupOfflineMessage(byte[] bs, String groupId, Date sendTime) {
		String msgId = null;
		try {
			Message groupMessage = new Message();
			groupMessage.setId(IDUtils.getId());
			groupMessage.setSendId(fromuserid);
			groupMessage.setReceiveId(touserid);
			groupMessage.setFirstLength(styleEndFlagIndex);
			groupMessage.setGroupId(groupId);
			groupMessage.setMsgContent(bs);
			groupMessage.setSendTime(sendTime);
			messageService.save(groupMessage);
			msgId = groupMessage.getId();
		} catch (Exception ex) {
			LOG.error(ex.getMessage(), ex);
		}
		return msgId;
	}
	
	/**
	 * @Description:  保存聊天记录
	 * @author guoyr
	 * @param bs
	 * @param sendTime
	 */
	public void saveRecord(byte[] bs, Date sendTime) {
		try {
			ChatRecord record = new ChatRecord();
			record.setSendId(fromuserid);
			record.setReceiveId(touserid);
			record.setGroupId(null);
			record.setFirstLength(styleEndFlagIndex);
			record.setMsgContent(bs);
			record.setSendTime(sendTime);
			recordService.saveOrUpdate(record);
		} catch (Exception ex) {
			LOG.error(ex.getMessage(), ex);
		}
	}
	
	
	/**
	 * @Description:  保存群组消息记录
	 * @author guoyr
	 * @param bs
	 * @param groupId
	 * @param sendTime
	 */
	public void writeGroupRecord(byte[] bs, String groupId, Date sendTime) {
		try {
			ChatRecord groupRecord = new ChatRecord();
			groupRecord.setId(IDUtils.getId());
			groupRecord.setSendId(fromuserid);
			groupRecord.setReceiveId(null);
			groupRecord.setFirstLength(styleEndFlagIndex);
			groupRecord.setGroupId(groupId);
			groupRecord.setMsgContent(bs);
			groupRecord.setSendTime(sendTime);
			recordService.save(groupRecord);
		} catch (Exception ex) {
			LOG.error(ex.getMessage(), ex);
		}
	}
	
	
	private String listToString(List<Integer> list) {
		if (list == null || list.isEmpty())
			return "";
		String str = "";
		for (Integer id : list) {
			str += id + ChatConstants.FENGEFU;
		}
		return str.substring(0, str.length() - 1);
	}
	
	/**
	 * @Description:  获得消息样式结尾标记的位置
     * @author guoyr
     * @return
     */
	private int getStyleEndFlagIndex(int styleAndMessateTotalByteSize) {
//		byte[] xb = new byte[getStyleAndMessageByteLength()];
		byte[] xb = new byte[styleAndMessateTotalByteSize];
		int xi = 0;
		for (byte[] b : byteList) {
			for (int j = 0; j < b.length; j++) {
				xb[xi++] = b[j];
			}
		}
		String str = new String(xb);
		
		// 获得消息样式结束标记的位置
		int endFlagIndex = str.indexOf(ChatConstants.END_TAG);
		
		// 如果未获得到消息结束标志，则说明未获得到一条完整的消息
		if (endFlagIndex == -1){
			return -1;
		}else {
			// 如果未获得到消息结束标志，则返加完整消息的长度
			return str.substring(0, endFlagIndex + ChatConstants.END_TAG.length()).getBytes().length;
		}
	}
	
	
	/**
	 * @Description:  获得当前消息样式和消息的内容字符串
	 * @author guoyr
	 * @return
	 */
	private String getStyleAndMessageStr(int styleAndMessateTotalByteSize) {
		byte[] xb = new byte[getStyleAndMessageByteLength()];
		int xi = 0;
		for (byte[] b : byteList) {
			for (int j = 0; j < b.length; j++) {
				xb[xi++] = b[j];
			}
		}
		String str = new String(xb);
		return str;
	}
	
	 /**
	  * @Description:  获得已接收到的消息字的字节包长度（包含消息样式和消息内容）
     * @author guoyr
     * @return
     */
	private int getStyleAndMessageByteLength() {
		int length = 0;
		for (int i = 0; i < byteList.size(); i++) {
			length += ((byte[]) byteList.get(i)).length;
		}
		return length;
	}
	
	
	
	/**
	 * @Description:  获得接收到的所有的消息的长度
	 * @author guoyr
	 * @return
	 */
	private byte[] getAllBytes() {
		byte[] bs = new byte[byteCountTotal];
		int ti = 0;
		for (byte[] b : byteList) {
			for (int j = 0; j < b.length; j++) {
				bs[ti++] = b[j];
			}
		}
		return bs;
	}
	
	public String getMyId(){
		return fromuserid;
	}
	public String getOtherId(){
		return touserid;
	}
	public String getGroupId(){
		return groupId;
	}
	public interface ChatListener{
		
		/**
		 * @Description:  关闭聊天线程（即关闭聊天窗口）
		 * @author guoyr
		 */
		public void close(ClientChatThread chatThread);
		
		
		/**
		 * @Description:  发送消息
		 * @author guoyr
		 * @param bs
		 */
		public void sendChatMessage(ClientChatThread chatThread, final byte[] bs, String sendId, String receiveId);
		
		/**
		 * @Description:  发送群消息
		 * @author guoyr
		 * @param bs
		 */
		public void sendGroupChatMessage(ClientChatThread chatThread, final byte[] bs, String sendId, String groupId);
	}
}
