package com.gx.obe.server.im.serverthread;

import java.net.InetAddress;

import com.gx.obe.server.im.SendFileParam;
import com.gx.obe.server.im.serverthread.SendFileThread.ServerSendFileListener;

/**
 * @Description: 服务器用于监听终端发来的消息
 * @Description:
 * @author guoyr
 */
interface ServerListener {

	
	/**
	 * @Description:  连接成功
	 * @author guoyr
	 * @param client
	 */
	public void connectionSuccess(ClientBean client);
	
	/**
	 * @Description:  退出
	 * @author guoyr
	 * @param client
	 */
	public void quitOut(ClientBean client);
	
	/**
	 * @Description:  发送文件请求
	 * @author guoyr
	 * @param receiveId
	 * @param order
	 */
	public void sendFileRequest(String receiveId, String order);
	
	/**
	 * @Description:  发送文件请求(通过服务器转发)
	 * @author guoyr
	 * @param receiveId
	 * @param order
	 */
	public ServerFileThread sendFileByServerRequest(SendFileParam params, ServerSendFileListener serverSendFileListener);
	
	/**
	 * @Description:  取消文件发送
	 * @author guoyr
	 * @param receiveId
	 * @param order
	 */
	public void cancelSendFile(String receiveId, String order);
	
	/**
	 * @Description:  发送离线文件请求
	 * @author guoyr
	 * @param receiveId
	 * @param order
	 */
	public void sendOfflineFileRequest(String receiveId, String order);
	
	/**
	 * @Description:  发送离线文件
	 * @author guoyr
	 * @param ip
	 * @param order
	 */
	public ServerFileThread sendOfflineFile(InetAddress ip, String order, ServerSendFileListener serverSendFileListener);
	
	/**
	 * @Description:  发送视频请求
	 * @author guoyr
	 * @param receiveId
	 * @param order
	 */
	public void sendVideoRequest(String receiveId, String order);
	
	/**
	 * @Description:  接收视频
	 * @author guoyr
	 * @param receiveId
	 * @param order
	 */
	public void receiveVideo(String receiveId, String order);
	
	/**
	 * @Description:  取消视频
	 * @author guoyr
	 * @param receiveId
	 * @param order
	 */
	public void cancelVideo(String receiveId, String order);
	
	/**
	 * @Description:  发送闪屏
	 * @author guoyr
	 * @param receiveId
	 * @param order
	 */
	public void sendShanPing(String receiveId, String order);
	/**
	 * @Description:  发送通知
	 * @author guoyr
	 * @param receiveId
	 * @param order
	 */
	public void sendNotice(String receiveId, String notice);
	
	/**
	 * @Description:  让客户端断线并重新登录
	 * @author guoyr
	 * @param receiveId
	 * @param order
	 */
	public void reLogin(String myId);
	
	/**
	 * @Description:  发送通知
	 * @author guoyr
	 * @param notice
	 */
	public void sendCommondToAllClient(String notice);
	
	
	/**
	 * @Description:  检查是否已经登录
	 * @author guoyr
	 * @param myId
	 */
	public void checkLogin(String myId, int threadId);
	
}
