package com.gx.obe.server.im.util;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;

import org.slf4j.Logger;

/**
 * @Description:
 * @Description:
 * @author guoyr
 */
public class SocketUtil {
	/**
	 * 分发文件
	 */
	public static final String FILE = "SEND_FILE";
	/**
	 * 登录系统
	 */
	public static final String LOGIN_IN = ":I";
	/**
	 * 退出系统
	 */
	public static final String LOGIN_OUT = ":O";
	
	/**
	 * 完成
	 */
	public static final String FINISH = "FINISH";
	/**
	 * 刷新在线状态
	 */
	public static final String FINISH_ONLINE_STATUS = ":R_ON";
	/**
	 * 下线
	 */
	public static final String FINISH_OUT_LINE_STATUS = ":R_OUT";
	public static Logger LOG = org.slf4j.LoggerFactory.getLogger(SocketUtil.class);
	
	/**
	 * @Description:  发指令
	 * @author guoyr
	 * @param order
	 * @param os
	 * @throws IOException
	 */
	public static void sendOrder(String order, OutputStream os) throws IOException {
		sendMessage(order, os);
	}
	
	/**
	 * @Description:  读取命令
	 * @author guoyr
	 * @param is
	 * @return
	 * @throws IOException
	 */
	public static String readOrder(InputStream is) throws IOException {

		int order_len = readInteger(is);
		byte[] result = new byte[order_len];
		is.read(result);

		return new String(result);

	}
	

	
	/**
	 * @Description:  分发文件
	 * @author guoyr
	 * @param fileDirectory
	 * @param filepath
	 * @param os
	 * @return
	 */
	public static boolean sendFile(String fileDirectory, String filepath, OutputStream os){
		boolean sendSuccess = true;
		File file = new File(filepath);
		FileInputStream fileInputStream = null;
		try {
		
			fileInputStream = new FileInputStream(filepath);
			// 发送socket指令
			SocketUtil.sendOrder(FILE, os);
			int length = (int) file.length();
			LOG.debug("发送文件：" + file.getName() + "，长度：" + length);
			
			// 输入文件目录
			SocketUtil.sendMessage(fileDirectory, os);
			// 发送文件名
			SocketUtil.sendFileName(file, os);

			// 输出文件长度
			SocketUtil.sendFileSize(length, os);

			// 输出文件内容
			byte[] buffer = new byte[4096];

			int size;
			while ((size = fileInputStream.read(buffer)) != -1) {
				os.write(buffer, 0, size);
			}
			
		} catch (FileNotFoundException e1) {
			sendSuccess = false;
			LOG.error(e1.getMessage());
		} catch (IOException e) {
			sendSuccess = false;
			LOG.error(e.getMessage());
		}finally{
			if(null != os){
				try {
					os.close();
				} catch (IOException e1) {
				}
			}
			if(null != fileInputStream){
				
				try {
					fileInputStream.close();
				} catch (IOException e) {
				}
			}
		}

		try {

		} catch(Exception e){
			sendSuccess = false;
		} finally {
			
		}
		return sendSuccess;

	}
	
	
	/**
	 * @Description:  从流中读取内容并保存
	 * @author guoyr
	 * @param is
	 * @throws IOException
	 */
	public static boolean readAndSaveFile(String savePath, InputStream is) throws IOException {
		boolean readSuccess = true;
		String fileDirectory = SocketUtil.readFileDirectory(is);
		String filename = SocketUtil.readFileName(is);
		int file_len = SocketUtil.readFileSize(is);
		LOG.debug("接收文件：" + filename + "，长度：" + file_len);

		// 文件保存位置
		StringBuilder filePath = new StringBuilder();
		filePath.append(savePath);
		filePath.append(File.separator);
		if(null != fileDirectory && fileDirectory.length() > 0){
			filePath.append(fileDirectory);
			filePath.append(File.separator);
		}
		filePath.append(filename);
		FileOutputStream os = getFileOutPutStream(filePath.toString());
		int readFileSize = readAndWrite(is, os, file_len);
		os.close();
		readSuccess = (readFileSize == file_len);
		if(readSuccess){
			LOG.debug("文件保存成功（" + file_len + "字节）。");
		}else {
			LOG.debug("文件保存失败，发送" + file_len + "字节，收到"+readFileSize+"字节）。");
		}
		return readSuccess;

	}

	/**
	 * @Description:  发送消息
	 * @author guoyr
	 * @param message
	 * @param os
	 * @throws IOException
	 */
	public static void sendMessage(String message, OutputStream os)throws IOException{
		byte[] fn_bytes = message.getBytes();
		
		os.write(i2b(fn_bytes.length)); // 输出文件名长度
		
		os.write(fn_bytes); // 输出消息
	}
	
	/**
	 * @Description:  读取文件名称（包括路径）
	 * @author guoyr
	 * @param is
	 * @return
	 * @throws IOException
	 */
	public static String readMessage(InputStream is) throws IOException {
		
		int message_len = readInteger(is);
		
		byte[] result = new byte[message_len];
		is.read(result);
		
		return new String(result);
		
	}
	/**
	 * @Description:  读取文件的目录
	 * @author guoyr
	 * @param is
	 * @return
	 * @throws IOException
	 */
	public static String readFileDirectory(InputStream is) throws IOException {
		return readMessage(is);
	}
	
	/**
	 * @Description:  输出文件名
	 * @author guoyr
	 * @param file
	 * @param os
	 * @throws IOException
	 */
	public static void sendFileName(File file, OutputStream os) throws IOException {
		// 输出文件名长度和名称
		sendMessage(file.getName(), os);
	}
	/**
	 * @Description:  读取文件名称（包括路径）
	 * @author guoyr
	 * @param is
	 * @return
	 * @throws IOException
	 */
	public static String readFileName(InputStream is) throws IOException {
		return readMessage(is);
	}
	
	/**
	 * @Description:  发送文件大小
	 * @author guoyr
	 * @param message
	 * @param os
	 * @throws IOException
	 */
	public static void sendFileSize(int length, OutputStream os)throws IOException{
		sendInteger(length, os);
	}
	
	/**
	 * @Description:  读取文件大小
	 * @author guoyr
	 * @param is
	 * @return
	 * @throws IOException
	 */
	public static int readFileSize(InputStream is) throws IOException {
		return readInteger(is);

	}
	
	/**
	 * @Description:  发送Integer
	 * @author guoyr
	 * @param message
	 * @param os
	 * @throws IOException
	 */
	public static void sendInteger(int integer, OutputStream os)throws IOException{
		os.write(i2b(integer));
	}
	
	/**
	 * @Description:  读取一个数字
	 * @author guoyr
	 * @param is
	 * @return
	 * @throws IOException
	 */
	public static int readInteger(InputStream is) throws IOException {
		
		byte[] bytes = new byte[4];
		
		is.read(bytes);
		
		return b2i(bytes);
		
	}
	
	/**
	 * @Description:  边读边写，直到读取 size 个字节
	 * @author guoyr
	 * @param is
	 * @param os
	 * @param size
	 * @throws IOException
	 */
	public static int readAndWrite(InputStream is, FileOutputStream os, int size)
			throws IOException {

		byte[] buffer = new byte[4096];
		int count = 0;
		while (count < size) {
		
					int n = is.read(buffer);
		
					// 这里没有考虑 n = -1 的情况
		
					os.write(buffer, 0, n);
		
					count += n;
				}
//		int len;
//		while ((len = is.read(buffer)) != -1) {
//			os.write(buffer, 0, len);
//			count += len;
//		}
		return count;
	}
	
	
	/**
	 * @Description:  创建文件并返回输出流
	 * @author guoyr
	 * @param path
	 * @return
	 * @throws IOException
	 */
	public static FileOutputStream getFileOutPutStream(String path) throws IOException {

		File file = new File(path);
		if (!file.exists()) {
			
			// 如果上级目录不存在，则先新建上级目录
			if(!file.getParentFile().exists()){
				file.getParentFile().mkdirs();
			}
			file.createNewFile();

		}
		return new FileOutputStream(file);
	}
	
	
	/**
	 * @Description:  将字节转成 int。b 长度不得小于 4，且只会取前 4 位。
	 * @author guoyr
	 * @param b
	 * @return
	 */
	public static int b2i(byte[] b) {

		int value = 0;

		for (int i = 0; i < 4; i++) {

			int shift = (4 - 1 - i) * 8;

			value += (b[i] & 0x000000FF) << shift;

		}
		return value;
	}
	
	/**
	 * @Description:  将 int 转成字节
	 * @author guoyr
	 * @param i
	 * @return
	 */
	public static byte[] i2b(int i) {

		return new byte[] {

		(byte) ((i >> 24) & 0xFF),

		(byte) ((i >> 16) & 0xFF),

		(byte) ((i >> 8) & 0xFF),

		(byte) (i & 0xFF)

		};

	}
}
