package com.gx.obe.server.management.dictionary.controller;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.gx.obe.server.common.base.BaseController;
import com.gx.obe.server.common.utils.IDUtils;
import com.gx.obe.server.common.utils.StringUtils;
import com.gx.obe.server.management.dictionary.entity.WorkMember;
import com.gx.obe.server.management.dictionary.service.WorkMemberService;

import io.swagger.annotations.Api;
import lombok.AllArgsConstructor;
/** 
 * @Description: 
 * @author mazc 
 */
@Api(tags = "")
@RestController
@AllArgsConstructor
@RequestMapping("/workMember")
public class WorkMemberController extends BaseController<WorkMemberService,WorkMember>{
	@Autowired
	public WorkMemberService workMemberServices;
	/**
	 * @Description: 批量插入
	 * @author mazc
	 * @param workMemberList 
	 */
	@PostMapping("/insertByBatch")
	public int insertByBatch(@RequestBody List<WorkMember> workMemberList) {
		return workMemberServices.insertByBatch(workMemberList);
	}
	/**
	 * @Description: 批量更新
	 * @author mazc
	 * @param workMemberList 
	 */
	@PostMapping("/updateByBatch")
	public int updateByBatch(@RequestBody List<WorkMember> workMemberList) {
		return workMemberServices.updateByBatch(workMemberList);
	}

	/**
	 * @Description: 返回指定类型的工作人员
	 * @author mazc
	 * @param tenderId
	 * @param workerType
	 * @return 
	 */
	@GetMapping("/getWorkerList")
	public List<WorkMember> getWorkerList(boolean isParentProject,String tenderId, String workerType){
		return workMemberServices.getWorkerList(isParentProject,tenderId, workerType);
	}
	
	/**
	 * @Description: 返回指定项目的工作人员
	 * @author mazc
	 * @param tenderId
	 * @param userId
	 * @return 
	 */
	@GetMapping("/getWorker")
	public WorkMember getWorker(String tenderId, String userId){
		return workMemberServices.getWorker(tenderId, userId);
	}
	
	@GetMapping("/deleteWorker")
	public boolean deleteWorker(String tenderId, String userId,String wokerType){
		return workMemberServices.deleteWorker(tenderId, userId, wokerType);
	}
	
	/**
	 * @Description: 删除所有工作人员
	 * @author mazc
	 * @param tenderId
	 * @param wokerType
	 * @return 
	 */
	@GetMapping("/delAllWorkers")
	public boolean delAllWorkers(String tenderId,String wokerType,String monitorType){
		return workMemberServices.delAllWorkers(tenderId, wokerType,monitorType);
	}
	
	
	@PostMapping("/batchSaveOrUpdate")
	public int batchSaveOrUpdate(@RequestBody List<WorkMember> workMemberList){
		return workMemberServices.batchSaveOrUpdate(workMemberList);
	}
	
	@PostMapping("/saveOrUpdate")
	public boolean saveOrUpdate(@RequestBody WorkMember worker){
		return workMemberServices.saveOrUpdate(worker);
	}
	
	@PostMapping("/saveOrUpdateReturn")
	public  WorkMember saveOrUpdateReturn(@RequestBody WorkMember worker){
		if(null == worker) {
			return worker;
		}
		if(StringUtils.isNotEmpty(worker.getId())) {
			workMemberServices.updateById(worker);
		}else {
			worker.setId(IDUtils.getId());
			workMemberServices.save(worker);
		}
		return worker;
	}
	
	@PostMapping("/updateAssignProperty")
	public boolean updateAssignProperty(@RequestBody WorkMember workMember) {
		return workMemberServices.updateAssignProperty(workMember);
	}
	
	@GetMapping("/getWorkerByUserIdAndTenderIdOrMemberType")
	public WorkMember getWorkerByUserIdAndTenderIdOrMemberType(String userId,String tenderId,String memberType){
		return workMemberServices.getWorkerByUserIdAndTenderIdOrMemberType(userId, tenderId, memberType);
	}
	
}