package com.gx.obe.server.management.dictionary.service;

import com.baomidou.mybatisplus.extension.service.IService;
import java.util.List;
import com.gx.obe.server.management.dictionary.entity.BaseDictionary;
import com.gx.obe.server.management.dictionary.entity.BaseDictionaryValue;
 
/** 
 * @Description: 
 * @author mazc 
 */
public interface BaseDictionaryService extends IService<BaseDictionary> {
 	/**
	 * @Description: 批量更新
	 * @author mazc
	 * @param BaseDictionaryList 
	 */
 	Integer updateByBatch (List<BaseDictionary> BaseDictionaryList);
	/**
	 * @Description: 批量插入
	 * @author mazc
	 * @param BaseDictionaryList 
	 */
	Integer insertByBatch (List<BaseDictionary> BaseDictionaryList);


	/**
	 * @Description: 验证字典名称唯一性
	 * @author mazc
	 * @param id
	 * @param dictName
	 * @return boolean true：是唯一；false:不是唯一
	 */
	public boolean isUniqueDictName(String id, String dictName);
	
	/**
	 * @Description: 验证字典编码唯一性
	 * @author mazc
	 * @param id
	 * @param dictCode
	 * @return
	 */
	public boolean isUniqueDictCode(String id, String dictCode);
	
	/**
	 * @Description: 根据编码，获得数据字典对象。
	 * @author mazc
	 * @param dictCode
	 * @return
	 */
	public BaseDictionary getDictionaryByCode(String dictCode);
	
	/**
	 * @Description: 根据编码，获得字典和字典值。
	 * @author mazc
	 * @param dictCode
	 * @return
	 */
	public BaseDictionary getDictionaryAndValuesByCode(String dictCode);
	
	/**
	 * @Description: 根据字典编码，获得有效字典值集合。
	 * @author mazc
	 * @param dictCode 字典编码
	 * @return List<DictionaryValue>
	 */
	public List<BaseDictionaryValue> getDictionaryValueList(String dictCode);
	
	
	 /**
	 * @Description: 查询所有，按照创建时间排序
	 * @author mazc
	 * @return 
	 */
	List<BaseDictionary> getDictionaryListOrderByCreateTime();
	

}



