package com.gx.obe.server.management.dictionary.service.impl;

import java.util.ArrayList;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.gx.obe.server.common.utils.StringUtils;
import com.gx.obe.server.management.dictionary.dao.AwardCertificateMapper;
import com.gx.obe.server.management.dictionary.entity.AwardCertificate;
import com.gx.obe.server.management.dictionary.service.AwardCertificateService;

/** 
 * @Description: 
 * @author mazc 
 */
@Service
@Transactional
public class AwardCertificateServiceImpl extends ServiceImpl<AwardCertificateMapper, AwardCertificate> implements AwardCertificateService {

	@Autowired
	private AwardCertificateMapper awardCertificateMapper;
	
	/**
	 * @Description: 批量更新
	 * @author mazc
	 * @param AwardCertificateList 
	 */
	 @Override
	 @Transactional(rollbackFor = Exception.class)
	public Integer updateByBatch (List<AwardCertificate> AwardCertificateList){
		return awardCertificateMapper.updateBatchList(AwardCertificateList);
	}
	
	/**
	 * @Description: 批量插入
	 * @author mazc
	 * @param AwardCertificateList 
	 */
	 @Override
	 @Transactional(rollbackFor = Exception.class)
	public Integer insertByBatch (List<AwardCertificate> AwardCertificateList){
		return awardCertificateMapper.insertByBatch(AwardCertificateList);
	}

	/** 
	 * @Description: 获得当前奖项类型下全部供应商
	 * @author mazc
	 * @param awardName
	 * @return 
	 */
	@Override
	public List<AwardCertificate> getAwardCertificateList(String awardName) {
 		return awardCertificateMapper.getAwardCertificateList(awardName);
	}

	/** 
	 * @Description: 是否含有此供应商
	 * @author mazc
	 * @param orgName
	 * @param awardName
	 * @return 
	 */
	@Override
	public boolean isHasSupplier(String orgName, String awardName) {
		List<String> awardNameList = new ArrayList<String>();
		awardNameList.add("国家科学技术进步奖");
		awardNameList.add("中国电力科学技术奖");
		return hasSupplier(orgName, awardName, !awardNameList.contains(awardName)) > 0;
	}
	
	/**
	 * @Description: 表单中是否含有
	 * @author chenxw
	 * @param orgName
	 * @param awardName
	 * @param isOrAwardOwner
	 * @return 
	 */
	private int hasSupplier(String orgName, String awardName, boolean isOrAwardOwner) {
		if (StringUtils.isEmpty(orgName) || StringUtils.isEmpty(awardName)) {
			return 0;
		} 
		return  awardCertificateMapper.hasSupplier(orgName,awardName, isOrAwardOwner);
	}
	
	/**
	 * @Description: 更新
	 * @author mazc
	 * @param awardCertificate
	 * @return 
	 */
	@Override
	public int update(AwardCertificate awardCertificate) {
		return awardCertificateMapper.updateById(awardCertificate);
	}

	/** 
	 * @Description: 判断厂商是否已经存在
	 * @author mazc
	 * @param orgName
	 * @param awardName
	 * @return 
	 */
	@Override
	public boolean hasSupplier(String orgName, String awardName) {
		return hasSupplier(orgName, awardName, false) > 0;
	}

	

}
