package com.gx.obe.server.management.dictionary.service.impl;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.gx.obe.server.management.dictionary.dao.DecEnvelopeMapper;
import com.gx.obe.server.management.dictionary.entity.DecEnvelope;
import com.gx.obe.server.management.dictionary.service.DecEnvelopeService;

/** 
 * @Description: 
 * @author mazc 
 */
@Service
@Transactional
public class DecEnvelopeServiceImpl extends ServiceImpl<DecEnvelopeMapper, DecEnvelope> implements DecEnvelopeService {

	@Autowired
	private DecEnvelopeMapper decEnvelopeMapper;
	
	/**
	 * @Description: 批量更新
	 * @author mazc
	 * @param ObeDecEnvelopeList 
	 */
	 @Override
	 @Transactional(rollbackFor = Exception.class)
	public Integer updateByBatch (List<DecEnvelope> ObeDecEnvelopeList){
		return decEnvelopeMapper.updateBatchList(ObeDecEnvelopeList);
	}
	
	/**
	 * @Description: 批量插入
	 * @author mazc
	 * @param ObeDecEnvelopeList 
	 */
	 @Override
	 @Transactional(rollbackFor = Exception.class)
	public Integer insertByBatch (List<DecEnvelope> ObeDecEnvelopeList){
		return decEnvelopeMapper.insertByBatch(ObeDecEnvelopeList);
	}

	/** 
	 * @Description: 获得解密信封
	 * @author mazc
	 * @param tenderId
	 * @param supplierId
	 * @return 
	 */
	@Override
	public DecEnvelope getDecEnvelope(String tenderId, String supplierId) {
 		return decEnvelopeMapper.getDecEnvelope(tenderId, supplierId);
	}

	/** 
	 * @Description: 上传解密信封
	 * @author mazc
	 * @param decEnvelope
	 * @return 
	 */
	@Override
	@Transactional(rollbackFor = Exception.class)
	public boolean uploadDecEnvelope(DecEnvelope decEnvelope) {
		QueryWrapper<DecEnvelope> queryWrapper = new QueryWrapper<DecEnvelope>();
		queryWrapper.lambda().eq(DecEnvelope::getTenderId, decEnvelope.getTenderId()).eq(DecEnvelope::getSupplierId, decEnvelope.getSupplierId());
		decEnvelopeMapper.delete(queryWrapper);
		return decEnvelopeMapper.insert(decEnvelope) > 0;
	}

	/** 
	 * @Description: 是否已经存在解密信封
	 * @author mazc
	 * @param tenderId
	 * @param supplierId
	 * @return 
	 */
	@Override
	public boolean hasDecEnvelope(String tenderId, String supplierId) {
		return decEnvelopeMapper.hasDecEnvelope(tenderId, supplierId) > 0;
	}
	

}
