package com.gx.obe.server.management.editprice.service.impl;

import java.util.ArrayList;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.gx.obe.server.common.entity.ReflecUtils;
import com.gx.obe.server.common.enumeration.BidPriceEnum;
import com.gx.obe.server.common.utils.IDUtils;
import com.gx.obe.server.management.editprice.dao.EditPriceItemMapper;
import com.gx.obe.server.management.editprice.dao.EditPriceItemResultMapper;
import com.gx.obe.server.management.editprice.entity.EditPriceItem;
import com.gx.obe.server.management.editprice.entity.EditPriceItemResult;
import com.gx.obe.server.management.editprice.service.EditPriceItemService;

/** 
 * @Description: 
 * @author mazc 
 */
@Service
@Transactional
public class EditPriceItemServiceImpl extends ServiceImpl<EditPriceItemMapper, EditPriceItem> implements EditPriceItemService {

	@Autowired
	private EditPriceItemMapper editPriceItemMapper;
	
	@Autowired
	private EditPriceItemResultMapper editPriceItemResultMapper;
	/**
	 * @Description: 批量更新
	 * @author mazc
	 * @param ObeEditPriceItemList 
	 */
	 @Override
	 @Transactional(rollbackFor = Exception.class)
	public Integer updateByBatch (List<EditPriceItem> ObeEditPriceItemList){
		return editPriceItemMapper.updateBatchList(ObeEditPriceItemList);
	}
	
	/**
	 * @Description: 批量插入
	 * @author mazc
	 * @param ObeEditPriceItemList 
	 */
	 @Override
	 @Transactional(rollbackFor = Exception.class)
	public Integer insertByBatch (List<EditPriceItem> ObeEditPriceItemList){
		return editPriceItemMapper.insertByBatch(ObeEditPriceItemList);
	}

	/** 
	 * @Description: 获取偏离性条款修正报价项
	 * @author mazc
	 * @param tenderId
	 * @return 
	 */
	@Override
	public EditPriceItem getDeviateCaluseEditPriceItem(String tenderId) {
		EditPriceItem editPriceItem = editPriceItemMapper.getDeviateCaluseEditPriceItem(tenderId);
		if(null != editPriceItem) {
			editPriceItem = new EditPriceItem();
			editPriceItem.setId(IDUtils.getId());
			editPriceItem.setTenderId(tenderId);
			editPriceItem.setItemCode(BidPriceEnum.DEVIATE_CALUSE);
			editPriceItem.setItemName("偏离性条款");
			editPriceItem.setEditType(BidPriceEnum.EDIT_FLOAT);
			editPriceItemMapper.insert(editPriceItem);
		}
		return null;
	}

	/** 
	 * @Description: 获取修正报价项列表
	 * @author mazc
	 * @param tenderId
	 * @return 
	 */
	@Override
	public List<EditPriceItem> getEditPriceItemList(String tenderId) {
		QueryWrapper<EditPriceItem> queryWrapper = new QueryWrapper<EditPriceItem>();
		queryWrapper.lambda().eq(EditPriceItem::getTenderId, tenderId).orderByAsc(EditPriceItem::getSortNo);
		return editPriceItemMapper.selectList(queryWrapper);
	}

	
	/** 
	 * @Description: 保存修正报价项列表
	 * @author mazc
	 * @param editPriceItemList
	 * @param removeList
	 * @return 
	 */
	@Override
	@Transactional(rollbackFor = Exception.class)
	public boolean saveEditPriceItemList(List<EditPriceItem> editPriceItemList, List<EditPriceItem> removeList) {
		batchSaveOrUpdate(editPriceItemList, new String[] { "id","itemName", "editType", "sortNo" });
		editPriceItemMapper.deleteBatchIds(removeList);
		for (EditPriceItem remove : removeList) {
			QueryWrapper<EditPriceItemResult> queryWrapper = new QueryWrapper<EditPriceItemResult>();
			queryWrapper.lambda().eq(EditPriceItemResult::getTenderId, remove.getTenderId()).eq(EditPriceItemResult::getEditPriceItemId, remove.getId());
			editPriceItemResultMapper.delete(queryWrapper);
		}
 		return true;
	}
	
 

	/** 
	 * @Description: 批量更新指定字段
	 * @author mazc
	 * @param editPriceItemResultList
	 * @param attributes
	 * @return 
	 */
	@Override
	@Transactional(rollbackFor = Exception.class)
	public Integer batchSaveOrUpdate(List<EditPriceItem> editPriceItemltList, String[] attributes) {
		int iCount = 0;
		List<EditPriceItem> insertEditPriceItemListTemp = new ArrayList<EditPriceItem>();
		List<EditPriceItem> UpdateeditPriceIteListTemp = new ArrayList<EditPriceItem>();
		for(EditPriceItem editPriceItemResult : editPriceItemltList) {
			EditPriceItem editPriceItemResultTemp = ReflecUtils.reflex(editPriceItemResult, attributes ,EditPriceItem.class);
			if(null != editPriceItemResultTemp) {
				if(null == editPriceItemResultTemp.getId()) {
					editPriceItemResultTemp.setId(IDUtils.getId());
					insertEditPriceItemListTemp.add(editPriceItemResultTemp);
				}else {
					UpdateeditPriceIteListTemp.add(editPriceItemResultTemp);
				}
			}
		}
		iCount += editPriceItemMapper.updateBatchList(UpdateeditPriceIteListTemp);
		iCount += editPriceItemMapper.insertByBatch(insertEditPriceItemListTemp);
		return iCount;	
 	}
}
