package com.gx.obe.server.management.evaluation.dao;

import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;

import com.gx.obe.server.management.evaluation.entity.EvaluationFactorResult;
import com.baomidou.mybatisplus.core.mapper.BaseMapper;

import java.math.BigDecimal;
import java.util.List;
import java.util.Map;
/** 
 * @Description: 
 * @author mazc 
 */
 /** 
 * @Description: 
 * @author mazc 
 */
@Mapper
public interface EvaluationFactorResultMapper extends BaseMapper<EvaluationFactorResult> {

	/**
	 * @Description: 批量修改
	 * @author mazc
	 * @param ObeEvaluationFactorResultList 
	 */
	Integer updateBatchList (@Param("ObeEvaluationFactorResultList")List<EvaluationFactorResult> ObeEvaluationFactorResultList);
	
	/**
	 * @Description: 批量插入
	 * @author mazc
	 * @param ObeEvaluationFactorResultList 
	 */
	Integer insertByBatch (@Param("ObeEvaluationFactorResultList")List<EvaluationFactorResult> ObeEvaluationFactorResultList);
	
	/**
	 * @Description: 获得评标专家的评标结果列表
	 * @author mazc
	 * @param tenderId
	 * @param stepId
	 * @param userId
	 * @param supplierId
	 * @return 
	 */
	List<EvaluationFactorResult> getExpertEvaluationFactorResult(@Param("tenderId")String tenderId, @Param("stepId")String stepId,
												  @Param("userId")String userId, @Param("supplierId")String supplierId);
	
	 /**
	 * @Description: 获取同一指标评标结果不同的列表
	 * @author mazc
	 * @param tenderId
	 * @param stepId
	 * @param isScoreFactor
	 * @return 
	 */
	List<Map<String, Object>> getSupplierDistinctFactorResultList(@Param("tenderId")String tenderId, @Param("stepId")String stepId,
			 @Param("isScoreFactor")boolean isScoreFactor);
	
	/**
	 * @Description: 删除评标专家对指定评标步骤的所有评审记录
	 * @author mazc
	 * @param tenderId
	 * @param stepId
	 * @param userId
	 * @return 
	 */
	Integer deleteEvaluationFactorResult(@Param("tenderId")String tenderId, @Param("stepId")String stepId,@Param("userId") String userId);
	
	/**
	 * @Description: 获得当前评标步骤下供应商偏离性指标个数
	 * @author mazc
	 * @param tenderId
	 * @param stepId
	 * @param minDeviateCount
	 * @return 
	 */
	List<Map<String, Object>> getSupplierDeviateClauseCountList(@Param("tenderId")String tenderId,@Param("stepId") String stepId, @Param("minDeviateCount")int minDeviateCount); 
	
	/**
	 * @Description: 删除一个或多个用户数据
	 * @author mazc
	 * @param tenderId
	 * @param stepId
	 * @param userIdList
	 * @return 
	 */
	Integer deleteEvaluationResultAndUserId(@Param("tenderId") String tenderId,@Param("stepId")String stepId,@Param("userIdList")List<String> userIdList);
	
	/**
	 * @Description: 重新开始评审  删除评审指标的评标结果
	 * @author mazc
	 * @param tenderId
	 * @return 
	 */
	Integer deleteRestartEvaluation(@Param("tenderId")String tenderId);

    /**
     * @Description: 获得指标的平均分
     * @author chenxw
     * @param tenderId
     * @param stepId
     * @param supplierId
     * @param removeMaxMin
     * @return 
     */
    List<Map<String, Object>> getEvaluationFactorAvgScoreList(@Param("tenderId")String tenderId, @Param("stepId")String stepId, @Param("supplierId")String supplierId, @Param("removeMaxMin")boolean removeMaxMin);

    /**
     * @Description: 根据指标Id获取对应的指标评审结果
     * @author chenxw
     * @param tenderId
     * @param stepId
     * @param factorId
     * @return 
     */
    List<EvaluationFactorResult> getEvaluationFactorResultList(@Param("tenderId")String tenderId, @Param("stepId")String stepId, @Param("factorId")String factorId, @Param("userId")String userId);

    /**
     * @Description: 获得评标专家的评标结果列表
     * @author chenxw
     * @param tenderId
     * @param factorType
     * @return 
     */
    List<EvaluationFactorResult> getResultListByFactorType(@Param("tenderId")String tenderId, @Param("factorType")String factorType);

    /**
     * @Description: 获取指定指标不同评审结果的个数
     * @author chenxw
     * @param tenderId
     * @param stepId
     * @param factorId
     * @param userId
     * @param result
     * @param score
     * @return 
     */
    Integer getSupplierDistinctFactorCount(@Param("tenderId")String tenderId, @Param("stepId")String stepId, @Param("factorId")String factorId, @Param("userId")String userId, @Param("result")String result, @Param("score")BigDecimal score);

    /**
     * @Description: 获得按供就商打分结果列表
     * @author chenxw
     * @param projectId
     * @param groupCode
     * @param userId
     * @param supplierName
     * @return 
     */
    List<Map<String, Object>> getSupplierEvaluationFactorResult(@Param("projectId")String projectId, @Param("groupCode")String groupCode, @Param("userId")String userId, @Param("supplierName")String supplierName);

    /**
     * @Description: 更新供应商评审指标得分
     * @author chenxw
     * @param evalScore
     * @param projectId
     * @param groupCode
     * @param userId
     * @param supplierName 
     */
    Integer updateSupplierEvalFactorScore(@Param("evalScore")BigDecimal evalScore, @Param("projectId")String projectId, @Param("groupCode")String groupCode, @Param("userId")String userId, @Param("supplierName")String supplierName);

    /**
     * @Description: 插入供应商评审指标结果
     * @author chenxw
     * @param projectId
     * @param groupCode
     * @param userId
     * @param supplierName
     * @param factorName
     * @param evalScore
     * @return 
     */
    Integer insertSupplierEvaluationFactorResult(@Param("projectId")String projectId, @Param("groupCode")String groupCode, @Param("userId")String userId, @Param("supplierName")String supplierName, @Param("factorName")String factorName, @Param("evalScore")BigDecimal evalScore);

}