package com.gx.obe.server.management.evaluation.service;

import java.util.Date;
import java.util.List;

import org.springframework.transaction.annotation.Transactional;

import com.baomidou.mybatisplus.extension.service.IService;
import com.gx.obe.server.management.evaluation.entity.BidPrice;
import com.gx.obe.server.management.evaluation.entity.BidPriceItem;
import com.gx.obe.server.management.evaluation.entity.BidPriceItemResult;
import com.gx.obe.server.management.evaluation.entity.BidPriceResult;
import com.gx.obe.server.management.project.entity.TenderProject;
import com.gx.obe.server.management.supplier.entity.SupplierOpeningResult;
 
/** 
 * @Description: 
 * @author mazc 
 */
public interface BidPriceResultService extends IService<BidPriceResult> {
 
	/**
	 * @Description: 获得供应商开标一览表
	 * @author guoyr
	 * @param tenderId
	 * @param supplierId
	 * @return 
	 */
	public List<BidPriceResult> getBidPriceResultTableList(String tenderId, String supplierId);
	
	public List<BidPriceResult> findBidPriceResultList(String tenderId, String bidPriceId);


	/**
	 * @Description: 指定字段修改
	 * @author chenxw
	 * @param ObeBidPriceResult
	 * @param attributes
	 */
	boolean updateAssignProperty(BidPriceResult ObeBidPriceResult, String[] attributes);

	/**
	 * @Description: 批量指定字段修改
	 * @author chenxw
	 * @param ObeBidPriceResultList
	 * @param attributes
	 */
	Integer batchUpdateProperty(List<BidPriceResult> ObeBidPriceResultList, String[] attributes);

	/**
	 * @Description: 批量添加或跟新
	 * @author chenxw
	 * @param ObeBidPriceResultList
	 * @param attributes
	 * @return
	 */
	Integer batchSaveOrUpdate(List<BidPriceResult> ObeBidPriceResultList, String[] attributes);
	
	 /**
	 * @Description: 获得分项报价结果
	 * @author mazc
	 * @param tenderId
	 * @return 
	 */
	List<BidPriceResult> getBidPriceResultList(String tenderId);
	
	
	/**
	 * @Description: getBidPriceResultList
	 * @author mazc
	 * @param tenderId
	 * @return 
	 */
	List<BidPriceResult> getSingBidPriceResultList(String tenderId);
	
	
	/**
	 * @Description: 获得供应商的分项报价
	 * @author mazc
	 * @param tenderId
	 * @param supplierId
	 * @return 
	 */
	List<BidPriceResult> getSupplierBidPriceResultList(String tenderId, String supplierId);
	
	
	/**
	 * @Description: 获得供应商的分项报价
	 * @author mazc
	 * @param bidPriceId
	 * @param supplierId
	 * @return 
	 */
	BidPriceResult getSupplierBidPriceResult(String bidPriceId, String supplierId);
	
	/**
	 * @Description: 保存或更新
	 * @author mazc
	 * @param supplierEvaluationFactorList
	 * @return 
	 */
	Integer batchSaveOrUpdate(List<BidPriceResult> bidPriceResultList);
	/**
	 * @Description: 保存分项报价和总报价
	 * @author mazc
	 * @param tenderId
	 * @param supplierOpeningResultList
	 * @param bidPriceResultList
	 * @param isEvaluationPrice
	 * @return 
	 */
	boolean saveSubBidPriceResult(String tenderId, List<SupplierOpeningResult> supplierOpeningResultList, List<BidPriceResult> bidPriceResultList, boolean isEvaluationPrice);

	/**
	 * @Description: 批量保存供应商开标一览表数据
	 * @author mazc
	 * @param tenderId
	 * @param supplierId
	 * @param bidPriceResultList
	 * @param bidPriceItemResultList 
	 */
	void batchSaveSupplierBidPriceResult(String tenderId, String supplierId, List<BidPriceResult> bidPriceResultList);
	
	
	 /**
	 * @Description: 保存供应商的报价信息
	 * @author mazc
	 * @param tenderId
	 * @param supplierOpeningResult
	 * @param businessBidStatus
	 * @param technologyBidStatus
	 * @param priceBidStatus 价格标解密
	 * @param bidOpeningStatus
	 * @param bidOpeningTime
	 * @param bidOpeningRecord
	 * @param newBidPriceList 新增的分项报价
	 * @param newBidPriceItemList 新增的分项报价明细
	 * @param bidPriceResultList 分项报价列表
	 * @param bidPriceItemResultList 分项报价明细列表
	 * @return 
	 */
	SupplierOpeningResult saveSupplierBidPriceResult(String tenderId, SupplierOpeningResult supplierOpeningResult, String businessBidStatus, String technologyBidStatus, String priceBidStatus, String bidOpeningStatus, Date bidOpeningTime, String bidOpeningRecord, List<BidPrice> newBidPriceList,  List<BidPriceResult> bidPriceResultList);
	
	
	 /**
	 * @Description: 补录开标结果s
	 * @author mazc
	 * @param supplierOpeningResult
	 * @param bidPriceResultList
	 * @param bidPriceItemResultList
	 * @param allBidStatus
	 * @return 
	 */
	boolean saveSupplierInputBidPriceResult(String tenderId, String projectId, SupplierOpeningResult supplierOpeningResult, List<BidPriceResult> bidPriceResultList, boolean allBidStatus);
	
	/**
	 * @Description: 保存供应商多轮报价
	 * @author guoyr
	 * @param tenderId
	 * @param supplierId
	 * @param bidPriceResultList
	 * @return 
	 */
	boolean saveSupplierReBidPriceResult(String tenderId, String supplierId,String roundId, List<BidPriceResult> bidPriceResultList);

	 /**
	 * @Description: 补录评标价
	 * @author mazc
	 * @param supplierOpeningResult
	 * @param bidPriceResultList
	 * @return 
	 */
	boolean saveSupplierInputEvaluationPriceResult(SupplierOpeningResult supplierOpeningResult, List<BidPriceResult> bidPriceResultList);
	
	/**
	 * @Description: 保存汇率转换后的评标价
	 * @author mazc
	 * @param tenderProjectEntity
	 * @param bidPriceResultList
	 * @return 
	 */
	boolean saveExchangeRateEvaluation(TenderProject tenderProjectEntity, List<BidPriceResult> bidPriceResultList);
	
	
	 /**
	 * @Description: 
	 * @author mazc
	 * @param tenderId
	 * @param bidPrice
	 * @return 
	 */
	List<BidPriceResult> getTenderIdAndBidPrice(String tenderId, String bidPrice);
	
	
	boolean hasBidPriceResult(String tenderId);

	boolean updateCurrencyCodeAndPriceUnitAndEvaluationPrice(String tenderId,String bidPriceId,String supplierId,String priceUnit,String currencyCode,String bidPrice, String evaluationPrice,String evaluationPriceOriginal);

	/**
	 * @Description: 更新投标报价
	 * @author chenxw
	 * @param bidPriceResult
	 * @return
	 */
	boolean updateBidPriceResultOrSum(BidPriceResult bidPriceResult);

	 /**
	 * @Description: 
	 * @author mazc
	 * @param tenderId
	 * @return 
	 */
	boolean updateSupplierIdNotRecordedBidPrice(String tenderId, String roundId );
	
}


