package com.gx.obe.server.management.evaluation.service.impl;

import java.util.ArrayList;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.gx.obe.server.common.entity.ReflecUtils;
import com.gx.obe.server.common.utils.IDUtils;
import com.gx.obe.server.common.utils.StringUtils;
import com.gx.obe.server.management.evaluation.dao.BidPriceItemResultMapper;
import com.gx.obe.server.management.evaluation.entity.BidPriceItemResult;
import com.gx.obe.server.management.evaluation.service.BidPriceItemResultService;

/** 
 * @Description: 
 * @author mazc 
 */
@Service
@Transactional
public class BidPriceItemResultServiceImpl extends ServiceImpl<BidPriceItemResultMapper, BidPriceItemResult> implements BidPriceItemResultService {

	@Autowired
	private BidPriceItemResultMapper bidPriceItemResultMapper;

	/** 
	 * @Description: 获得分项明细结果
	 * @author mazc
	 * @param supplierId
	 * @param bidPriceId
	 * @return 
	 */
	@Override
	public List<BidPriceItemResult> getBidPriceItemResultList(String supplierId, String bidPriceId) {
		return bidPriceItemResultMapper.getBidPriceItemResultList(supplierId, bidPriceId);
	}

	@Override
	public List<BidPriceItemResult> conditionalQueryList(String tenderId, String bidPriceId, String supplierId) {
		QueryWrapper<BidPriceItemResult> queryWrapper = new QueryWrapper<BidPriceItemResult>();
		queryWrapper.lambda().eq(BidPriceItemResult::getTenderId, tenderId)
		.eq(BidPriceItemResult::getBidPriceId, bidPriceId)
		.eq(BidPriceItemResult::getSupplierId, supplierId);
		return bidPriceItemResultMapper.selectList(queryWrapper);
	}

	@Override
	public int updateItemContent(String tenderId, String bidPriceId, String bidPriceItemId, String supplierId,
			String itemUnit, String itemContent, String itemValue) {
		return bidPriceItemResultMapper.updateItemContent(tenderId, bidPriceId, bidPriceItemId, supplierId, itemUnit, itemContent, itemValue);
	}

	@Override
	@Transactional(rollbackFor = Exception.class)
	public Integer batchSaveOrUpdate(List<BidPriceItemResult> bidPriceItemResultList, String[] attributes) {
		int count = 0;
		List<BidPriceItemResult>  insertBidPriceItemResultList = new ArrayList<BidPriceItemResult>();
		List<BidPriceItemResult>  updateBidPriceItemResultList = new ArrayList<BidPriceItemResult>();	
		List<BidPriceItemResult>  newBidPriceItemResultList;
		if(null != attributes && attributes.length > 0){
			newBidPriceItemResultList  = new ArrayList<BidPriceItemResult>();
			for(BidPriceItemResult evaluationFactor : bidPriceItemResultList){
				BidPriceItemResult ruleEntity = ReflecUtils.reflex(evaluationFactor, attributes,BidPriceItemResult.class);
				if(null != ruleEntity){
					newBidPriceItemResultList.add(ruleEntity);
				}
			}
 		}else{
 			newBidPriceItemResultList = bidPriceItemResultList;
		}	
		
		for(BidPriceItemResult bidPriceitemResult : newBidPriceItemResultList) {
			if(StringUtils.isEmpty(bidPriceitemResult.getId())) {
				bidPriceitemResult.setId(IDUtils.getId());
				insertBidPriceItemResultList.add(bidPriceitemResult);
			}else {
				updateBidPriceItemResultList.add(bidPriceitemResult);
			}
		}
		if(insertBidPriceItemResultList.size() > 0) {
			count += bidPriceItemResultMapper.insertByBatch(insertBidPriceItemResultList);			
		}
		if(updateBidPriceItemResultList.size() > 0) {
			count += bidPriceItemResultMapper.updateObeBidPriceItemResultBatchList(updateBidPriceItemResultList);			
		}
		return count;
	
	}

}
