package com.gx.obe.server.management.evaluation.service.impl;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.gx.obe.server.management.evaluation.dao.BidPriceItemMapper;
import com.gx.obe.server.management.evaluation.dao.BidPriceItemResultMapper;
import com.gx.obe.server.management.evaluation.entity.BidPriceItem;
import com.gx.obe.server.management.evaluation.service.BidPriceItemService;

/** 
 * @Description: 
 * @author mazc 
 */
@Service
public class BidPriceItemServiceImpl extends ServiceImpl<BidPriceItemMapper, BidPriceItem> implements BidPriceItemService {

	@Autowired
	private BidPriceItemMapper bidPriceItemMapper;
	
	@Autowired
	private BidPriceItemResultMapper bidPriceItemResultMapper;

 
 	
	/**
	 * @Description: 批量更新
	 * @author mazc
	 * @param ObeBidPriceItemList 
	 */
	 @Override
	 @Transactional(rollbackFor = Exception.class)
	public Integer updateByBatch (List<BidPriceItem> ObeBidPriceItemList){
		return bidPriceItemMapper.updateObeBidPriceItemBatchList(ObeBidPriceItemList);
	}
	
	/**
	 * @Description: 批量插入
	 * @author mazc
	 * @param ObeBidPriceItemList 
	 */
	 @Override
	 @Transactional(rollbackFor = Exception.class)
	public Integer insertByBatch (List<BidPriceItem> ObeBidPriceItemList){
		return bidPriceItemMapper.insertByBatch(ObeBidPriceItemList);
	}
	

	
	
	@Override
	@Transactional(rollbackFor = Exception.class)
	public boolean deleteBidPriceItem(String bidPriceItemId) {
		int count = 0;
		count += bidPriceItemMapper.deletByPriceTimeId(bidPriceItemId);
		count += bidPriceItemResultMapper.deletByPriceTimeId(bidPriceItemId);
		return count > 0;
	}

	/** 
	 * @Description: 交换两个分项报价明细的排序
	 * @author mazc
	 * @param sourceSortNo
	 * @param sourceid
	 * @param targetSortNo
	 * @param targetId
	 * @return 
	 */
	@Override
	@Transactional(rollbackFor = Exception.class)
	public boolean changeBidPriceItemSortNo(String sourceSortNo, String sourceid, String targetSortNo,String targetId) {
		int count = 0;
		UpdateWrapper<BidPriceItem> sourceUpdateWrapper = new UpdateWrapper<>();
		sourceUpdateWrapper.set(BidPriceItem.SORT_NO,targetSortNo);
		sourceUpdateWrapper.lambda().eq(BidPriceItem::getId, sourceid);
		count += bidPriceItemMapper.update(null, sourceUpdateWrapper);
		
		UpdateWrapper<BidPriceItem> targetUpdateWrapper = new UpdateWrapper<>();
		targetUpdateWrapper.set(BidPriceItem.SORT_NO,sourceSortNo);
		targetUpdateWrapper.lambda().eq(BidPriceItem::getId, targetId);
		count +=  bidPriceItemMapper.update(null, targetUpdateWrapper); 
		
		return count == 2;
	}

	@Override
	public List<BidPriceItem> getItemByBidPriceIdAndSortNoOrderBy(String bidPriceId) {
		QueryWrapper<BidPriceItem> queryWrapper = new QueryWrapper<>();
		queryWrapper.lambda().eq(BidPriceItem::getBidPriceId, bidPriceId).orderByAsc(BidPriceItem::getSortNo);
		
		return bidPriceItemMapper.selectList(queryWrapper);
	}

	@Override
	public int getMaxDictValueSortNo(String bidPriceId) {
		Integer maxSortNo = bidPriceItemMapper.getMaxDictValueSortNo(bidPriceId);
		return null != maxSortNo ? maxSortNo.intValue() : 0;
	}

}
