package com.gx.obe.server.management.evaluation.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.gx.obe.server.common.enumeration.BidPriceEnum;
import com.gx.obe.server.common.utils.IDUtils;
import com.gx.obe.server.common.utils.StringUtils;
import com.gx.obe.server.common.vo.Result;
import com.gx.obe.server.management.evaluation.dao.*;
import com.gx.obe.server.management.evaluation.entity.BidPrice;
import com.gx.obe.server.management.evaluation.service.BidPriceService;
import com.gx.obe.server.management.supplier.dao.SupplierOpeningResultMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;

/** 
 * @Description: 
 * @author mazc 
 */
@Service
@Transactional
public class BidPriceServiceImpl extends ServiceImpl<BidPriceMapper, BidPrice> implements BidPriceService {

	@Autowired
	private BidPriceMapper bidPriceMapper;
	
	@Autowired
	private BidPriceItemMapper BidPriceItemMapper;
 
	@Autowired
	private BidPriceItemResultMapper bidPriceItemResultMapper;
	
	@Autowired
	private BidPriceResultMapper bidPriceResultMapper;
	
	@Autowired
	private BidPriceHisResultMapper bidPriceHisResultMapper;
	
	@Autowired
	private SupplierOpeningResultMapper supplierOpeningResultMapper;


	/**
	 * @Description: 指定字段修改
	 * @author chenxw
	 * @param ObeBidPrice
	 * @param attributes
	 */
	@Override
	public boolean updateAssignProperty(BidPrice ObeBidPrice, String[] attributes) {
		return bidPriceMapper.updateAssignProperty(ObeBidPrice, attributes) > 0;
	}

	/**
	 * @Description: 批量指定字段修改
	 * @author chenxw
	 * @param ObeBidPriceList
	 * @param attributes
	 */
	@Override
	public Integer batchUpdateProperty(List<BidPrice> ObeBidPriceList, String[] attributes) {
		return bidPriceMapper.batchUpdateProperty(ObeBidPriceList, attributes);
	}

	/**
	 * @Description: 批量添加或跟新
	 * @author chenxw
	 * @param ObeBidPriceList
	 * @param attributes
	 * @return
	 */
	@Override
	@Transactional(rollbackFor = Exception.class)
	public Integer batchSaveOrUpdate(List<BidPrice> ObeBidPriceList, String[] attributes) {
		List<BidPrice> insertObeBidPriceList = new ArrayList<>();
		List<BidPrice> updateObeBidPriceList = new ArrayList<>();
		for (BidPrice ObeBidPrice : ObeBidPriceList) {
			if (StringUtils.isEmpty(ObeBidPrice.getId())) {
				ObeBidPrice.setId(IDUtils.getId());
				insertObeBidPriceList.add(ObeBidPrice);
			} else {
				updateObeBidPriceList.add(ObeBidPrice);
			}
		}
		int count = 0;
		if (insertObeBidPriceList.size() > 0) {
			count += bidPriceMapper.insertByBatch(insertObeBidPriceList);
		}
		if (updateObeBidPriceList.size() > 0) {
			count += bidPriceMapper.batchUpdateProperty(updateObeBidPriceList, attributes);
		}
		return count;
	}

	/** 
	 * @Description: 获得可做为价格指标的分项报价
	 * @author mazc
	 * @param tenderId
	 * @return 
	 */
	@Override
	public List<BidPrice> getBidPriceFactorList(String tenderId) {
		return bidPriceMapper.selectBidPriceFactorList(tenderId);
	}

	/** 
	 * @Description:  批量保存已解析项目的开标一览表
	 * @author guoyr
	 * @param bidPriceList
	 * @return 
	 */
	@Override
	@Transactional(rollbackFor = Exception.class)
	public boolean batchSaveResolverBidPriceTable(List<BidPrice> bidPriceList) {
		boolean flag = false;
		if(null != bidPriceList && bidPriceList.size() > 0) {
			Integer size = bidPriceMapper.insertByBatch(bidPriceList);
			if(size > 0) {
				flag = true ;
			}
		}
		return flag;
	}

	/** 
	 * @Description: 删除分项报价
	 * @author mazc
	 * @param tenderId
	 * @param bidPrice
	 * @return 
	 */
	@Override
	@Transactional(rollbackFor = Exception.class)
	public boolean deleteBidPrice(String tenderId, String bidPriceId ,String joinTotal) {
		int count = 0;
		count += bidPriceItemResultMapper.deleteByPriceId(bidPriceId);
		count += BidPriceItemMapper.deleteByPriceId(bidPriceId);
		count += bidPriceResultMapper.deleteByPriceId(bidPriceId);
		count += bidPriceHisResultMapper.deleteByPriceId(bidPriceId);
		count += bidPriceMapper.deleteByPriceId(bidPriceId);
		if (null != joinTotal && joinTotal.equals(BidPriceEnum.JOIN_TOTAL)) {
			supplierOpeningResultMapper.updateSupplierSumPrice(tenderId, null, true, true);
		}
		return count > 0;
	}

	/** 
	 * @Description: 删除项目的分项报价
	 * @author mazc
	 * @param tenderId
	 * @return 
	 */
	@Override
	public boolean deleteProjectBidPrice(String tenderId) {
		return bidPriceMapper.deleteProjectBidPrice(tenderId) > 0;
	}

	/** 
	 * @Description: 交换两个分项报价的排序
	 * @author mazc
	 * @param sourceSortNo
	 * @param sourceid
	 * @param targetSortNo
	 * @param targetId
	 * @return 
	 */
	@Override
	public boolean changeBidPriceSortNo(String sourceSortNo, String sourceid, String targetSortNo,
			String targetId) {
		int count = 0;
		UpdateWrapper<BidPrice> sourceUpdateWrapper = new UpdateWrapper<>();
		sourceUpdateWrapper.set(BidPrice.SORT_NO,targetSortNo);
		sourceUpdateWrapper.lambda().eq(BidPrice::getId, sourceid);
		count += bidPriceMapper.update(null, sourceUpdateWrapper);
		
		UpdateWrapper<BidPrice> targetUpdateWrapper = new UpdateWrapper<>();
		targetUpdateWrapper.set(BidPrice.SORT_NO,sourceSortNo);
		targetUpdateWrapper.lambda().eq(BidPrice::getId, targetId);
		count +=  bidPriceMapper.update(null, targetUpdateWrapper); 
		
		return count == 2;
 	}

	@Override
	public BidPrice getBidPriceByJoinTotal(String tenderId) {
		QueryWrapper<BidPrice> queryWrapper = new QueryWrapper<BidPrice>();
		queryWrapper.lambda().eq(BidPrice::getTenderId, tenderId).eq(BidPrice::getJoinTotal, BidPriceEnum.JOIN_TOTAL).orderByAsc(BidPrice::getSortNo);
		return bidPriceMapper.selectList(queryWrapper).get(0);
	}

	/** 
	 * @Description: 获取项目中开标一览表的数量
	 * @author guoyr
	 * @param tenderId
	 * @return 
	 */
	@Override
	public int getBidPriceTableCount(String tenderId) {
		QueryWrapper<BidPrice> queryWrapper = new QueryWrapper<BidPrice>();
		queryWrapper.select(BidPrice.BID_PRICE_ID).lambda().eq(BidPrice::getTenderId, tenderId);
		return	bidPriceMapper.selectList(queryWrapper).size();
	}
	
	/**
	 * @Description: 保存报价项
	 * @author chenxw
	 * @param bidPrice
	 * @return
	 */
	@Override
	public Result<Integer> saveBidPrice(BidPrice bidPrice) {
//		if (BidPriceEnum.IS_PRICE_ITEM.equals(bidPrice.getBidPriceType()) && BidPriceEnum.JOIN_TOTAL.equals(bidPrice.getJoinTotal())) {
//			QueryWrapper<BidPrice> queryWrapper = new QueryWrapper<>();
//			queryWrapper.lambda()
//					.eq(BidPrice::getTenderId, bidPrice.getTenderId())
//					.eq(BidPrice::getBidPriceType, BidPriceEnum.IS_PRICE_ITEM)
//					.eq(BidPrice::getJoinTotal, BidPriceEnum.JOIN_TOTAL)
//					.ne(BidPrice::getPriceType, bidPrice.getPriceType());
//			if (bidPriceMapper.selectCount(queryWrapper) > 0) {
//				return new Result<>(1, "计入总价报价项报价类型不一致", Result.FAIL);
//			}
//		}
		bidPrice.setId(IDUtils.getId());
		if (save(bidPrice)) {
			return new Result<>(Result.SUCCESS);
		}
		return new Result<>(0, "保存失败", Result.FAIL);
	}
	/**
	 * @Description: 保存报价项
	 * @author chenxw
	 * @param bidPrice
	 * @return
	 */
	@Override
	public boolean updateBidPrice(BidPrice ObeBidPrice, String[] attributes) {
		if(bidPriceMapper.updateAssignProperty(ObeBidPrice, attributes) > 0) {
			if (BidPriceEnum.JOIN_TOTAL.equals(ObeBidPrice.getJoinTotal()) || BidPriceEnum.IS_PRICE_ITEM.equals(ObeBidPrice.getBidPriceType())) {
				supplierOpeningResultMapper.updateSupplierSumPrice(ObeBidPrice.getTenderId(), null, true, true);
			}
			return true;
		}
		return false;
	}

	@Override
	public int getMaxBidPriceSortNo(String tenderId) {
 		Integer maxSortNo = bidPriceMapper.getMaxBidPriceSortNo(tenderId);
 		return null != maxSortNo ? maxSortNo : 0;
	}
}
