package com.gx.obe.server.management.evaluation.service.impl;

import java.math.BigDecimal;
import java.util.List;
import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.gx.obe.server.common.utils.CollectionUtils;
import com.gx.obe.server.common.utils.ObjectUtils;
import com.gx.obe.server.management.evaluation.dao.EvaluationFactorResultMapper;
import com.gx.obe.server.management.evaluation.entity.EvaluationFactorResult;
import com.gx.obe.server.management.evaluation.service.EvaluationFactorResultService;

/** 
 * @Description: 
 * @author mazc 
 */
@Service
@Transactional
public class EvaluationFactorResultServiceImpl extends ServiceImpl<EvaluationFactorResultMapper, EvaluationFactorResult> implements EvaluationFactorResultService {

	@Autowired
	private EvaluationFactorResultMapper evaluationFactorResultMapper;
	
	/**
	 * @Description: 批量更新
	 * @author mazc
	 * @param ObeEvaluationFactorResultList 
	 */
	 @Override
	 @Transactional(rollbackFor = Exception.class)
	public Integer updateByBatch (List<EvaluationFactorResult> ObeEvaluationFactorResultList){
		return evaluationFactorResultMapper.updateBatchList(ObeEvaluationFactorResultList);
	}
	
	/**
	 * @Description: 批量插入
	 * @author mazc
	 * @param ObeEvaluationFactorResultList 
	 */
	 @Override
	 @Transactional(rollbackFor = Exception.class)
	public Integer insertByBatch (List<EvaluationFactorResult> ObeEvaluationFactorResultList){
	     if (CollectionUtils.isNull(ObeEvaluationFactorResultList)) {
            return 0;
        }
		return evaluationFactorResultMapper.insertByBatch(ObeEvaluationFactorResultList);
	}

	/** 
	 * @Description:  删除所有专家为投标人指标的的评标结果
	 * @author mazc
	 * @param tenderId
	 * @return 
	 */
	@Override
	public int deleteStepByTenderId(String tenderId) {
		QueryWrapper<EvaluationFactorResult> queryWrapper = new QueryWrapper<EvaluationFactorResult>();
		queryWrapper.lambda().eq(EvaluationFactorResult::getTenderId, tenderId);
		return evaluationFactorResultMapper.delete(queryWrapper);
	}

	/** 
	 * @Description: 获得评标专家的评标结果列表
	 * @author mazc
	 * @param tenderId
	 * @param stepId
	 * @param userId
	 * @param supplierId
	 * @return 
	 */
	@Override
	public List<EvaluationFactorResult> getExpertEvaluationFactorResult(String tenderId, String stepId, String userId, String supplierId) {
		return evaluationFactorResultMapper.getExpertEvaluationFactorResult(tenderId, stepId, userId, supplierId);
	}

	/** 
	 * @Description: 获取同一指标评标结果不同的列表
	 * @author mazc
	 * @param tenderId
	 * @param stepId
	 * @param isScoreFactor
	 * @return 
	 */
	@Override
	public List<Map<String, Object>> getSupplierDistinctFactorResultList(String tenderId, String stepId,
			boolean isScoreFactor) {
		return evaluationFactorResultMapper.getSupplierDistinctFactorResultList(tenderId, stepId, isScoreFactor);
	}

    /** 
     * @Description: 保存按供应商打分结果
     * @author chenxw
     * @param projectId
     * @param groupCode
     * @param userId
     * @param supplierEvaluationFactorResultList 
     */
    @Override
    public void saveEvaluationResultBySupplier(String projectId, String groupCode, String userId, List<Map<String, Object>> supplierEvaluationFactorResultList) {
        if(null != groupCode && groupCode.length() == 0){
            groupCode = null;
        }
        if (CollectionUtils.isNull(supplierEvaluationFactorResultList))
        	return;
        for(Map<String, Object> factorResultMap : supplierEvaluationFactorResultList){
            saveEvaluationResultBySupplier(projectId, groupCode, userId, (String)factorResultMap.get("SUPPLIER_NAME"), (String)factorResultMap.get("FACTOR_NAME"), ObjectUtils.getObjBigDecimal(factorResultMap.get("EVALUATION_SCORE")));
        }
    }

    /**
     * @Description: 保存按供应商打分结果
     * @author chenxw
     * @param projectId
     * @param groupCode
     * @param userId
     * @param supplierName
     * @param factorName
     * @param evalScore 
     */
    private void saveEvaluationResultBySupplier(String projectId, String groupCode, String userId, String supplierName, String factorName, BigDecimal evalScore) {
        if (evaluationFactorResultMapper.updateSupplierEvalFactorScore(evalScore, projectId, groupCode, userId, supplierName) == 0) {
            evaluationFactorResultMapper.insertSupplierEvaluationFactorResult(projectId, groupCode, userId, supplierName, factorName, evalScore);
        }
    }

    /** 
     * @Description: 获得指标的平均分
     * @author chenxw
     * @param tenderId
     * @param stepId
     * @param supplierId
     * @param removeMaxMin
     * @return 
     */
    @Override
    public List<Map<String, Object>> getEvaluationFactorAvgScoreList(String tenderId, String stepId, String supplierId, boolean removeMaxMin) {
        return evaluationFactorResultMapper.getEvaluationFactorAvgScoreList(tenderId, stepId, supplierId, removeMaxMin);
    }

    /** 
     * @Description: 根据指标Id获取当前专家对应的指标评审结果
     * @author chenxw
     * @param tenderId
     * @param stepId
     * @param factorId
     * @param userId
     * @return 
     */
    @Override
    public List<EvaluationFactorResult> getEvaluationFactorResultList(String tenderId, String stepId, String factorId, String userId) {
        return evaluationFactorResultMapper.getEvaluationFactorResultList(tenderId, stepId, factorId, userId);
    }

    /** 
     * @Description: 获得评标专家的评标结果列表
     * @author chenxw
     * @param tenderId
     * @param factorType
     * @return 
     */
    @Override
    public List<EvaluationFactorResult> getResultListByFactorType(String tenderId, String factorType) {
        return evaluationFactorResultMapper.getResultListByFactorType(tenderId, factorType);
    }

    /** 
     * @Description: 获取指定指标不同评审结果的个数
     * @author chenxw
     * @param tenderId
     * @param stepId
     * @param factorId
     * @param userId
     * @param result
     * @param score
     * @return 
     */
    @Override
    public int getSupplierDistinctFactorCount(String tenderId, String stepId, String factorId, String userId, String result, BigDecimal score) {
        return evaluationFactorResultMapper.getSupplierDistinctFactorCount(tenderId, stepId, factorId, userId, result, score);
    }

    /** 
     * @Description: 获得按供就商打分结果列表
     * @author chenxw
     * @param projectId
     * @param groupCode
     * @param userId
     * @param supplierName
     * @return 
     */
    @Override
    public List<Map<String, Object>> getSupplierEvaluationFactorResult(String projectId, String groupCode, String userId, String supplierName) {
        return evaluationFactorResultMapper.getSupplierEvaluationFactorResult(projectId, groupCode, userId, supplierName);
    }

    /** 
     * @Description: 获取得分列表
     * @author chenxw
     * @param tenderId
     * @param stepId
     * @param supplierId
     * @param factorId
     * @return 
     */
    @Override
    public List<Object> getEvaluationScoreList(String tenderId, String stepId, String supplierId, String factorId) {
        QueryWrapper<EvaluationFactorResult> wrapper = new QueryWrapper<EvaluationFactorResult>();
        wrapper.lambda().select(EvaluationFactorResult::getEvaluationScore)
                        .eq(EvaluationFactorResult::getTenderId, tenderId)
                        .eq(EvaluationFactorResult::getStepId, stepId)
                        .eq(EvaluationFactorResult::getSupplierId, supplierId)
                        .eq(EvaluationFactorResult::getFactorId, factorId);
        return listObjs(wrapper);
    }
    
}
