package com.gx.obe.server.management.evaluation.service.impl;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.gx.obe.server.common.utils.CollectionUtils;
import com.gx.obe.server.common.utils.StringUtils;
import com.gx.obe.server.management.evaluation.dao.EvaluationGroupMapper;
import com.gx.obe.server.management.evaluation.entity.EvaluationGroup;
import com.gx.obe.server.management.evaluation.service.EvaluationGroupService;

/**
 * @Description:
 * @author mazc
 */
@Service
@Transactional
public class EvaluationGroupServiceImpl extends ServiceImpl<EvaluationGroupMapper, EvaluationGroup> implements EvaluationGroupService {
    
    @Autowired
    private EvaluationGroupMapper evaluationGroupMapper;
    
    /**
     * @Description: 批量更新
     * @author mazc
     * @param ObeEvaluationGroupList
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Integer updateByBatch(List<EvaluationGroup> ObeEvaluationGroupList) {
        // TODO Auto-generated method stub
        return evaluationGroupMapper.updateBatchList(ObeEvaluationGroupList);
    }
    
    /**
     * @Description: 批量插入
     * @author mazc
     * @param ObeEvaluationGroupList
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Integer insertByBatch(List<EvaluationGroup> ObeEvaluationGroupList) {
        if (CollectionUtils.isNull(ObeEvaluationGroupList)) {
            return 0;
        }
        return evaluationGroupMapper.insertByBatch(ObeEvaluationGroupList);
    }
    
    /**
     * @Description: 根据用户id获取评审分组
     * @author chenxw
     * @param tenderId
     * @return
     */
    @Override
    public List<EvaluationGroup> getEvaluationGroupByUserId(String tenderId, String userId, String stepId) {
        QueryWrapper<EvaluationGroup> wrapper = new QueryWrapper<EvaluationGroup>();
        wrapper.lambda().select(
                    EvaluationGroup::getId, 
                    EvaluationGroup::getTenderId, 
                    EvaluationGroup::getSupplierId, 
                    EvaluationGroup::getUserId, 
                    EvaluationGroup::getEvalBidType, 
                    EvaluationGroup::getStepId
                )
                        .eq(EvaluationGroup::getTenderId, tenderId);
        if (StringUtils.isNotEmpty(userId)) {
            wrapper.lambda().eq(EvaluationGroup::getUserId, userId);
        }
        if (StringUtils.isNotEmpty(stepId)) {
            wrapper.lambda().eq(EvaluationGroup::getStepId, stepId);
        }
        return list(wrapper);
    }
    
    /**
     * @Description: 通过tenderid 与 stepId删除
     * @author mazc
     * @param tenderId
     * @param stepId
     * @return
     */
    @Override
    public int deleteByTenderIdAndStepId(String tenderId, String stepId) {
        QueryWrapper<EvaluationGroup> wrapper = new QueryWrapper<EvaluationGroup>();
        wrapper.lambda().eq(EvaluationGroup::getTenderId, tenderId);
        wrapper.lambda().eq(EvaluationGroup::getStepId, stepId);
        return evaluationGroupMapper.delete(wrapper);
    }
    
}
