package com.gx.obe.server.management.expert.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.gx.obe.server.common.base.BaseController;
import com.gx.obe.server.common.log.OperationType;
import com.gx.obe.server.common.log.SysLogAnnotation;
import com.gx.obe.server.common.type.ExpertType;
import com.gx.obe.server.common.vo.RequestBodyEntity;
import com.gx.obe.server.management.evaluation.entity.EvaluationStep;
import com.gx.obe.server.management.expert.entity.ExpertEntity;
import com.gx.obe.server.management.expert.service.ExpertService;
import com.gx.obe.server.management.user.entity.AuthUserEntity;
import com.gx.obe.server.management.user.entity.AuthUserRoleEntity;
import com.gx.obe.server.management.user.service.AuthUserService;
import com.gx.obe.server.management.vo.SaveTempExpertVo;
import com.gx.obe.server.management.vo.UpdateAssignPropertyVo;
import com.gx.obe.server.management.vo.UpdateExpertInfoVo;

import io.swagger.annotations.Api;
import lombok.AllArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;
/** 
 * @Description: 
 * @author mazc 
 */
@Api(tags = "评标专家管理")
@RestController
@AllArgsConstructor
@RequestMapping("/expert")
public class ExpertController extends BaseController<ExpertService,ExpertEntity>{

	@Autowired
	ExpertService expertService;
	@Autowired
	AuthUserService authUserService;

	/**
	 * @Description: 插入
	 * @author chenxw
	 * @param expertEntity
	 * @return 
	 */
	@PostMapping("/insertReturn")
	@SysLogAnnotation(detail="插入专家",level = 0,operationType = OperationType.INSERT)
	public ExpertEntity insertReturn(@RequestBody ExpertEntity expertEntity) {
	    return expertService.insert(expertEntity);
	}
	
	@GetMapping("/findEvalBidTypeByTenderId")
	public List<ExpertEntity> findEvalBidTypeAndTenderIdList(String tenderId){
		QueryWrapper<ExpertEntity> wrapper = new QueryWrapper<>();
		wrapper.select(ExpertEntity.EXPERT_ID,ExpertEntity.EVAL_BID_TYPE).lambda().eq(ExpertEntity::getTenderId,tenderId)
																			   .orderByAsc(ExpertEntity::getExpertType);
		return expertService.list(wrapper);
	}
	
	
	/**
	 * @Description: 通过tenderid 查询所需要的字段
	 * @author mazc
	 * @param tenderId
	 * @param strCoulmns
	 * @return 
	 */
	@GetMapping("/finCoulmnsByTenderId")
	public List<ExpertEntity> finCoulmnsByTenderId(String tenderId , String [] strCoulmns){
		QueryWrapper<ExpertEntity> wrapper = new QueryWrapper<>();
		wrapper.select(strCoulmns).lambda().eq(ExpertEntity::getTenderId,tenderId);
		return expertService.list(wrapper);
	}
	
	/**
	 * @Description: 获得是否存在评标专家组长
	 * @author mazc
	 * @param tenderId
	 * @return 
	 */
	@GetMapping("/isExistExpertLeader")
	public boolean isExistExpertLeader(String tenderId){
		QueryWrapper<ExpertEntity> wrapper = new QueryWrapper<>();
		wrapper.select(ExpertEntity.EXPERT_ID).lambda().eq(ExpertEntity::getTenderId, tenderId)
																  .eq(ExpertEntity::getIsLeader, ExpertType.LEADER_YES);
		return  expertService.list(wrapper).size() > 0;
	}
	
	/**
	 * @Description: 获得指定项目的评审成员
	 * @author mazc
	 * @param tenderId
	 * @param userId
	 * @return 
	 */
	@GetMapping("/findExpert")
	public ExpertEntity findExpert(String tenderId, String userId){
		return expertService.findExpertBatch(tenderId, userId);
	}
	
	/**
	 * @Description: 修改签到状态
	 * @author mazc
	 * @param signType
	 * @param tenderId
	 * @return 
	 */
	@GetMapping("/updateSignType")
	@SysLogAnnotation(detail="修改签到状态",level = 0,operationType = OperationType.UPDATE)
	public int updateSignType(String signType , String tenderId ) {
		return expertService.updateSignType(signType, tenderId);
	}
	
	/**
	 * @Description: 获得评标专家列表
	 * @author mazc
	 * @param tenderId
	 * @param signType
	 * @return 
	 */
	@GetMapping("/findExpertByTenderIdAndSignTypeBatch")
	public 	List<ExpertEntity> findExpertByTenderIdBatch(String tenderId, String signType){
		return expertService.findExpertByTenderIdAndSignTypeBatch(tenderId, signType);
 	}
	
	
	/** 
	 * @Description: 通过项目id 获得的评标专家
	 * @author mazc
	 * @param tenderId
	 * @return 
	 */
	@GetMapping("/findExpertByTenderId")
	public List<ExpertEntity> findExpertByTenderId(String tenderId) {
		return expertService.findExpertByTenderId(tenderId);
	}
	
	/**
	 * @Description: 获得评标步骤的评标专家
	 * @author mazc
	 * @param requestBodyEntity
	 * @return
	 */
	@PostMapping("/findEvaluationStepUserList")
	public List<AuthUserEntity> findEvaluationStepUserList(@RequestBody RequestBodyEntity<EvaluationStep> requestBodyEntity){
		String tenderId = (String) requestBodyEntity.getParam("tenderId");
		EvaluationStep evaluationStep = requestBodyEntity.getEntity();
		return authUserService.findEvaluationStepUserList(tenderId, evaluationStep);
	}
	
	/**
	 * @Description: 获得评标专家组长
	 * @author mazc
	 * @param tenderId
	 * @return 
	 */
	@GetMapping("/findExpertLeader")
	public ExpertEntity findExpertLeader(String tenderId) {
		return expertService.findExpertLeader(tenderId);
	}
	
	
	/**
	 * @Description: 
	 * @author mazc
	 * @param requestBodyEntity
	 * @return
	 */
	@PostMapping("/findEvaluationStepExpertList")
	public List<ExpertEntity> findEvaluationStepExpertList(@RequestBody RequestBodyEntity<EvaluationStep> requestBodyEntity){
		String tenderId = (String) requestBodyEntity.getParam("tenderId");
		EvaluationStep evaluationStep = requestBodyEntity.getEntity();
		return expertService.findEvaluationStepExpertList(tenderId, evaluationStep);
	}
	
	
	/**
	 * @Description: 获得评标步骤特定的专家
	 * @author mazc
	 * @param requestBodyEntity
	 * @return
	 */
	@PostMapping("/findEvaluationStepExpert")
	public ExpertEntity findEvaluationStepExpert(@RequestBody RequestBodyEntity<EvaluationStep> requestBodyEntity) {
		String tenderId = (String) requestBodyEntity.getParam("tenderId");
		EvaluationStep evaluationStep = requestBodyEntity.getEntity();
		return  expertService.findEvaluationStepExpert(tenderId, evaluationStep);
	}
	
	
	/** 
	 * @Description: 根据项目父级ID查询当前项目（包含子项目）下所有的评审专家
	 * @author mazc
	 * @param projectId
	 * @return 
	 */
	@GetMapping("/findExpertListByTenderId")
	public List<ExpertEntity> findExpertListByTenderId(String projectId) {
		return expertService.findExpertListByTenderId(projectId);
	}
	
	/**
	 * @Description: 获得参与当前评标步骤的专家数
	 * @author mazc
	 * @param requestBodyEntity
	 * @return 
	 */
	@PostMapping("/findEvaluationStepExpertCount")
	public int findEvaluationStepExpertCount(@RequestBody RequestBodyEntity<EvaluationStep> requestBodyEntity) {
		String tenderId = (String) requestBodyEntity.getParam("tenderId");
		EvaluationStep evaluationStep = requestBodyEntity.getEntity();
		return expertService.findEvaluationStepExpertCount(tenderId, evaluationStep);
	}
	
	
	/**
	 * @Description: 获得评标专家列表
	 * @author mazc
	 * @param tenderId
	 * @param signType
	 * @return 
	 */
	@GetMapping("/findExpertList")
	public List<ExpertEntity> findExpertList(String tenderId, String signType) {
		return expertService.findExpertList(tenderId, signType);
	}
	
	
	/**
	 * @Description: 查询userId votesNum 、votes  
	 * @author mazc
	 * @param tenderId
	 * @return 
	 */
	@GetMapping("/findExperColumnstByTenderId")
	public List<ExpertEntity> findExperColumnstByTenderId(String tenderId){
		return expertService.findExperColumnstByTenderId(tenderId);
	}

	/**
	 * @Description: 指定字段修改
	 * @param vo
	 * @return
	 */
	@PostMapping("/updateAssignProperty")
	public boolean  updateAssignProperty(@RequestBody UpdateAssignPropertyVo<ExpertEntity> vo) {
		return expertService.updateAssignProperty(vo.getEntity(), vo.getAttributes());
	}
	
	/**
	 * @Description: 获得系统中所有的评标专家
	 * @author wangxiang
	 * @return 
	 */
	@GetMapping("/getSystemExpertList")
	public List<ExpertEntity> getSystemExpertList(String userType) {
		return expertService.getSystemExpertList(userType);
	}
	
	/**
	 * @Description: 获得指定评标步骤的自定义评标专家小组列表
	 * @author wangxiang
	 * @param tenderId 项目id
	 * @param stepId 步骤Id
	 * @return
	 */
	@GetMapping("/getCustomExpertList")
	public List<ExpertEntity> getCustomExpertList(String tenderId, String stepId){
		return expertService.getCustomExpertList(tenderId,stepId);
	}
	
	/**
	 * @Description: 获得最大的专家账号
	 * @author wangxiang
	 * @return
	 */
	@GetMapping("/getMaxExpertAccountName")
	public int getMaxExpertAccountName() {
		return expertService.getMaxExpertAccountName();
	}
	
	/**
	 * @Description: 为专家组长投票
	 * @author wangxiang
	 * @param myId 投票者id
	 * @param voterId 投票对象的id
	 * @return
	 */
	@GetMapping("/votesExpertLeader")
	public boolean votesExpertLeader(String myId, String voterId){
		return expertService.votesExpertLeader(myId,voterId);
	}
	
	/**
	 * @Description: 新增临时专家
	 * @author wangxiang
	 * @param saveTempExpertVo
	 */
	@PostMapping("/saveTemExpert")
	@SysLogAnnotation(detail="新增临时专家",level = 0,operationType = OperationType.INSERT)
	public boolean saveTemExpert(@RequestBody SaveTempExpertVo saveTempExpertVo){
		ExpertEntity expert = saveTempExpertVo.getExpert();
		List<AuthUserRoleEntity> userRoleList = saveTempExpertVo.getUserRoleList();
		AuthUserEntity user = saveTempExpertVo.getUser();
		return expertService.saveTemExpert(expert,userRoleList,user);
	}
	
	/**
	 * @Description: 删除所有评审专家
	 * @author wangxiang
	 * @param tenderId
	 * @return 
	 */
	@GetMapping("/deleteAllExperts")
	@SysLogAnnotation(detail="删除所有评审专家",level = 0,operationType = OperationType.DELETE)
	public boolean deleteAllExperts(String tenderId){
		return expertService.deleteAllExperts(tenderId);
	}
	
	/**
	 * @Description: 获取专家可分配的分组列表
	 * @author wangxiang
	 * @param tenderId
	 * @param expertId
	 * @return 
	 */
	@GetMapping("/getEvaGroupList")
	public List<Object> getEvaGroupList(String tenderId, String expertId){
		return expertService.getEvaGroupList(tenderId,expertId);
	}
	
	/**
	 * @Description: 获得评标单元指定的专家
	 * @author wangxiang
	 * @param tenderId
	 * @param evalGroup
	 * @return
	 */
	@GetMapping("/getExpertByevalGroup")
	public ExpertEntity getExpertByevalGroup(String tenderId, String evalGroup){
		return expertService.getExpertByevalGroup(tenderId,evalGroup);
	}
	
	/**
	 * @Description: 获取所有指标单元
	 * @author wangxiang
	 * @param tenderId
	 * @return
	 */
	@GetMapping("/getAllEvaGroupList")
	public List<Object> getAllEvaGroupList(String tenderId){
		return expertService.getAllEvaGroupList(tenderId);
	}
	
	/**
	 * @Description: 删除垃圾专家
	 * @author wangxiang
	 * @param tenderId 
	 */
	@GetMapping("/deleteDirtyExpert")
	@SysLogAnnotation(detail="删除垃圾专家",level = 0,operationType = OperationType.DELETE)
	public boolean deleteDirtyExpert(String tenderId){
		return expertService.deleteDirtyExpert(tenderId);
	}
	
	/**
	 * @Description: 通过项目ID和专家ID列表查询专家
	 * @author wangxiang
	 * @param requestBodyEntity
	 * @return
	 */
	@PostMapping("/getExpertListByIdList")
	public List<ExpertEntity> getExpertListByIdList(@RequestBody RequestBodyEntity<String> requestBodyEntity) {
		String tenderId = (String) requestBodyEntity.getParam("tenderId");
		List<String> objIdList = requestBodyEntity.getEntityList();
		QueryWrapper<ExpertEntity> wraaper = new QueryWrapper<>();
		wraaper.lambda().eq(ExpertEntity::getTenderId, tenderId)
						.in(ExpertEntity::getId, objIdList);
		
		return expertService.list(wraaper);
	}
	
	
	/**
	 * @Description: 获得评标专家列表
	 * @author mazc
	 * @param tenderId 项目id
	 * @param signType 签到类型
	 * @return
	 */
	@GetMapping("/getExpertList")
	public List<ExpertEntity> getExpertList(String tenderId, String signType){
		return expertService.getExpertList(tenderId, signType);
	}
	
	@GetMapping("/getEvaGroupListByTenderId")
	public List<Object> getEvaGroupList(String tenderId){
		return  expertService.getEvaGroupList(tenderId);
	}
	
	
	
	/**
	 * @Description: 批量插入
	 * @author mazc
	 * @param ObeExpertList 
	 */
	@PostMapping("/insertByBatch")
	public int insertByBatch(@RequestBody List<ExpertEntity> ObeExpertList) {
		return expertService.insertByBatch(ObeExpertList);
	}
 
	@GetMapping("/deleteExpert")
	@SysLogAnnotation(detail="删除专家",level = 0,operationType = OperationType.DELETE)
	public boolean deleteExpert(String tenderId, String userId){ 
		return expertService.deleteExpert(tenderId, userId);
	}
	
	@GetMapping("/getExpertListByTenderId")
	public List<ExpertEntity> getExpertListByTenderId(String tenderId){ 
		return expertService.getExpertListByTenderId(tenderId);
	}
	
	@GetMapping("/getIdListByTenderIdAndUserId")
	public boolean getIdListByTenderIdAndUserId( String tenderId,String userId){
		return expertService.getIdListByTenderIdAndUserId(tenderId, userId);
	}
	
	@GetMapping("/updateVotesAndVotesNumByTenderId")
	public boolean updateVotesAndVotesNumByTenderId(int votes ,int votesNum,String tenderId ){ 
		return expertService.updateVotesAndVotesNumByTenderId(votes, votesNum, tenderId);
	}
	
	@GetMapping("/getUserIdByTenderId")
	public  List<Object> getUserIdByTenderId(String tenderId){ 
		return expertService.getUserIdByTenderId(tenderId);
		
	}
	
	@GetMapping("/getExpertIdListByTenderIdAndSignTypeisNull")
	public List<Object> getExpertIdListByTenderIdAndSignTypeisNull(String tenderId){ 
		return expertService.getExpertIdListByTenderIdAndSignTypeisNull(tenderId);
	}
	
	@GetMapping("/getExpertByUserIdAndTenderId")
	public ExpertEntity getExpertByUserIdAndTenderId(String userId,String tenderId){ 
		return expertService.getExpertByUserIdAndTenderId(userId,tenderId);
	}
	
	
	@PostMapping("/saveOrUpdateReturn")
	@SysLogAnnotation(detail="修改专家信息",level = 0,operationType = OperationType.UPDATE)
	public ExpertEntity saveOrUpdateReturn (@RequestBody ExpertEntity expert){ 
		return expertService.saveOrUpdateReturn(expert);
		
	}

	@PostMapping("/batchSaveOrUpdate")
	public boolean batchSaveOrUpdate(@RequestBody List<ExpertEntity> expertList){ 
		return expertService.batchSaveOrUpdate(expertList);
	}
	
	@GetMapping("/updateSignTypeByTenderIdOrUserId")
	public boolean updateSignTypeByTenderIdOrUserId(String signType,String tenderId,String userId){ 
		return expertService.updateSignTypeByTenderIdOrUserId(signType, tenderId, userId);
	}
	@GetMapping("/getExpertListByProjectId")
	public List<ExpertEntity> getExpertListByProjectId(String projectId) {
		return expertService.getExpertListByProjectId(projectId);
	}
	
	@PostMapping("/updateExpertInfo")
	@SysLogAnnotation(detail="修改专家信息",level = 0,operationType = OperationType.UPDATE)
	public boolean updateExpertInfo(@RequestBody UpdateExpertInfoVo updateExpertInfoVo){ 
		ExpertEntity selectExpert = updateExpertInfoVo.getSelectExpert();
		ExpertEntity curExpert = updateExpertInfoVo.getCurExpert();
		AuthUserEntity user = updateExpertInfoVo.getUser();
		
		return expertService.updateExpertInfo(selectExpert,curExpert,user);
	}

	/**
	 * @Description: 两位专家是否在同一项目中
	 * @author chenxw
	 * @param tenderId
	 * @param selectUserId
	 * @param tempUserId
	 * @return
	 */
	@GetMapping("/isTenderExpertContain")
	public boolean isTenderExpertContain(String tenderId, String selectUserId, String tempUserId) {
		return expertService.isTenderExpertContain(tenderId, selectUserId,tempUserId);
	}

	/**
	 * @Description: 修改临时账号信息
	 * @author chenxw
	 * @param selectUserId
	 * @param tempUserId
	 * @return
	 */
	@GetMapping("/updateTempExpertMessgae")
	@SysLogAnnotation(detail="修改临时账号信息",level = 0,operationType = OperationType.UPDATE)
	public boolean updateTempExpertMessgae(String selectUserId, String tempUserId) {
		return expertService.updateTempExpertMessgae(selectUserId,tempUserId);
	}

	/**
	 * @Description: 获得专家姓名的专家
	 * @author wangxiang
	 * @return 
	 */
	@GetMapping("/getExpertListByUserName")
	public List<ExpertEntity> getExpertListByUserName(String userName) {
		return expertService.getExpertListByUserName(userName);
	}
	
}
