package com.gx.obe.server.management.expert.dao;

import java.util.List;

import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;
import org.springframework.web.bind.annotation.GetMapping;

import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.gx.obe.server.management.evaluation.entity.EvaluationStep;
import com.gx.obe.server.management.expert.entity.ExpertEntity;
import com.gx.obe.server.management.user.entity.AuthUserEntity;
/** 
 * @Description: 
 * @author mazc 
 */
@Mapper
public interface ExpertMapper extends BaseMapper<ExpertEntity> {

	/**
	 * @Description: 批量插入
	 * @author mazc
	 * @param ObeExpertList 
	 */
	Integer insertByBatch (@Param("ObeExpertList")List<ExpertEntity> ObeExpertList);

	Integer updateExpertBatchList (@Param("ExpertList")List<ExpertEntity> expertList);

	/**
	 * @Description: 指定字段修改
	 * @author chenxw
	 * @param ObeExpert
	 * @param attributes
	 */
	Integer updateAssignProperty(@Param("ObeExpert")ExpertEntity ObeExpert, @Param("attributes")String[] attributes);

	/**
	 * @Description: 批量指定字段修改
	 * @author chenxw
	 * @param ObeExpertList
	 * @param attributes
	 */
	Integer batchUpdateProperty(@Param("ObeExpertList")List<ExpertEntity> ObeExpertList, @Param("attributes")String[] attributes);

	ExpertEntity findExpertBatch(@Param("tenderId")String tenderId, @Param("userId")String userId);
	
	Integer updateSignType(@Param("signType")String signType, @Param("tenderId")String tenderId);
	
	/**
	 * @Description: 获得评标专家列表
	 * @author mazc
	 * @param tenderId
	 * @param signType
	 * @return 
	 */
	List<ExpertEntity> findExpertByTenderIdAndSignTypeBatch(@Param("tenderId")String tenderId, @Param("signType")String signType);
	
	
	/**
	 * @Description: 通过项目id 获得的评标专家
	 * @author mazc
	 * @param tenderId
	 * @return 
	 */
	List<ExpertEntity> findExpertByTenderId(@Param("tenderId") String tenderId);
	
	/**
	 * @Description: 获得评标专家组长
	 * @author mazc
	 * @param tenderId
	 * @return 
	 */
	ExpertEntity findExpertLeader(@Param("tenderId") String tenderId);
	
	
	/**
	 * @Description: 获得评标步骤的评标专家
	 * @author mazc
	 * @param tenderId
	 * @param evaluationStep
	 * @return 
	 */
	List<ExpertEntity> findEvaluationStepExpertList(@Param("tenderId")String tenderId, @Param("evaluationStep")EvaluationStep evaluationStep);
	
	/**
	 * @Description: 根据项目父级ID查询当前项目（包含子项目）下所有的评审专家
	 * @author mazc
	 * @param projectId
	 * @return 
	 */
	List<ExpertEntity> findExpertListByTenderId(@Param("projectId") String projectId);
	
	
	/**
	 * @Description: 获得参与当前评标步骤的专家数
	 * @author mazc
	 * @param tenderId
	 * @param evaluationStep
	 * @return 
	 */
	Integer findEvaluationStepExpertCount (@Param("tenderId")String tenderId, @Param("evaluationStep")EvaluationStep evaluationStep);
	
	
	
	/**
	 * @Description: 获得评标专家列表
	 * @author mazc
	 * @param tenderId
	 * @param signType
	 * @return 
	 */
	List<ExpertEntity> findExpertList(@Param("tenderId")String tenderId,@Param("signType") String signType);
	
	/**
	 * @Description:  获得评标步骤的评标专家信息
	 * @author mazc
	 * @param tenderId
	 * @param evaluationStep
	 * @return 
	 */
	List<AuthUserEntity> getEvaluationStepUserList(@Param("tenderId")String tenderId, @Param("evaluationStep")EvaluationStep evaluationStep);
	
	/**
	 * @Description: 重新开始评审  修改专家的签到状态
	 * @author mazc
	 * @param tenderId
	 * @return
	 */
	Integer updateRestartEvaluation(@Param("tenderId")String tenderId);

	/**
	 * @Description: 获得系统中所有的评标专家
	 * @author wangxiang
	 * @return 
	 */
	List<ExpertEntity> getSystemExpertList(@Param("userType")String userType);

	/**
	 * @Description: 获得指定评标步骤的自定义评标专家小组列表
	 * @author wangxiang
	 * @param tenderId 项目id
	 * @param stepId 步骤Id
	 * @return
	 */
	List<ExpertEntity> getCustomExpertList(@Param("tenderId")String tenderId, @Param("stepId")String stepId, @Param("signType")String signType);

	/**
	 * @Description: 获得最大的专家账号
	 * @author wangxiang
	 * @return
	 */
	Integer getMaxExpertAccountName();


	Integer votesVoterIdCount(@Param("voterId")String voterId);
	
	Integer votesMyIdCount(@Param("myId")String myId);

	Integer deleteAllByTenderId(@Param("tenderId")String tenderId);

	/**
	 * @Description: 获取专家可分配的分组列表
	 * @author wangxiang
	 * @param tenderId
	 * @param expertId
	 * @return 
	 */
	List<Object> getEvaGroupList(@Param("tenderId")String tenderId, @Param("expertId")String expertId);


	/**
	 * @Description: 获得评标单元指定的专家
	 * @author wangxiang
	 * @param tenderId
	 * @param evalGroup
	 * @return
	 */
	ExpertEntity getExpertByevalGroup(@Param("tenderId")String tenderId, @Param("evalGroup")String evalGroup);

	/**
	 * @Description: 获取所有指标单元
	 * @author wangxiang
	 * @param tenderId
	 * @return
	 */
	List<Object> getAllEvaGroupList(@Param("tenderId")String tenderId);

	/**
	 * @Description: 删除垃圾专家
	 * @author wangxiang
	 * @param tenderId 
	 */
	Integer deleteDirtyExpert(@Param("tenderId")String tenderId);

	/**
	 * @Description: 获取未签到专家id列表
	 * @author chenxw
	 * @param tenderId
	 * @return 
	 */
	List<Object> getUnsignedExpertIdList(@Param("tenderId")String tenderId);
	
	
	List<ExpertEntity> getExpertListByProjectId(@Param("projectId")String projectId);
	
	
	/**
	 * @Description: 获得项目的评标专家
	 * @author mazc
	 * @param tenderId
	 * @return 
	 */
	List<ExpertEntity> getExpertListByTenderId(@Param("tenderId")String tenderId);

	/**
	 * @Description: 两位专家是否在同一项目中
	 * @author chenxw
	 * @param selectUserId
	 * @param tempUserId
	 * @return
	 */
	Integer getTenderExpertContainCount(@Param("tenderId") String tenderId, @Param("selectUserId") String selectUserId, @Param("tempUserId") String tempUserId);

	/**
	 * @Description: 获得专家姓名的专家
	 * @author wangxiang
	 * @return 
	 */
	@GetMapping("/getExpertListByUserName")
	List<ExpertEntity> getExpertListByUserName(String userName);

}