package com.gx.obe.server.management.fileload.controller;

import com.gx.obe.server.common.utils.StringUtils;
import com.gx.obe.server.management.fileload.service.DownloadFileService;
import com.gx.obe.server.management.fileload.utill.FileMd5Util;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.util.Assert;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.Base64;

@RestController
@RequestMapping("/download")
public class DownloadController {
	
	private static final org.slf4j.Logger LOG = org.slf4j.LoggerFactory.getLogger(DownloadController.class);

	@Value("${upload.folder}")
	private String uploadFolder;
	
	@Value("${upload.videofolder}")
	private String videofolder;

	@Value("${upload.templateFolder}")
	private String templateFolder;
	
	
	@Value("${upload.softwareFolder}")
	private String softwareFolder;
	@Autowired
    private DownloadFileService downloadFileService;
	
	
	@PostMapping("/downloadFile")
	public void downloadFile(HttpServletRequest request, HttpServletResponse response,String fileURL){
		fileURL = new String(Base64.getDecoder().decode(fileURL),StandardCharsets.UTF_8);	
		StringBuilder sb = new StringBuilder();
		sb.append(uploadFolder).append(File.separator).append(fileURL);
		Path paht = Paths.get(sb.toString());
		Assert.notNull(paht,"路径不存在");
		download(request,response,paht.toAbsolutePath().toString());
	}
	
	
	@PostMapping("/downloadVideoFile")
	public void downloadVideoFile(HttpServletRequest request, HttpServletResponse response,String fileURL){
		fileURL = new String(Base64.getDecoder().decode(fileURL),StandardCharsets.UTF_8);	
		StringBuilder sb = new StringBuilder();
		sb.append(videofolder).append(File.separator).append(fileURL);
		Path paht = Paths.get(sb.toString());
		Assert.notNull(paht,"路径不存在");
		download(request,response,paht.toAbsolutePath().toString());
	}

	@PostMapping("/downloadSoftware")
	public void downloadSoftware(HttpServletRequest request, HttpServletResponse response, String fileName) {
		Path path = Paths.get(softwareFolder + File.separator + fileName);
		Assert.notNull(path, "路径不存在");
		download(request, response, path.toAbsolutePath().toString());
	}
	
	@PostMapping("/downloadTemplateFile")
	public void downloadTemplateFile(HttpServletRequest request, HttpServletResponse response, String fileName) {
		Path path = Paths.get(templateFolder + File.separator + fileName);
		Assert.notNull(path, "路径不存在");
		download(request, response, path.toAbsolutePath().toString());
	}
	

	public void download(HttpServletRequest req,HttpServletResponse resp,String path){
		BufferedInputStream bis = null;
		try {
			File file = new File(path);
			if (file.exists()) {
				long startLoc = 0L;
				long toLength = 0L;
				long contentLength = 0L;
				int rangeSwitch = 0; // 0,从头开始的全文下载；1,从某字节开始的下载（bytes=27000-）；2,从某字节开始到某字节结束的下载（bytes=27000-39000）
				long fileLength;
				String rangBytes = "";
				fileLength = file.length();
				//获取文件流
				InputStream ins = new FileInputStream(file);
				bis = new BufferedInputStream(ins);
				resp.reset();
				resp.setHeader("Accept-Ranges", "bytes");
				// 客户端请求文件块下载开始字节
				String range = req.getHeader("bytesRange");
				if (range != null && range.trim().length() > 0 && !"null".equals(range)) {
					resp.setStatus(javax.servlet.http.HttpServletResponse.SC_PARTIAL_CONTENT);
					rangBytes = range.replaceAll("bytes=", "");
					if (rangBytes.endsWith("-")) {  // bytes=270000-
						rangeSwitch = 1;
						startLoc = Long.parseLong(rangBytes.substring(0, rangBytes.indexOf("-")));
						contentLength = fileLength - startLoc;  // 客户端请求的是270000之后的字节（包括bytes下标索引为270000的字节）
					} else {
						// bytes=270000-320000
						rangeSwitch = 2;
						String temp1 = rangBytes.substring(0, rangBytes.indexOf("-"));
						String temp2 = rangBytes.substring(rangBytes.indexOf("-") + 1, rangBytes.length());
						startLoc = Long.parseLong(temp1);
						toLength = Long.parseLong(temp2);
						contentLength = toLength - startLoc + 1; // 客户端请求的是 270000-320000 之间的字节
					}
				} else {
					contentLength = fileLength;
				}
				// 如果设设置了Content-Length，则客户端会自动进行多线程下载。如果不希望支持多线程，则不要设置这个参数。
				// Content-Length: [文件的总大小] - [客户端请求的下载的文件块的开始字节]
				resp.setHeader("Content-Length", new Long(contentLength).toString());
				resp.setHeader("Yc-Length", new Long(contentLength).toString());
				// 如果请求获取文件mD5值则放到header中返回请求
				String reqMD5 = req.getHeader("GetContent-MD5");
				if(StringUtils.isNotEmpty(reqMD5)) {
					resp.setHeader("Content-MD5", FileMd5Util.getFileMD5(file));					
				}
				//断点开始 响应的格式是: Content-Range: bytes [文件块的开始字节]-[文件的总大小 - 1]/[文件的总大小]
				if (rangeSwitch == 1) {
					String contentRange = new StringBuffer("bytes ").append(new Long(startLoc).toString()).append("-")
							.append(new Long(fileLength - 1).toString()).append(File.separator)
							.append(new Long(fileLength).toString()).toString();
					resp.setHeader("Content-Range", contentRange);
					bis.skip(startLoc);
				} else if (rangeSwitch == 2) {
					String contentRange = range.replace("=", " ") + File.separator + new Long(fileLength).toString();
					resp.setHeader("Content-Range", contentRange);
					bis.skip(startLoc);
				} else {
					String contentRange = new StringBuffer("bytes ").append("0-")
							.append(fileLength - 1).append(File.separator)
							.append(fileLength).toString();
					resp.setHeader("Content-Range", contentRange);
				}
				String fileName = file.getName();
				resp.setContentType("application/octet-stream");
				if (req.getHeader("User-Agent").toUpperCase().indexOf("MSIE") > 0) {
					fileName = URLEncoder.encode(fileName, "UTF-8");
				} else {
					fileName = new String(fileName.getBytes("UTF-8"), "ISO8859-1");
				}
				resp.addHeader("Content-Disposition", "attachment;filename=" + fileName);
				resp.setHeader("Yc-filename", fileName);

				OutputStream out = resp.getOutputStream();
				int n = 0;
				long readLength = 0;
				int bsize = 1024;
				byte[] bytes = new byte[bsize];
				if (rangeSwitch == 2) {
					// 针对 bytes=27000-39000 的请求，从27000开始写数据
					while (readLength <= contentLength - bsize) {
						n = bis.read(bytes);
						readLength += n;
						out.write(bytes, 0, n);
					}
					if (readLength <= contentLength) {
						n = bis.read(bytes, 0, (int) (contentLength - readLength));
						out.write(bytes, 0, n);
					}
				} else {
					while ((n = bis.read(bytes)) != -1) {
						out.write(bytes,0,n);
					}
				}
				out.flush();
				out.close();
				bis.close();
			} else {
				if (LOG.isDebugEnabled()) {
					LOG.debug("Error: file " + path + " not found.");
				}
			}
		} catch (IOException ie) {
			// 忽略 ClientAbortException 之类的异常
	        } catch (Exception e) {
	        	LOG.error(e.getMessage());
	        }finally {
	            try {
	            	if(bis != null) {
	            		bis.close();
	            	}
	            } catch (IOException e) {
	                e.printStackTrace();
	            }
	        }

	    }
}
