package com.gx.obe.server.management.im.service.impl;

import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.beans.factory.annotation.Autowired;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;

import java.util.ArrayList;
import java.util.List;

import com.gx.obe.server.common.utils.IDUtils;
import com.gx.obe.server.common.utils.StringUtils;
import com.gx.obe.server.im.enumeration.GroupEnum;
import com.gx.obe.server.management.im.dao.ChatGroupMapper;
import com.gx.obe.server.management.im.dao.GroupItemMapper;
import com.gx.obe.server.management.im.entity.ChatGroup;
import com.gx.obe.server.management.im.entity.GroupItem;
import com.gx.obe.server.management.im.service.ChatGroupService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;

/** 
 * @Description: 
 * @author mazc 
 */
@Service
public class ChatGroupServiceImpl extends ServiceImpl<ChatGroupMapper, ChatGroup> implements ChatGroupService {

	@Autowired
	private ChatGroupMapper chatGroupMapper;
	@Autowired
	private  GroupItemMapper groupItemMapper;
	
	/**
	 * @Description: 批量更新
	 * @author mazc
	 * @param ChatGroupList 
	 */
	@Override
	@Transactional(rollbackFor = Exception.class)
	public Integer updateByBatch(List<ChatGroup> ChatGroupList) {
		return chatGroupMapper.updateBatchList(ChatGroupList);
	}
	
	/**
	 * @Description: 批量插入
	 * @author mazc
	 * @param ChatGroupList 
	 */
	@Override
	@Transactional(rollbackFor = Exception.class)
	public Integer insertByBatch(List<ChatGroup> ChatGroupList) {
		return chatGroupMapper.insertByBatch(ChatGroupList);
	}
	
	/**
     * @Description: 指定字段修改
     * @author chenxw
     * @param ChatGroup
	 * @param attributes
     */
	@Override
    public boolean updateAssignProperty(ChatGroup ChatGroup, String[] attributes) {
		return chatGroupMapper.updateAssignProperty(ChatGroup, attributes) > 0;
	}
    
    /**
     * @Description: 批量指定字段修改
     * @author chenxw
     * @param ChatGroupList
	 * @param attributes
     */
	@Override
    public Integer batchUpdateProperty(List<ChatGroup> ChatGroupList, String[] attributes) {
		return chatGroupMapper.batchUpdateProperty(ChatGroupList, attributes);
	}
	
	/** 
	 * @Description: 批量添加或跟新
	 * @author chenxw
	 * @param ChatGroupList
	 * @param attributes
	 * @return 
	 */
	@Override
	@Transactional(rollbackFor = Exception.class)
    public Integer batchSaveOrUpdate(List<ChatGroup> ChatGroupList, String[] attributes) {
        List<ChatGroup> insertChatGroupList = new ArrayList<>();
        List<ChatGroup> updateChatGroupList = new ArrayList<>();
        for (ChatGroup ChatGroup : ChatGroupList) {
            if (StringUtils.isEmpty(ChatGroup.getId())) {
                ChatGroup.setId(IDUtils.getId());
                insertChatGroupList.add(ChatGroup);
            } else {
                updateChatGroupList.add(ChatGroup);
            }
        }
        int count = 0;
        if (insertChatGroupList.size() > 0) {
            count += chatGroupMapper.insertByBatch(insertChatGroupList);
        }
        if (updateChatGroupList.size() > 0) {
            count += chatGroupMapper.batchUpdateProperty(updateChatGroupList, attributes);
        }
        return count;
    }

	@Override
	public List<ChatGroup> getGroupList(String userId) {
		return chatGroupMapper.getGroupList(userId);
	}

	@Override
	@Transactional(rollbackFor = Exception.class)
	public ChatGroup saveGroup(ChatGroup group, List<String> itemObjIdList) {
 		if(StringUtils.isEmpty(group.getId())) {
			group.setId(IDUtils.getId());
			chatGroupMapper.insert(group);
		}else {
			chatGroupMapper.updateById(group);
		}
		
		QueryWrapper<GroupItem> queryWrapper = new QueryWrapper<GroupItem>();
		queryWrapper.lambda().eq(GroupItem::getGroupId, group.getId());
		groupItemMapper.delete(queryWrapper);
		
		List<GroupItem> groupItemList = new ArrayList<GroupItem>();
		if(!itemObjIdList.contains(group.getManagerId())){
			itemObjIdList.add(group.getManagerId());
		}
		
		for(String objId : itemObjIdList){
			GroupItem groupItem = new GroupItem();
			groupItem.setId(IDUtils.getId());
			groupItem.setGroupId(group.getId());
			groupItem.setUserId(objId);
			if(objId.equals(group.getManagerId())){
				groupItem.setItemType(GroupEnum.ADMIN);
			}else {
				groupItem.setItemType(GroupEnum.MEMBER);
			}
			groupItemList.add(groupItem);
		}
		groupItemMapper.insertByBatch(groupItemList);
		return group;
	}

}
