package com.gx.obe.server.management.im.service.impl;

import java.util.ArrayList;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.gx.obe.server.common.enumeration.CommonEnum;
import com.gx.obe.server.common.utils.IDUtils;
import com.gx.obe.server.common.utils.StringUtils;
import com.gx.obe.server.management.im.dao.DepartmentMapper;
import com.gx.obe.server.management.im.dao.EmployeeDepartmentMapper;
import com.gx.obe.server.management.im.entity.Department;
import com.gx.obe.server.management.im.entity.EmployeeDepartment;
import com.gx.obe.server.management.im.service.DepartmentService;

/** 
 * @Description: 
 * @author mazc 
 */
@Service
public class DepartmentServiceImpl extends ServiceImpl<DepartmentMapper, Department> implements DepartmentService {

	@Autowired
	private DepartmentMapper departmentMapper;
	
	@Autowired
	private EmployeeDepartmentMapper employeeDepartmentMapper;
	
	/**
	 * @Description: 批量更新
	 * @author mazc
	 * @param DepartmentList 
	 */
	@Override
	@Transactional(rollbackFor = Exception.class)
	public Integer updateByBatch(List<Department> DepartmentList) {
		return departmentMapper.updateBatchList(DepartmentList);
	}
	
	/**
	 * @Description: 批量插入
	 * @author mazc
	 * @param DepartmentList 
	 */
	@Override
	@Transactional(rollbackFor = Exception.class)
	public Integer insertByBatch(List<Department> DepartmentList) {
		return departmentMapper.insertByBatch(DepartmentList);
	}
	
	/**
     * @Description: 指定字段修改
     * @author chenxw
     * @param Department
	 * @param attributes
     */
	@Override
    public boolean updateAssignProperty(Department Department, String[] attributes) {
		return departmentMapper.updateAssignProperty(Department, attributes) > 0;
	}
    
    /**
     * @Description: 批量指定字段修改
     * @author chenxw
     * @param DepartmentList
	 * @param attributes
     */
	@Override
    public Integer batchUpdateProperty(List<Department> DepartmentList, String[] attributes) {
		return departmentMapper.batchUpdateProperty(DepartmentList, attributes);
	}
	
	/** 
	 * @Description: 批量添加或跟新
	 * @author chenxw
	 * @param DepartmentList
	 * @param attributes
	 * @return 
	 */
	@Override
	@Transactional(rollbackFor = Exception.class)
    public Integer batchSaveOrUpdate(List<Department> DepartmentList, String[] attributes) {
        List<Department> insertDepartmentList = new ArrayList<>();
        List<Department> updateDepartmentList = new ArrayList<>();
        for (Department Department : DepartmentList) {
            if (StringUtils.isEmpty(Department.getId())) {
                Department.setId(IDUtils.getId());
                insertDepartmentList.add(Department);
            } else {
                updateDepartmentList.add(Department);
            }
        }
        int count = 0;
        if (insertDepartmentList.size() > 0) {
            count += departmentMapper.insertByBatch(insertDepartmentList);
        }
        if (updateDepartmentList.size() > 0) {
            count += departmentMapper.batchUpdateProperty(updateDepartmentList, attributes);
        }
        return count;
    }

	@Override
	public int getMaxDepartmentSortNo(String parentId) {
		Integer sortNo = departmentMapper.getMaxDepartmentSortNo(parentId);
		return null != sortNo ? sortNo.intValue() : 0;
	}

	@Override
	public List<Department> getAllDepartmentList() {
		QueryWrapper<Department> queryWrapper = new QueryWrapper<Department>();
		queryWrapper.orderByDesc("sortNo");
		return departmentMapper.selectList(queryWrapper);
	}

	@Override
	@Transactional(rollbackFor = Exception.class)
	public List<Department> getDirectAssignedDepartment(String employeeId) {
		List<Department> directDepartmentList = new ArrayList<Department>();
		List<Department> departmentList = departmentMapper.getAssignedDepartment(employeeId);
		if(null != departmentList && departmentList.size() > 1){
			for(Department parentDepartment: departmentList){
				for(Department subDdepartment: departmentList){
					if(StringUtils.isNotEmpty(subDdepartment.getParentId()) && parentDepartment.getId().equals(subDdepartment.getParentId())){
						subDdepartment.setParentDepartment(parentDepartment);
						parentDepartment.addChild(subDdepartment);
					}
				}
			}
			QueryWrapper<EmployeeDepartment> queryWrapper = new QueryWrapper<EmployeeDepartment>();
			queryWrapper.lambda().select(EmployeeDepartment::getDepartmentId).eq(EmployeeDepartment::getDepartmentId, CommonEnum.YES)
				.eq(EmployeeDepartment::getEmployeeId,employeeId);
			List<Object> directDepartmentIdList = employeeDepartmentMapper.selectObjs(queryWrapper);
			for(Department department: departmentList){
				if(directDepartmentIdList.contains(department.getId())){
					directDepartmentList.add(department);
				}
			}
		}else {
			return departmentList;
		}
		return directDepartmentList;
	}
	
	 
	/**
	 * @Description:   封装上下级部门并只获得上级
	 * @author mazc
	 * @param list
	 * @return 
	 */
	private List<Department> initListDepartmentList(List<Department> list){
		List<Department> departmentList = new ArrayList<Department>();
		for(Department parentDepartment: list){
			for(Department subDdepartment: list){
				if(StringUtils.isNotEmpty(subDdepartment.getParentId()) && parentDepartment.getId().equals(subDdepartment.getParentId())){
					subDdepartment.setParentDepartment(parentDepartment);
					parentDepartment.addChild(subDdepartment);
				}
			}
			// 如果是顶级指标
			if(StringUtils.isEmpty(parentDepartment.getParentId())){
				departmentList.add(parentDepartment);
			}
		}
		return departmentList;
	}
	
}
