package com.gx.obe.server.management.im.service.impl;

import java.util.ArrayList;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.gx.obe.server.common.utils.IDUtils;
import com.gx.obe.server.common.utils.StringUtils;
import com.gx.obe.server.management.im.dao.GroupMessageMapper;
import com.gx.obe.server.management.im.entity.GroupMessage;
import com.gx.obe.server.management.im.service.GroupMessageService;

/** 
 * @Description: 
 * @author mazc 
 */
@Service
public class GroupMessageServiceImpl extends ServiceImpl<GroupMessageMapper, GroupMessage> implements GroupMessageService {

	@Autowired
	private GroupMessageMapper groupMessageMapper;
	
	/**
	 * @Description: 批量更新
	 * @author mazc
	 * @param GroupMessageList 
	 */
	@Override
	@Transactional(rollbackFor = Exception.class)
	public Integer updateByBatch(List<GroupMessage> GroupMessageList) {
		return groupMessageMapper.updateBatchList(GroupMessageList);
	}
	
	/**
	 * @Description: 批量插入
	 * @author mazc
	 * @param GroupMessageList 
	 */
	@Override
	@Transactional(rollbackFor = Exception.class)
	public Integer insertByBatch(List<GroupMessage> GroupMessageList) {
		return groupMessageMapper.insertByBatch(GroupMessageList);
	}
	
	/**
     * @Description: 指定字段修改
     * @author chenxw
     * @param GroupMessage
	 * @param attributes
     */
	@Override
    public boolean updateAssignProperty(GroupMessage GroupMessage, String[] attributes) {
		return groupMessageMapper.updateAssignProperty(GroupMessage, attributes) > 0;
	}
    
    /**
     * @Description: 批量指定字段修改
     * @author chenxw
     * @param GroupMessageList
	 * @param attributes
     */
	@Override
    public Integer batchUpdateProperty(List<GroupMessage> GroupMessageList, String[] attributes) {
		return groupMessageMapper.batchUpdateProperty(GroupMessageList, attributes);
	}
	
	/** 
	 * @Description: 批量添加或跟新
	 * @author chenxw
	 * @param GroupMessageList
	 * @param attributes
	 * @return 
	 */
	@Override
	@Transactional(rollbackFor = Exception.class)
    public Integer batchSaveOrUpdate(List<GroupMessage> GroupMessageList, String[] attributes) {
        List<GroupMessage> insertGroupMessageList = new ArrayList<>();
        List<GroupMessage> updateGroupMessageList = new ArrayList<>();
        for (GroupMessage GroupMessage : GroupMessageList) {
            if (StringUtils.isEmpty(GroupMessage.getId())) {
                GroupMessage.setId(IDUtils.getId());
                insertGroupMessageList.add(GroupMessage);
            } else {
                updateGroupMessageList.add(GroupMessage);
            }
        }
        int count = 0;
        if (insertGroupMessageList.size() > 0) {
            count += groupMessageMapper.insertByBatch(insertGroupMessageList);
        }
        if (updateGroupMessageList.size() > 0) {
            count += groupMessageMapper.batchUpdateProperty(updateGroupMessageList, attributes);
        }
        return count;
    }

	@Override
	public List<Object> getGruptMessageByReceiveIdlist(String id) {
		QueryWrapper<GroupMessage> messageObject = new QueryWrapper<GroupMessage>();
        messageObject.lambda().select(GroupMessage::getGroupId).eq(GroupMessage::getReceiveId, id);
		return groupMessageMapper.selectObjs(messageObject);
	}

	@Override
	public List<Object> getReceiveIdListByMessageIdAndGroupId(String id, String groupId) {
		QueryWrapper<GroupMessage> queryWrapper = new QueryWrapper<GroupMessage>();
    	queryWrapper.lambda().select(GroupMessage::getReceiveId).eq(GroupMessage::getMessageId, id).eq(GroupMessage::getGroupId,groupId);
    	return groupMessageMapper.selectObjs(queryWrapper);
	}

	@Override
	public int deleteGroupMessageByMessageIdAndGroupIdAndReceiveId(String id, String groupId, String myId) {
		QueryWrapper<GroupMessage> queryWrapper = new QueryWrapper<GroupMessage>();
		queryWrapper.lambda().eq(GroupMessage::getMessageId,  id).eq(GroupMessage::getGroupId, groupId)
		 .eq(GroupMessage::getReceiveId, myId);
		return groupMessageMapper.delete(queryWrapper);
	}

}
