package com.gx.obe.server.management.menu.service.impl;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.gx.obe.server.common.utils.IDUtils;
import com.gx.obe.server.management.menu.dao.AuthRoleMenuMapper;
import com.gx.obe.server.management.menu.entity.AuthMenuEntity;
import com.gx.obe.server.management.menu.entity.AuthRoleMenu;
import com.gx.obe.server.management.menu.service.AuthRoleMenuService;

/** 
 * @Description: 
 * @author mazc 
 */
@Service
@Transactional
public class AuthRoleMenuServiceImpl extends ServiceImpl<AuthRoleMenuMapper, AuthRoleMenu> implements AuthRoleMenuService {

	@Autowired
	private AuthRoleMenuMapper authRoleMenuMapper;
	
	/**
	 * @Description: 批量更新
	 * @author mazc
	 * @param AuthRoleMenuList 
	 */
	 @Override
	 @Transactional(rollbackFor = Exception.class)
	public Integer updateByBatch (List<AuthRoleMenu> AuthRoleMenuList){
		return authRoleMenuMapper.updateBatchList(AuthRoleMenuList);
	}
	
	/**
	 * @Description: 批量插入
	 * @author mazc
	 * @param AuthRoleMenuList 
	 */
	 @Override
	 @Transactional(rollbackFor = Exception.class)
	public Integer insertByBatch (List<AuthRoleMenu> AuthRoleMenuList){
		return authRoleMenuMapper.insertByBatch(AuthRoleMenuList);
	}

	@Override
	public List<Object> getRoleResourceList(String roleId) {
		QueryWrapper<AuthRoleMenu> query = new QueryWrapper<AuthRoleMenu>();
		query.select(AuthRoleMenu.MENU_ID).lambda().eq(AuthRoleMenu::getRoleId, roleId);
		return authRoleMenuMapper.selectObjs(query);
	}

	/** 
	 * @Description: 
	 * @author mazc
	 * @param authRoleId
	 * @param userId
	 * @param menuList
	 * @return 
	 */
	@Override
	public boolean assignRoleResource(String authRoleId, String userId, List<AuthMenuEntity> menuList) {
		QueryWrapper<AuthRoleMenu> query = new QueryWrapper<>();
		query.lambda().eq(AuthRoleMenu::getRoleId, authRoleId);
		authRoleMenuMapper.delete(query);
		List<AuthRoleMenu> roleMenuList = new ArrayList<>();
		for (AuthMenuEntity menu : menuList) {
			AuthRoleMenu roleMenu = new AuthRoleMenu();
			roleMenu.setCreateUserId(userId);
			roleMenu.setCreateTime(new Date());
			roleMenu.setId(IDUtils.getId());
			roleMenu.setRoleId(authRoleId);
			roleMenu.setMenuId(menu.getId());
			roleMenuList.add(roleMenu);
		}
		if (roleMenuList.size() > 0) {
			return authRoleMenuMapper.insertByBatch(roleMenuList) > 0;
		}
		return true;
	}
	

}
