package com.gx.obe.server.management.price.controller;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.gx.obe.server.common.base.BaseController;
import com.gx.obe.server.common.log.OperationType;
import com.gx.obe.server.common.log.SysLogAnnotation;
import com.gx.obe.server.common.vo.RequestBodyEntity;
import com.gx.obe.server.management.price.entity.ReBidPriceRound;
import com.gx.obe.server.management.price.service.ReBidPriceRoundService;
import com.gx.obe.server.management.supplier.entity.Supplier;

import io.swagger.annotations.Api;
import lombok.AllArgsConstructor;

/** 
 * @Description: 多轮报价
 * @author wangxiang 
 */
@Api(tags = "")
@RestController
@AllArgsConstructor
@RequestMapping("/reBidPriceRound")
public class ReBidPriceRoundController extends BaseController<ReBidPriceRoundService,ReBidPriceRound>{ 

	@Autowired
	public ReBidPriceRoundService reBidPriceRoundService;
	
	/**添加轮次
	 * @param tenderId
	 * @param round
	 * @return
	 */
	@PostMapping("/moveHisBidPrice")
	public boolean moveHisBidPrice(@RequestBody RequestBodyEntity<Supplier> requestBodyEntity){
		List<Supplier> supplierList = requestBodyEntity.getEntityList(); 
		String tenderId = (String) requestBodyEntity.getParam("tenderId"); 
		int round = (int) requestBodyEntity.getParam("round"); 
		return reBidPriceRoundService.moveHisBidPrice(tenderId,supplierList,round);
	}
	
	/**
	 * @Description: 获得项目的最大伦次
	 * @author wangxiang
	 * @param tenderId
	 * @return 
	 */
	@GetMapping("getMaxRound")
	public int getMaxRound(String tenderId){
		return reBidPriceRoundService.getMaxRound(tenderId);
	}
	
	/**
	 * @Description: 复原初始报价
	 * @author wangxiang
	 * @return 
	 */
	@GetMapping("moveInitBidPrice")
	public boolean moveInitBidPrice(String tenderId){
		return reBidPriceRoundService.moveInitBidPrice(tenderId);
	}
	
	
	@Deprecated
	@GetMapping("getBidPriceRoundByIndex")
	public ReBidPriceRound getBidPriceRoundByIndex(String tenderId, int round){
		return reBidPriceRoundService.getBidPriceRoundByIndex(tenderId, round);
	}
	@GetMapping("getBidPriceRoundList")
	public List<ReBidPriceRound> getBidPriceRoundList(String tenderId){
		return reBidPriceRoundService.getBidPriceRoundList(tenderId);
	}
	
	@PostMapping("/updateAssignProperty") 
	public boolean updateAssignProperty(@RequestBody ReBidPriceRound bidPriceRound ){
		return reBidPriceRoundService.updateAssignProperty(bidPriceRound);
	} 
	
	/**
	 * @Description: 开启新一轮报价
	 * @author guoyr
	 * @param requestBodyEntity
	 * @return 
	 */
	@PostMapping("/insertNewBidpriceRound") 
	@SysLogAnnotation(detail="开启新一轮远程报价",level = 0,operationType = OperationType.INSERT)
	public boolean insertNewBidpriceRound(@RequestBody RequestBodyEntity<ReBidPriceRound> requestBodyEntity){
		
		ReBidPriceRound reBidPriceRound = requestBodyEntity.getEntity();
//		boolean useLastBidPrice = requestBodyEntity.getBoolean("copyLastBidPrice", false);
		String tenderId = (String)requestBodyEntity.getParam("tenderId");
		String supplierIds = (String)requestBodyEntity.getParam("supplierIds");
		if(null != supplierIds && supplierIds.split(",").length == 0) {
			return false;
		}
		
		String evaluationStepSupplierId = (String)requestBodyEntity.getParam("evaluationStepSupplierIds");
		if(null != evaluationStepSupplierId && evaluationStepSupplierId.split(",").length == 0) {
			return false;
		}
		return reBidPriceRoundService.insertNewBidpriceRound(reBidPriceRound, tenderId, supplierIds.split(","), evaluationStepSupplierId.split(","));
	} 
	
	
	@PostMapping("/insertNewLocalBidpriceRound") 
	@SysLogAnnotation(detail="开启新一轮本地报价",level = 0,operationType = OperationType.INSERT)
	public boolean insertNewLocalBidpriceRound(@RequestBody RequestBodyEntity<ReBidPriceRound> requestBodyEntity){
		ReBidPriceRound reBidPriceRound = requestBodyEntity.getEntity();
 		String tenderId = (String)requestBodyEntity.getParam("tenderId");
		String supplierIds = (String)requestBodyEntity.getParam("supplierIds");
		if(null != supplierIds && supplierIds.split(",").length == 0) {
			return false;
		}
		return reBidPriceRoundService.insertNewLocalBidpriceRound(reBidPriceRound, tenderId, supplierIds.split(","));
	} 
	
	/**
	 * @Description: 完成当前轮次结果保存
	 * @author mazc
	 * @param bidPriceRound
	 * @return 
	 */
	@PostMapping("/completeCurrentRound") 
	@SysLogAnnotation(detail="完成当前轮次报价",level = 0,operationType = OperationType.INSERT)
	public boolean completeCurrentRound(@RequestBody ReBidPriceRound bidPriceRound) {
		bidPriceRound.setRoundStatus("01");
		return reBidPriceRoundService.completeCurrentRound(bidPriceRound);
	}
	
	
	
	
}
