package com.gx.obe.server.management.struct_new.controller;

import com.gx.obe.server.common.base.BaseController;
import com.gx.obe.server.common.log.SysLogAnnotation;
import com.gx.obe.server.common.vo.RequestBodyEntity;
import com.gx.obe.server.management.struct_new.entity.ObeEvaluationContent;
import com.gx.obe.server.management.struct_new.entity.StructDateInfo;
import com.gx.obe.server.management.struct_new.service.ObeEvaluationContentService;
import io.swagger.annotations.Api;
import lombok.AllArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * @author chenxw
 * @Description:
 */
@Api(tags = "")
@RestController
@AllArgsConstructor
@RequestMapping("/obeEvaluationContent")
public class ObeEvaluationContentController extends BaseController<ObeEvaluationContentService, ObeEvaluationContent> {

    @Autowired
    public ObeEvaluationContentService obeEvaluationContentServices;

    /**
     * @param ObeEvaluationContentList
     * @Description: 批量插入
     * @author mazc
     */
    @PostMapping("/insertByBatch")
    public int insertByBatch(@RequestBody List<ObeEvaluationContent> ObeEvaluationContentList) {
        return obeEvaluationContentServices.insertByBatch(ObeEvaluationContentList);
    }

    /**
     * @param ObeEvaluationContentList
     * @Description: 批量更新
     * @author mazc
     */
    @PostMapping("/updateByBatch")
    public int updateByBatch(@RequestBody List<ObeEvaluationContent> ObeEvaluationContentList) {
        return obeEvaluationContentServices.updateByBatch(ObeEvaluationContentList);
    }

    /**
     * @param tenderId
     * @return
     * @Description: 获取评审内容
     * @author chenxw
     */
    @GetMapping("/getEvaluationContentList")
    public List<ObeEvaluationContent> getEvaluationContentList(String tenderId) {
        return obeEvaluationContentServices.getEvaluationContentList(tenderId);
    }

    /**
     * @param tenderId
     * @param factorCode
     * @return
     * @Description: 根据指标编号获取评审内容列表
     * @author chenxw
     */
    @GetMapping("/getEvaluationContentListByFactorCode")
    public List<ObeEvaluationContent> getEvaluationContentListByFactorCode(String tenderId, String factorCode) {
        return obeEvaluationContentServices.getEvaluationContentListByFactorCode(tenderId, factorCode);
    }

    /**
     * @param structDateInfo
     * @Description: 保存结构化信息数据
     * @author chenxw
     */
    @PostMapping("/saveStructDateInfo")
    public boolean saveStructDateInfo(@RequestBody StructDateInfo structDateInfo) {
        return obeEvaluationContentServices.saveStructDateInfo(structDateInfo);
    }

    /**
     * @param tenderId
     * @return
     * @Description: 清空结构化信息数据
     * @author chenxw
     */
    @GetMapping("/deleteStructDateInfo")
    public boolean deleteStructDateInfo(String tenderId) {
        return obeEvaluationContentServices.deleteStructDateInfo(tenderId);
    }

    /**
     * @param structDateInfoRequestBodyEntity
     * @return
     * @Description: 清空并保存结构化信息数据
     * @author chenxw
     */
    @PostMapping("/deleteOrSaveStructDateInfo")
    @SysLogAnnotation(detail = "清空并保存结构化信息数据")
    public Boolean deleteOrSaveStructDateInfo(@RequestBody RequestBodyEntity<StructDateInfo> structDateInfoRequestBodyEntity) {
        String tenderId = (String) structDateInfoRequestBodyEntity.getParam("tenderId");
        StructDateInfo structDateInfo = structDateInfoRequestBodyEntity.getEntity();
        return obeEvaluationContentServices.deleteOrSaveStructDateInfo(tenderId, structDateInfo);
    }

}