package com.gx.obe.server.management.struct_new.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.gx.obe.server.common.utils.IDUtils;
import com.gx.obe.server.common.utils.StringUtils;
import com.gx.obe.server.management.struct_new.dao.ObeBidderBasicInfoMapper;
import com.gx.obe.server.management.struct_new.entity.ObeBidderBasicInfo;
import com.gx.obe.server.management.struct_new.service.ObeBidderBasicInfoService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;

/**
 * @author mazc
 * @Description:
 */
@Service
@Transactional
public class ObeBidderBasicInfoServiceImpl extends ServiceImpl<ObeBidderBasicInfoMapper, ObeBidderBasicInfo> implements ObeBidderBasicInfoService {

    @Autowired
    private ObeBidderBasicInfoMapper obeBidderBasicInfoMapper;

    /**
     * @param ObeBidderBasicInfoList
     * @Description: 批量更新
     * @author mazc
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Integer updateByBatch(List<ObeBidderBasicInfo> ObeBidderBasicInfoList) {
        return obeBidderBasicInfoMapper.updateBatchList(ObeBidderBasicInfoList);
    }

    /**
     * @param ObeBidderBasicInfoList
     * @Description: 批量插入
     * @author mazc
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Integer insertByBatch(List<ObeBidderBasicInfo> ObeBidderBasicInfoList) {
        return obeBidderBasicInfoMapper.insertByBatch(ObeBidderBasicInfoList);
    }

    /**
     * @param ObeBidderBasicInfo
     * @param attributes
     * @Description: 指定字段修改
     * @author chenxw
     */
    @Override
    public boolean updateAssignProperty(ObeBidderBasicInfo ObeBidderBasicInfo, String[] attributes) {
        return obeBidderBasicInfoMapper.updateAssignProperty(ObeBidderBasicInfo, attributes) > 0;
    }

    /**
     * @param ObeBidderBasicInfoList
     * @param attributes
     * @Description: 批量指定字段修改
     * @author chenxw
     */
    @Override
    public Integer batchUpdateProperty(List<ObeBidderBasicInfo> ObeBidderBasicInfoList, String[] attributes) {
        return obeBidderBasicInfoMapper.batchUpdateProperty(ObeBidderBasicInfoList, attributes);
    }

    /**
     * @param ObeBidderBasicInfoList
     * @param attributes
     * @return
     * @Description: 批量添加或跟新
     * @author chenxw
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Integer batchSaveOrUpdate(List<ObeBidderBasicInfo> ObeBidderBasicInfoList, String[] attributes) {
        List<ObeBidderBasicInfo> insertObeBidderBasicInfoList = new ArrayList<>();
        List<ObeBidderBasicInfo> updateObeBidderBasicInfoList = new ArrayList<>();
        for (ObeBidderBasicInfo ObeBidderBasicInfo : ObeBidderBasicInfoList) {
            if (StringUtils.isEmpty(ObeBidderBasicInfo.getId())) {
                ObeBidderBasicInfo.setId(IDUtils.getId());
                insertObeBidderBasicInfoList.add(ObeBidderBasicInfo);
            } else {
                updateObeBidderBasicInfoList.add(ObeBidderBasicInfo);
            }
        }
        int count = 0;
        if (insertObeBidderBasicInfoList.size() > 0) {
            count += obeBidderBasicInfoMapper.insertByBatch(insertObeBidderBasicInfoList);
        }
        if (updateObeBidderBasicInfoList.size() > 0) {
            count += obeBidderBasicInfoMapper.batchUpdateProperty(updateObeBidderBasicInfoList, attributes);
        }
        return count;
    }

    /**
     * @param tenderId
     * @param modelDataId
     * @return
     * @Description: 获取投标人基本情况列表
     * @author chenxw
     */
    public List<ObeBidderBasicInfo> getBidderBasicInfoList(String tenderId, String modelDataId) {
        QueryWrapper<ObeBidderBasicInfo> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(ObeBidderBasicInfo::getModelDataId, modelDataId);
        return obeBidderBasicInfoMapper.selectList(queryWrapper);
    }

}
