package com.gx.obe.server.management.struct_new.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.gx.obe.server.common.utils.IDUtils;
import com.gx.obe.server.common.utils.StringUtils;
import com.gx.obe.server.management.struct_new.dao.ObeBusinessLicenseMapper;
import com.gx.obe.server.management.struct_new.entity.ObeBusinessLicense;
import com.gx.obe.server.management.struct_new.service.ObeBusinessLicenseService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;

/**
 * @author mazc
 * @Description:
 */
@Service
@Transactional
public class ObeBusinessLicenseServiceImpl extends ServiceImpl<ObeBusinessLicenseMapper, ObeBusinessLicense> implements ObeBusinessLicenseService {

    @Autowired
    private ObeBusinessLicenseMapper obeBusinessLicenseMapper;

    /**
     * @param ObeBusinessLicenseList
     * @Description: 批量更新
     * @author mazc
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Integer updateByBatch(List<ObeBusinessLicense> ObeBusinessLicenseList) {
        return obeBusinessLicenseMapper.updateBatchList(ObeBusinessLicenseList);
    }

    /**
     * @param ObeBusinessLicenseList
     * @Description: 批量插入
     * @author mazc
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Integer insertByBatch(List<ObeBusinessLicense> ObeBusinessLicenseList) {
        return obeBusinessLicenseMapper.insertByBatch(ObeBusinessLicenseList);
    }

    /**
     * @param ObeBusinessLicense
     * @param attributes
     * @Description: 指定字段修改
     * @author chenxw
     */
    @Override
    public boolean updateAssignProperty(ObeBusinessLicense ObeBusinessLicense, String[] attributes) {
        return obeBusinessLicenseMapper.updateAssignProperty(ObeBusinessLicense, attributes) > 0;
    }

    /**
     * @param ObeBusinessLicenseList
     * @param attributes
     * @Description: 批量指定字段修改
     * @author chenxw
     */
    @Override
    public Integer batchUpdateProperty(List<ObeBusinessLicense> ObeBusinessLicenseList, String[] attributes) {
        return obeBusinessLicenseMapper.batchUpdateProperty(ObeBusinessLicenseList, attributes);
    }

    /**
     * @param ObeBusinessLicenseList
     * @param attributes
     * @return
     * @Description: 批量添加或跟新
     * @author chenxw
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Integer batchSaveOrUpdate(List<ObeBusinessLicense> ObeBusinessLicenseList, String[] attributes) {
        List<ObeBusinessLicense> insertObeBusinessLicenseList = new ArrayList<>();
        List<ObeBusinessLicense> updateObeBusinessLicenseList = new ArrayList<>();
        for (ObeBusinessLicense ObeBusinessLicense : ObeBusinessLicenseList) {
            if (StringUtils.isEmpty(ObeBusinessLicense.getId())) {
                ObeBusinessLicense.setId(IDUtils.getId());
                insertObeBusinessLicenseList.add(ObeBusinessLicense);
            } else {
                updateObeBusinessLicenseList.add(ObeBusinessLicense);
            }
        }
        int count = 0;
        if (insertObeBusinessLicenseList.size() > 0) {
            count += obeBusinessLicenseMapper.insertByBatch(insertObeBusinessLicenseList);
        }
        if (updateObeBusinessLicenseList.size() > 0) {
            count += obeBusinessLicenseMapper.batchUpdateProperty(updateObeBusinessLicenseList, attributes);
        }
        return count;
    }

    /**
     * @param tenderId
     * @param modelDataId
     * @return
     * @Description: 获取营业执照列表
     * @author chenxw
     */
    public List<ObeBusinessLicense> getBusinessLicenseList(String tenderId, String modelDataId) {
        QueryWrapper<ObeBusinessLicense> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(ObeBusinessLicense::getModelDataId, modelDataId);
        return obeBusinessLicenseMapper.selectList(queryWrapper);
    }

}
