package com.gx.obe.server.management.struct_new.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.gx.obe.server.common.utils.IDUtils;
import com.gx.obe.server.common.utils.StringUtils;
import com.gx.obe.server.management.struct_new.dao.ObePerformanceMapper;
import com.gx.obe.server.management.struct_new.entity.ObePerformance;
import com.gx.obe.server.management.struct_new.service.ObePerformanceService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;

/**
 * @author mazc
 * @Description:
 */
@Service
@Transactional
public class ObePerformanceServiceImpl extends ServiceImpl<ObePerformanceMapper, ObePerformance> implements ObePerformanceService {

    @Autowired
    private ObePerformanceMapper obePerformanceMapper;

    /**
     * @param ObePerformanceList
     * @Description: 批量更新
     * @author mazc
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Integer updateByBatch(List<ObePerformance> ObePerformanceList) {
        return obePerformanceMapper.updateBatchList(ObePerformanceList);
    }

    /**
     * @param ObePerformanceList
     * @Description: 批量插入
     * @author mazc
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Integer insertByBatch(List<ObePerformance> ObePerformanceList) {
        return obePerformanceMapper.insertByBatch(ObePerformanceList);
    }

    /**
     * @param ObePerformance
     * @param attributes
     * @Description: 指定字段修改
     * @author chenxw
     */
    @Override
    public boolean updateAssignProperty(ObePerformance ObePerformance, String[] attributes) {
        return obePerformanceMapper.updateAssignProperty(ObePerformance, attributes) > 0;
    }

    /**
     * @param ObePerformanceList
     * @param attributes
     * @Description: 批量指定字段修改
     * @author chenxw
     */
    @Override
    public Integer batchUpdateProperty(List<ObePerformance> ObePerformanceList, String[] attributes) {
        return obePerformanceMapper.batchUpdateProperty(ObePerformanceList, attributes);
    }

    /**
     * @param ObePerformanceList
     * @param attributes
     * @return
     * @Description: 批量添加或跟新
     * @author chenxw
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Integer batchSaveOrUpdate(List<ObePerformance> ObePerformanceList, String[] attributes) {
        List<ObePerformance> insertObePerformanceList = new ArrayList<>();
        List<ObePerformance> updateObePerformanceList = new ArrayList<>();
        for (ObePerformance ObePerformance : ObePerformanceList) {
            if (StringUtils.isEmpty(ObePerformance.getId())) {
                ObePerformance.setId(IDUtils.getId());
                insertObePerformanceList.add(ObePerformance);
            } else {
                updateObePerformanceList.add(ObePerformance);
            }
        }
        int count = 0;
        if (insertObePerformanceList.size() > 0) {
            count += obePerformanceMapper.insertByBatch(insertObePerformanceList);
        }
        if (updateObePerformanceList.size() > 0) {
            count += obePerformanceMapper.batchUpdateProperty(updateObePerformanceList, attributes);
        }
        return count;
    }

    /**
     * @param tenderId
     * @param modelDataId
     * @return
     * @Description: 获取业绩列表
     * @author chenxw
     */
    public List<ObePerformance> getPerformanceList(String tenderId, String modelDataId, String statue) {
        QueryWrapper<ObePerformance> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(ObePerformance::getModelDataId, modelDataId);
        return obePerformanceMapper.selectList(queryWrapper);
    }

}
