package com.gx.obe.server.management.struct_new.service.impl;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.gx.obe.server.common.utils.IDUtils;
import com.gx.obe.server.common.utils.StringUtils;
import com.gx.obe.server.management.struct_new.dao.ObeTemplateDataItemMapper;
import com.gx.obe.server.management.struct_new.entity.ObeTemplateDataItem;
import com.gx.obe.server.management.struct_new.service.ObeTemplateDataItemService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;

/**
 * @author mazc
 * @Description:
 */
@Service
@Transactional
public class ObeTemplateDataItemServiceImpl extends ServiceImpl<ObeTemplateDataItemMapper, ObeTemplateDataItem> implements ObeTemplateDataItemService {

    @Autowired
    private ObeTemplateDataItemMapper obeTemplateDataItemMapper;

    /**
     * @param ObeTemplateDataItemList
     * @Description: 批量更新
     * @author mazc
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Integer updateByBatch(List<ObeTemplateDataItem> ObeTemplateDataItemList) {
        return obeTemplateDataItemMapper.updateBatchList(ObeTemplateDataItemList);
    }

    /**
     * @param ObeTemplateDataItemList
     * @Description: 批量插入
     * @author mazc
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Integer insertByBatch(List<ObeTemplateDataItem> ObeTemplateDataItemList) {
        return obeTemplateDataItemMapper.insertByBatch(ObeTemplateDataItemList);
    }

    /**
     * @param ObeTemplateDataItem
     * @param attributes
     * @Description: 指定字段修改
     * @author chenxw
     */
    @Override
    public boolean updateAssignProperty(ObeTemplateDataItem ObeTemplateDataItem, String[] attributes) {
        return obeTemplateDataItemMapper.updateAssignProperty(ObeTemplateDataItem, attributes) > 0;
    }

    /**
     * @param ObeTemplateDataItemList
     * @param attributes
     * @Description: 批量指定字段修改
     * @author chenxw
     */
    @Override
    public Integer batchUpdateProperty(List<ObeTemplateDataItem> ObeTemplateDataItemList, String[] attributes) {
        return obeTemplateDataItemMapper.batchUpdateProperty(ObeTemplateDataItemList, attributes);
    }

    /**
     * @param ObeTemplateDataItemList
     * @param attributes
     * @return
     * @Description: 批量添加或跟新
     * @author chenxw
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Integer batchSaveOrUpdate(List<ObeTemplateDataItem> ObeTemplateDataItemList, String[] attributes) {
        List<ObeTemplateDataItem> insertObeTemplateDataItemList = new ArrayList<>();
        List<ObeTemplateDataItem> updateObeTemplateDataItemList = new ArrayList<>();
        for (ObeTemplateDataItem ObeTemplateDataItem : ObeTemplateDataItemList) {
            if (StringUtils.isEmpty(ObeTemplateDataItem.getId())) {
                ObeTemplateDataItem.setId(IDUtils.getId());
                insertObeTemplateDataItemList.add(ObeTemplateDataItem);
            } else {
                updateObeTemplateDataItemList.add(ObeTemplateDataItem);
            }
        }
        int count = 0;
        if (insertObeTemplateDataItemList.size() > 0) {
            count += obeTemplateDataItemMapper.insertByBatch(insertObeTemplateDataItemList);
        }
        if (updateObeTemplateDataItemList.size() > 0) {
            count += obeTemplateDataItemMapper.batchUpdateProperty(updateObeTemplateDataItemList, attributes);
        }
        return count;
    }

    /**
     * @param tenderId
     * @param supplierId
     * @param factorCode
     * @return
     * @Description: 获取小范本表单列表
     * @author chenxw
     */
    @Override
    public List<ObeTemplateDataItem> getTemplateDataItemList(String tenderId, String supplierId, String factorCode) {
        return obeTemplateDataItemMapper.getTemplateDataItemList(tenderId, supplierId, factorCode);
    }

    /**
     * @param tenderId
     * @param supplierId
     * @param contentId
     * @return
     * @Description: 获取小范本表单列表
     * @author chenxw
     */
    @Override
    public List<ObeTemplateDataItem> getListByContentId(String tenderId, String supplierId, String contentId) {
        return obeTemplateDataItemMapper.getListByContentId(tenderId, supplierId, contentId);
    }
}
