package com.gx.obe.server.management.struct_new.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.gx.obe.server.common.utils.IDUtils;
import com.gx.obe.server.common.utils.StringUtils;
import com.gx.obe.server.management.struct_new.dao.ObeTemplateTableMapper;
import com.gx.obe.server.management.struct_new.entity.ObeTemplateTable;
import com.gx.obe.server.management.struct_new.service.ObeTemplateTableService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;

/**
 * @author mazc
 * @Description:
 */
@Service
@Transactional
public class ObeTemplateTableServiceImpl extends ServiceImpl<ObeTemplateTableMapper, ObeTemplateTable> implements ObeTemplateTableService {

    @Autowired
    private ObeTemplateTableMapper obeTemplateTableMapper;

    /**
     * @param ObeTemplateTableList
     * @Description: 批量更新
     * @author mazc
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Integer updateByBatch(List<ObeTemplateTable> ObeTemplateTableList) {
        return obeTemplateTableMapper.updateBatchList(ObeTemplateTableList);
    }

    /**
     * @param ObeTemplateTableList
     * @Description: 批量插入
     * @author mazc
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Integer insertByBatch(List<ObeTemplateTable> ObeTemplateTableList) {
        return obeTemplateTableMapper.insertByBatch(ObeTemplateTableList);
    }

    /**
     * @param ObeTemplateTable
     * @param attributes
     * @Description: 指定字段修改
     * @author chenxw
     */
    @Override
    public boolean updateAssignProperty(ObeTemplateTable ObeTemplateTable, String[] attributes) {
        return obeTemplateTableMapper.updateAssignProperty(ObeTemplateTable, attributes) > 0;
    }

    /**
     * @param ObeTemplateTableList
     * @param attributes
     * @Description: 批量指定字段修改
     * @author chenxw
     */
    @Override
    public Integer batchUpdateProperty(List<ObeTemplateTable> ObeTemplateTableList, String[] attributes) {
        return obeTemplateTableMapper.batchUpdateProperty(ObeTemplateTableList, attributes);
    }

    /**
     * @param ObeTemplateTableList
     * @param attributes
     * @return
     * @Description: 批量添加或跟新
     * @author chenxw
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Integer batchSaveOrUpdate(List<ObeTemplateTable> ObeTemplateTableList, String[] attributes) {
        List<ObeTemplateTable> insertObeTemplateTableList = new ArrayList<>();
        List<ObeTemplateTable> updateObeTemplateTableList = new ArrayList<>();
        for (ObeTemplateTable ObeTemplateTable : ObeTemplateTableList) {
            if (StringUtils.isEmpty(ObeTemplateTable.getId())) {
                ObeTemplateTable.setId(IDUtils.getId());
                insertObeTemplateTableList.add(ObeTemplateTable);
            } else {
                updateObeTemplateTableList.add(ObeTemplateTable);
            }
        }
        int count = 0;
        if (insertObeTemplateTableList.size() > 0) {
            count += obeTemplateTableMapper.insertByBatch(insertObeTemplateTableList);
        }
        if (updateObeTemplateTableList.size() > 0) {
            count += obeTemplateTableMapper.batchUpdateProperty(updateObeTemplateTableList, attributes);
        }
        return count;
    }

    /**
     * @param tenderId
     * @param supplierId
     * @param relChapterType
     * @param dataCode
     * @return
     * @Description: 获取小范本表格
     * @author chenxw
     */
    @Override
    public ObeTemplateTable getTemplateTable(String tenderId, String supplierId, String relChapterType, String dataCode) {
        return obeTemplateTableMapper.selectOne(
                new QueryWrapper<ObeTemplateTable>().lambda()
                        .eq(ObeTemplateTable::getTenderId, tenderId)
                        .eq(ObeTemplateTable::getSupplierId, supplierId)
                        .eq(ObeTemplateTable::getRelChapterType, relChapterType)
                        .eq(ObeTemplateTable::getDataCode, dataCode)
        );
    }

    /**
     * @param tenderId
     * @param relChapterType
     * @param dataCode
     * @return
     * @Description: 获取小范本表格列表
     * @author chenxw
     */
    @Override
    public List<ObeTemplateTable> getTemplateTableList(String tenderId, String relChapterType, String dataCode) {
    	obeTemplateTableMapper.selectList(
                new QueryWrapper<ObeTemplateTable>().lambda()
                        .eq(ObeTemplateTable::getTenderId, tenderId)
                        .eq(ObeTemplateTable::getRelChapterType, relChapterType)
                        .eq(ObeTemplateTable::getDataCode, dataCode)
        );
    	System.out.println(obeTemplateTableMapper.selectList(
                new QueryWrapper<ObeTemplateTable>().lambda()
                .eq(ObeTemplateTable::getTenderId, tenderId)
                .eq(ObeTemplateTable::getRelChapterType, relChapterType)
                .eq(ObeTemplateTable::getDataCode, dataCode)
));
        return obeTemplateTableMapper.selectList(
                new QueryWrapper<ObeTemplateTable>().lambda()
                        .eq(ObeTemplateTable::getTenderId, tenderId)
                        .eq(ObeTemplateTable::getRelChapterType, relChapterType)
                        .eq(ObeTemplateTable::getDataCode, dataCode)
        );
    }
}
