package com.gx.obe.server.management.supplier.controller;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.gx.obe.server.common.base.BaseController;
import com.gx.obe.server.common.utils.IDUtils;
import com.gx.obe.server.management.supplier.entity.SupplierOpeningResult;
import com.gx.obe.server.management.supplier.service.SupplierOpeningResultService;
import com.gx.obe.server.management.vo.BatchSaveOrUpdateVo;
import com.gx.obe.server.management.vo.BatchUpdatePropertyVo;
import com.gx.obe.server.management.vo.UpdateAssignPropertyVo;

import io.swagger.annotations.Api;
import lombok.AllArgsConstructor;
/** 
 * @Description: 
 * @author mazc 
 */
@Api(tags = "")
@RestController
@AllArgsConstructor
@RequestMapping("/supplierOpeningResult")
public class SupplierOpeningResultController extends BaseController<SupplierOpeningResultService,SupplierOpeningResult>{
	
	@Autowired
	public SupplierOpeningResultService supplierOpeningResultService;
	
	/**
	 * @Description: 指定字段批量更新
	 * @author chenxw
	 * @param batchUpdatePropertyVo
	 * @return 
	 */
	@GetMapping("/batchUpdateProperty")
    public int batchUpdateProperty(@RequestBody BatchUpdatePropertyVo<SupplierOpeningResult> batchUpdatePropertyVo) {
        List<SupplierOpeningResult> supplierOpeningResultList = batchUpdatePropertyVo.getEntityList();
        String[] attributes = batchUpdatePropertyVo.getAttributes();
        return supplierOpeningResultService.batchUpdateProperty(supplierOpeningResultList, attributes);
    }
	
	/**
	 * @Description: 指定字段批量保存或更新
	 * @author chenxw
	 * @param supplierOpeningResultList
	 * @return 
	 */
	@PostMapping("/batchSaveOrUpdate")
    public Integer batchSaveOrUpdate(@RequestBody BatchSaveOrUpdateVo<SupplierOpeningResult> batchSaveOrUpdateVo) {
        List<SupplierOpeningResult> supplierList = batchSaveOrUpdateVo.getEntityList();
        String[] attributes = batchSaveOrUpdateVo.getAttributes();
        return supplierOpeningResultService.batchSaveOrUpdate(supplierList, attributes);
    }
	
	/**
	 * @Description: 获取投标人开标结果
	 * @author wangxiang
	 * @param tenderId
	 * @param supplierId
	 * @return 
	 */
	@GetMapping("/getSupplierOpeningResult")
	public SupplierOpeningResult getSupplierOpeningResult(String tenderId, String supplierId) {
		return supplierOpeningResultService.getSupplierOpeningResult(tenderId,supplierId);
	}
	
	/**
	 * @Description: 指定字段更新
	 * @author chenxw
	 * @param updateAssignPropertyVo
	 * @return 
	 */
	@PostMapping("/updateAssignProperty")
    public boolean updateAssignProperty(@RequestBody UpdateAssignPropertyVo<SupplierOpeningResult> updateAssignPropertyVo) {
        SupplierOpeningResult supplierOpeningResult = updateAssignPropertyVo.getEntity();
        String[] attributes = updateAssignPropertyVo.getAttributes();
        return supplierOpeningResultService.updateAssignProperty(supplierOpeningResult, attributes);
    }
	
	/**
	 * @Description: 保存或者更新后返回
	 * @author wangxiang
	 * @param supplierOpeningResult
	 * @return 
	 */
	@PostMapping("/saveOrUpdateReturn")
	public boolean saveOrUpdateReturn(@RequestBody SupplierOpeningResult supplierOpeningResult) {
	    supplierOpeningResult.setId(IDUtils.getId());
		return supplierOpeningResultService.save(supplierOpeningResult);
	}
	
	/**
	 * @Description: 查询或获取供应商开标结果
	 * @author wangxiang
	 * @param tenderId
	 * @param supplierId
	 * @return 
	 */
	@GetMapping("/queryOrCreateSupplierOpeningResult")
	public SupplierOpeningResult queryOrCreateSupplierOpeningResult(String tenderId, String supplierId){
		SupplierOpeningResult supplierOpeningResult = null;
		QueryWrapper<SupplierOpeningResult> resultWrapper = new QueryWrapper<SupplierOpeningResult>();
		resultWrapper.lambda().eq(SupplierOpeningResult::getTenderId, tenderId).eq(SupplierOpeningResult::getSupplierId, supplierId);
		supplierOpeningResult = supplierOpeningResultService.getOne(resultWrapper);
		if(null == supplierOpeningResult) {
			supplierOpeningResult = new SupplierOpeningResult();
			supplierOpeningResult.setId(IDUtils.getId());
			supplierOpeningResult.setTenderId(tenderId);
			supplierOpeningResult.setSupplierId(supplierId);
			if(supplierOpeningResultService.saveOrUpdate(supplierOpeningResult)) {
				return supplierOpeningResult;
			}else {
				return null ;
			}
		}else {
			return supplierOpeningResult;
		}
	}
	
	/**
	 * @Description: 获得开标结果主键
	 * @author wangxiang
	 * @param tenderId
	 * @param supplierId
	 * @return 
	 */
	@GetMapping("/getSupplierOpeningResultId")
	public String getSupplierOpeningResultId(String tenderId, String supplierId){
		return supplierOpeningResultService.getSupplierOpeningResultId(tenderId,supplierId);
	}
	
	/**
	 * @Description: 删除供应商开标结果
	 * @author wangxiang
	 * @param tenderId 
	 */
	@GetMapping("/deleteSupplierBidOpeningResult")
	public boolean deleteSupplierBidOpeningResult(String tenderId, String supplierId){
		QueryWrapper<SupplierOpeningResult> wrapper = new  QueryWrapper<SupplierOpeningResult>();
		wrapper.lambda().eq(SupplierOpeningResult::getTenderId, tenderId).eq(SupplierOpeningResult::getSupplierId, supplierId);
		return supplierOpeningResultService.remove(wrapper);
	}
	
	/**
	 * @Description: 获取开标结果列表
	 * @author wangxiang
	 * @param tenderId
	 * @return 
	 */
	@GetMapping("/getSupplierOpeningResultList")
	public List<SupplierOpeningResult> getSupplierOpeningResultList(String tenderId){
		QueryWrapper<SupplierOpeningResult> queryWrapper = new QueryWrapper<SupplierOpeningResult>();
		queryWrapper.lambda().eq(SupplierOpeningResult::getTenderId, tenderId);
		return supplierOpeningResultService.list(queryWrapper);
	}
	
	
	/**
	 * @Description: 
	 * @author mazc
	 * @param 
	 * @return 
	 */
	@PostMapping("/saveOrUpdateNoEmptyReturn")
	public SupplierOpeningResult saveOrUpdateNoEmptyReturn(@RequestBody SupplierOpeningResult supplierOpeningResult){
		return supplierOpeningResultService.saveOrUpdateNoEmptyReturn(supplierOpeningResult);
	}
	
	@GetMapping("/getSupplierOpeningResultListByTenderId")
	public List<SupplierOpeningResult> getSupplierOpeningResultListByTenderId(String tenderId){
		return supplierOpeningResultService.getSupplierOpeningResultListByTenderId(tenderId);
	}
	
	
	@GetMapping("/getSupplierOpeningResultListByTenderIdAndBidOpeningStatus")
	public List<SupplierOpeningResult> getSupplierOpeningResultListByTenderIdAndBidOpeningStatus(String tenderId, String bidOpeningStatus){
		return supplierOpeningResultService.getSupplierOpeningResultListByTenderIdAndBidOpeningStatus(tenderId, bidOpeningStatus);			
	}
	
	
	/**
	 * @Description: 修改供应商的总价
	 */
	@GetMapping("/updateSupplierSumPrice")
	public Integer updateSupplierSumPrice(String tenderId, boolean bidPrice, boolean evalPrice){ 
		return supplierOpeningResultService.updateSupplierSumPrice(tenderId, bidPrice, evalPrice);
		
	}
	
	@GetMapping("/updateSupplierSumPriceAndSupplierId")
	public Integer updateSupplierSumPriceAndSupplierId(String tenderId, String supplierId, boolean bidPrice, boolean evalPrice){ 
		return supplierOpeningResultService.updateSupplierSumPrice(tenderId, supplierId, bidPrice, evalPrice);
	}
	
	
}