package com.gx.obe.server.management.supplier.dao;

import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.gx.obe.server.management.supplier.entity.Supplier;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;

import java.util.List;
import java.util.Map;

/** 
 * @Description: 
 * @author mazc 
 */
@Mapper
public interface SupplierMapper extends BaseMapper<Supplier> {

	List<Supplier> getSupplierList(@Param("tenderId")String tenderId,@Param("useStatus")String useStatus);

	List<Supplier> getOpenBidSuccessSupplier(@Param("tenderId")String tenderId, @Param("useStatus")String useStatus, 
						  @Param("bidOpeningStatus")String bidOpeningStatus, @Param("revokeStatus")String revokeStatus);

	Integer  removeSupplier(@Param("supplierId")String supplierId);

	List<Supplier> getSupplierOpenBidList(@Param("tenderId")String tenderId);
	
	Integer updateObeSupplierBatchList (@Param("ObeSupplierList")List<Supplier> ObeSupplierList);
	
	Integer insertByBatch (@Param("ObeSupplierList")List<Supplier> ObeSupplierList);
	
	Integer findSupplierOpeningCount(@Param("tenderId") String tenderId);
	
	/**
	 * @Description: 修改供应商的投标保证金情况
	 * @author mazc
	 * @param supplierId
	 * @param bidBondType
	 * @return 
	 */
	Integer updateSupplierBidBondType(@Param("supplierId")String supplierId, @Param("bidBondType")String bidBondType);

	/**
	 * @Description: 按照签到顺序排序
	 * @author wangxiang
	 * @param id
	 * @param b 
	 */
	Integer sortSupplierBySignTime(@Param("supplierList")List<Supplier> supplierList);

	/**
	 * @Description: 获得所有未投标的投标人
	 * @author wangxiang
	 * @param tenderId
	 * @return 
	 */
	List<Supplier> getAllUnTenderSupplierList(@Param("tenderId")String tenderId);

	/**
	 * @Description: 验证供应商属性是否唯一
	 * @author wangxiang
	 * @param tenderId 项目 
	 * @param supplierId 主键
	 * @param column 数据库字段
	 * @param columnValue 字段值
	 * @return 
	 */
	Integer isUniqueSupplier(@Param("tenderId")String tenderId, @Param("supplierId")String supplierId, @Param("column")String column, @Param("columValue")String columnValue);

	/**
	 * @Description: 获得开标成功的投标人列表包含开标结果
	 * @author wangxiang
	 * @param tenderId
	 * @return 
	 */
	List<Supplier> getOpenBidSuccessSupplier(@Param("tenderId")String tenderId);
	

	/**
	 * @Description: 获得指定评标步骤下的供应商列表
	 * @author wangxiang
	 * @param tenderId
	 * @param unqualified 
	 * @param complete 
	 * @param openbidSuccess 
	 * @param revokeNo 
	 * @param stepIndex
	 * @return 
	 */
	List<Supplier> getEvaluationStepSupplierList(@Param("tenderId")String tenderId, @Param("revokeNo")String revokeNo, @Param("openbidSuccess")String openbidSuccess, @Param("completeStaues")String completeStaues, @Param("unqualified")String unqualified, @Param("stepIndex")int stepIndex);

	/**
	 * @Description: 获得投标人的开标结果和评标结果
	 * @author wangxiang
	 * @param tenderId
	 * @return
	 */
	List<Supplier> getSupplierOpenbidAndEvaluateResultList(@Param("tenderId")String tenderId, @Param("useStatus")String useStatus);

	/**
	 * @Description: 获得投标人的最大序号
	 * @author wangxiang
	 * @param tenderId
	 * @return
	 */
	Integer getMaxSupplierSortNo(@Param("tenderId")String tenderId);

	/**
	 * @Description: 获得指定步骤评审不通过的投标人
	 * @author wangxiang
	 * @param tenderId
	 * @param stepId 指定的评标步骤
	 * @return
	 */
	List<Supplier> getDisAccessmentSupplier(@Param("tenderId")String tenderId,@Param("accessment")String accessment, @Param("stepId")String stepId,@Param("useStatus")String useStatus);

	/**
	 * @Description: 根据项目Id，获得项目下所有分包的投标人集合(去重复)
	 * @author wangxiang
	 * @param tenderParentId 项目Id
	 * @return List<Supplier>
	 */
	List<Supplier> getSupplierListByTenderProject(@Param("tenderParentId")String tenderParentId,@Param("useStatus")String useStatus);

	/**
	 * @Description: 模糊查询投标人名称或编码，获得项目下的投标人列表。
	 * @author wangxiang
	 * @param tenderId 项目ID
	 * @param content 查询条件(投标人名称和编码)
	 * @return List<Supplier>
	 */
	List<Supplier> getSupplierListByName(@Param("tenderId")String tenderId,@Param("useStatus")String useStatus,  @Param("content")String content);

	/**
	 * @Description: 根据父项目id查询所有包件下的投标人信息
	 * @author wangxiang
	 * @param parentId
	 * @return 
	 */
	List<Map<String, Object>> getparentSupplierByParentTenderId(@Param("tenderId")String parentId);

	/**
	 * @Description: 获得报名供应商的个数
	 * @author wangxiang
	 * @param tenderId
	 * @return 
	 */
	Integer getSupplierCount(@Param("tenderId")String tenderId);

	/**
	 * @Description: 获得未废标且已签到的供应商家数
	 * @author wangxiang
	 * @param tenderId
	 * @return 
	 */
	Integer getSupplierSignCount(@Param("tenderId")String tenderId,@Param("revokeStatus")String revokeStatus);

	/**
	 * @Description: 获得推荐的供应商列表
	 * @author wangxiang
	 * @param tenderId
	 * @param stepIndex
	 * @return 
	 */
	List<Supplier> getRecommendSupplierList(@Param("tenderId")String tenderId, @Param("revokeStatus")String revokeStatus, @Param("bidOpeningStatus")String openbidSuccess, @Param("stepStatus")String complete, @Param("unqualified")String unqualified, @Param("sortNo")int stepIndex);

	
	
	List<Supplier> getPrivceRoundSupplierList(@Param("roundId") String roundId);
	
	/**
	 * @Description: 获得按项目开标的供应商
	 * @author wangxiang
	 * @param projectId
	 * @return 
	 */
	List<Supplier> getDecryptSupplierList(@Param("projectId")String projectId, @Param("tenderId")String tenderId, @Param("onlyTenderBid")boolean onlyTenderBid);

	/**
	 * @Description: 通过项目id查找项目下所有包件中资审为通过供应商数量
	 * @author wangxiang
	 * @param tenderParentId
	 * @param type [01：通过；02：不通过]
	 * @return 
	 */
	List<Supplier> getComplianceFailSupplierCount(@Param("tenderId")String tenderParentId, @Param("accessment")String accessment);

	/**
	 * @Description: 获得整个项目下开标成功的投标人列表
	 * @author wangxiang
	 * @param tenderParentId
	 * @param realStatus
	 * @param openbidSuccess
	 * @param revokeNo
	 * @return 
	 */
	List<Supplier> getOpenBidSuccessByParentId(@Param("tenderId")String tenderParentId,@Param("useStatus")String useStatus, @Param("bidOpeningStatus")String openbidSuccess,  @Param("revokeStatus")String revokeNo);

	/**
	 * @Description: 获得供应商开标信息
	 * @author wangxaing
	 * @param tenderId
	 * @return 
	 */
	List<Map<String, Object>> getSupplierBidOpeningInfoList(@Param("tenderId")String tenderId);

	/**
	 * @Description: 根据包件id查询该包件下，供应商投标人数
	 * @author wangxiang
	 * @param tenderId
	 * @return 
	 */
	Integer getBidSupplierCount(@Param("tenderId")String tenderId);

	/**
	 * @Description: 根据包件id查询该包件下，供应商商务技术解密人数
	 * @author wangxiang
	 * @param tenderId
	 * @param openbidSuccess 
	 * @param economicsFlag
	 * @return 
	 */
	Integer getSupplierBusinessDecryptCount(@Param("tenderId")String tenderId, @Param("openbidSuccess")String openbidSuccess,@Param("priceBidStatus") boolean priceBidStatus);

	/**
	 * @Description: 验证投标人名称是否唯一
	 * @author wangxiang
	 * @param tenderId
	 * @param supplierId
	 * @param supplierName
	 * @return 
	 */
	Integer isUniqueSupplierByName(@Param("tenderId")String tenderId, @Param("supplierId")String supplierId,@Param("supplierName") String supplierName);

	List<Map<String, Object>> queryToMap(@Param("tenderId")String tenderId,@Param("hasParent") boolean hasParent);
	
	/**
     * @Description: 指定字段修改
     * @author chenxw
     * @param ObeSupplier
     */
    Integer updateAssignProperty (@Param("ObeSupplier")Supplier ObeSupplier, @Param("attributes")String[] attributes);
    
    /**
     * @Description: 批量指定字段修改
     * @author chenxw
     * @param ObeSupplierList 
     */
    Integer batchUpdateProperty (@Param("ObeSupplierList") List<Supplier> ObeSupplierList, @Param("attributes")String[] attributes);
		
}