package com.gx.obe.server.management.supplier.service;

import com.baomidou.mybatisplus.extension.service.IService;
import com.gx.obe.server.management.supplier.entity.Supplier;
import com.gx.obe.server.management.vo.DecryptSupplier;
import com.gx.obe.server.management.vo.EvaluationSupplierVo;

import java.util.List;
import java.util.Map;

/** 
 * @Description: 供应商管理
 * @author wangxiang 
 */
public interface SupplierService extends IService<Supplier>{

	/**
	 * @Description: 查询项目或标段下的投标人
	 * @author wangxiang
	 * @param tenderId
	 * @return 
	 */
	List<Supplier> getSupplierList(String tenderId,String useStatus);

	/**
	 * @Description: 获得开标成功的供应商列表
	 * @author wangxiang
	 * @param tenderId
	 * @param bidFileType
	 * @return
	 */
	List<Supplier> getOpenBidSuccessSupplier(String tenderId, String bidFileType);

	/**
	 * @Description: 删除投标人
	 * @author wangxiang
	 * @param tenderId
	 * @param supplierId
	 * @return 
	 */
	boolean removeSupplier(String tenderId,String supplierId);

	/**
	 * @Description: 获得要开标的所有投标人列表，包括开标结果
	 * @author wangxiang
	 * @param tenderId
	 * @return 
	 */
	List<Supplier> getSupplierOpenBidList(String tenderId);
	
	
	/**
	 * @Description: 批量添加或跟新
	 * @author mazc
	 * @param supplierList
	 * @return 
	 */
    int batchSaveOrUpdate(List<Supplier> supplierList, String[] attributes);
	
	int insertByBatch (List<Supplier> supplierList);
	
	
	Integer updateByBatch (List<Supplier> ObeSupplierList);
	
	/**
	 * @Description: 获取开标供应商的数量
	 * @author mazc
	 * @param tenderId
	 * @return 
	 */
	Integer findSupplierOpeningCount(String tenderId);
	
	
    /**
     * @Description:批量指定字段修改
     * @author chenxw
     * @param supplierList
     * @param attributes
     * @return
     */
    Integer batchUpdateProperty(List<Supplier> supplierList, String[] attributes);

	/**
	 * @Description: 按照签到顺序排序
	 * @author wangxiang
	 * @param id
	 * @param b 
	 */
	boolean sortSupplierBySignTime(String tenderProjectId, boolean order);

	/**
	 * @Description: 获得所有未投标的投标人
	 * @author wangxiang
	 * @param tenderId
	 * @return 
	 */
	List<Supplier> getAllUnTenderSupplierList(String tenderId);

	
	/**
	 * @Description: 指定字段修改
	 * @author chenxw
	 * @param supplier
	 * @param attributes
	 * @return 
	 */
	boolean updateAssignProperty(Supplier supplier, String[] attributes);

	
	/**
	 * @Description: 验证供应商属性是否唯一
	 * @author wangxiang
	 * @param tenderId 项目 
	 * @param supplierId 主键
	 * @param column 数据库字段
	 * @param columnValue 字段值
	 * @return 
	 */
	boolean isUniqueSupplier(String tenderId, String supplierId, String column, String columnValue);

	
	/**
	 * @Description: 获得指定评标步骤下的供应商列表
	 * @author wangxiang
	 * @param tenderId
	 * @param stepIndex
	 * @return 
	 */
	List<Supplier> getEvaluationStepSupplierList(EvaluationSupplierVo evaluationSupplierVo);

	/**
	 * @Description: 获得投标人的开标结果和评标结果
	 * @author wangxiang
	 * @param tenderId
	 * @return
	 */
	List<Supplier> getSupplierOpenbidAndEvaluateResultList(String tenderId);

	/**
	 * @Description: 获得投标人的最大序号
	 * @author wangxiang
	 * @param tenderId
	 * @return
	 */
	int getMaxSupplierSortNo(String tenderId);

	/**
	 * @Description: 获得指定步骤评审不通过的投标人
	 * @author wangxiang
	 * @param tenderId
	 * @param stepId 指定的评标步骤
	 * @return
	 */
	List<Supplier> getDisAccessmentSupplier(String tenderId, String stepId);

	/**
	 * @Description: 根据项目Id，获得项目下所有分包的投标人集合(去重复)
	 * @author wangxiang
	 * @param tenderParentId 项目Id
	 * @return List<Supplier>
	 */
	List<Supplier> getSupplierListByTenderProject(String tenderParentId);

	/**
	 * @Description: 模糊查询投标人名称或编码，获得项目下的投标人列表。
	 * @author wangxiang
	 * @param tenderId 项目ID
	 * @param content 查询条件(投标人名称和编码)
	 * @return List<Supplier>
	 */
	List<Supplier> getSupplierListByName(String tenderId, String content);

	/**
	 * @Description: 根据父项目id查询所有包件下的投标人信息
	 * @author wangxiang
	 * @param parentId
	 * @return 
	 */
	List<Map<String, Object>> getparentSupplierByParentTenderId(String parentId);

	/**
	 * @Description: 获得报名供应商的个数
	 * @author wangxiang
	 * @param tenderId
	 * @return 
	 */
	Integer getSupplierCount(String tenderId);
	
	/**
	 * @Description: 获得未废标且已签到的供应商家数
	 * @author wangxiang
	 * @param tenderId
	 * @return 
	 */
	public Integer getSupplierSignCount(String tenderId);

	/**
	 * @Description: 获得推荐的供应商列表
	 * @author wangxiang
	 * @param tenderId
	 * @param stepIndex
	 * @return 
	 */
	List<Supplier> getRecommendSupplierList(String tenderId, int stepIndex);

	/**
	 * @Description: 获得按项目开标的供应商
	 * @author wangxiang
	 * @param projectId
	 * @return 
	 */
	List<DecryptSupplier> getDecryptSupplierList(String projectId, String tenderId, boolean onlyTenderBid);

	/**
	 * @Description: 通过项目id查找项目下所有包件中资审为通过供应商数量
	 * @author wangxiang
	 * @param tenderParentId
	 * @param type [01：通过；02：不通过]
	 * @return 
	 */
	List<Supplier> getComplianceFailSupplierCount(String tenderParentId, String accessment);

	
	/**
	 * @Description: 获得整个项目下开标成功的投标人列表
	 * @author wangxiang
	 * @param tenderParentId
	 * @return
	 */
	List<Supplier> getOpenBidSuccessByParentId(String tenderParentId);

	/**
	 * @Description: 获得供应商开标信息
	 * @author wangxaing
	 * @param tenderId
	 * @return 
	 */
	List<Map<String, Object>> getSupplierBidOpeningInfoList(String tenderId);

	/**
	 * @Description: 根据包件id查询该包件下，供应商投标人数
	 * @author wangxiang
	 * @param tenderId
	 * @return 
	 */
	Integer getBidSupplierCount(String tenderId);

	/**
	 * @Description: 根据包件id查询该包件下，供应商商务技术解密人数
	 * @author wangxiang
	 * @param tenderId
	 * @param economicsFlag
	 * @return 
	 */
	Integer getSupplierBusinessDecryptCount(String tenderId, boolean priceBidStatus);

	/**
	 * @Description: 验证投标人名称是否唯一
	 * @author wangxiang
	 * @param tenderId
	 * @param supplierId
	 * @param supplierName
	 * @return 
	 */
	Integer isUniqueSupplierByName(String tenderId, String supplierId, String supplierName);

	List<Map<String, Object>> queryToMap(String tenderId, boolean hasParent);

	boolean deleteAllSupplierByTenderId(String tenderId);
	
	/**
	 * 
	 * @Description: 通过轮次信息获取供应商信息
	 * @author mazc
	 * @param roundId
	 * @return
	 */
	List<Supplier> getPrivceRoundSupplierList(String roundId);

}

