package com.gx.obe.server.management.supplier.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.gx.obe.server.common.utils.CollectionUtils;
import com.gx.obe.server.common.utils.IDUtils;
import com.gx.obe.server.common.utils.ObjectUtils;
import com.gx.obe.server.common.utils.StringUtils;
import com.gx.obe.server.management.evaluation.entity.BidPriceResult;
import com.gx.obe.server.management.evaluation.service.BidPriceResultService;
import com.gx.obe.server.management.supplier.dao.SupplierOpeningResultMapper;
import com.gx.obe.server.management.supplier.entity.SupplierOpeningResult;
import com.gx.obe.server.management.supplier.service.SupplierOpeningResultService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

/** 
 * @Description: 
 * @author mazc 
 */
@Service
@Transactional
public class SupplierOpeningResultServiceImpl extends ServiceImpl<SupplierOpeningResultMapper, SupplierOpeningResult> implements SupplierOpeningResultService {

	@Autowired
	private BidPriceResultService bidPriceResultService;

	@Autowired
	private SupplierOpeningResultMapper supplierOpeningResultMapper;

	/** 
	 * @Description: 批量插入
	 * @author chenxw
	 * @param ObeSupplierOpeningResultList
	 * @return 
	 */
	@Override
	public int insertByBatch(List<SupplierOpeningResult> ObeSupplierOpeningResultList) {
	    return supplierOpeningResultMapper.insertByBatch(ObeSupplierOpeningResultList);
	}
	
	/** 
	 * @Description: 批量更新指定字段数据
	 * @author mazc
	 * @param supplierOpeningResultList
	 * @param attributes
	 * @return 
	 */
	@Override
    public int batchUpdateProperty(List<SupplierOpeningResult> supplierOpeningResultList, String[] attributes) {
        return supplierOpeningResultMapper.batchUpdateProperty(supplierOpeningResultList, attributes);
    }

	/** 
	 * @Description: 指定字段修改
	 * @author chenxw
	 * @param supplierOpeningResult
	 * @param attributes
	 * @return 
	 */
	@Override
	@Transactional(rollbackFor = Exception.class)
    public boolean updateAssignProperty(SupplierOpeningResult supplierOpeningResult, String[] attributes) {
        return supplierOpeningResultMapper.updateAssignProperty(supplierOpeningResult, attributes) > 0;
    }

	/** 
	 * @Description: 指定字段批量保存或修改
	 * @author chenxw
	 * @param supplierList
	 * @param attributes
	 * @return 
	 */
	@Override
	@Transactional(rollbackFor = Exception.class)
    public Integer batchSaveOrUpdate(List<SupplierOpeningResult> supplierList, String[] attributes) {
        List<SupplierOpeningResult> insertSupplierOpeningResultList = new ArrayList<>();
        List<SupplierOpeningResult> updateSupplierOpeningResultList = new ArrayList<>();
        for (SupplierOpeningResult supplier : supplierList) {
            if (StringUtils.isEmpty(supplier.getId())) {
                supplier.setId(IDUtils.getId());
                insertSupplierOpeningResultList.add(supplier);
            } else {
                updateSupplierOpeningResultList.add(supplier);
            }
        }
        int count = 0;
        if (insertSupplierOpeningResultList.size() > 0) {
            count = count + supplierOpeningResultMapper.insertByBatch(insertSupplierOpeningResultList);
        }
        if (updateSupplierOpeningResultList.size() > 0) {
            count = count + supplierOpeningResultMapper.batchUpdateProperty(supplierList, attributes);
        }
        return count;
    }


	/** 
	 * @Description:  修改供应商的总价
	 * @author mazc
	 * @param tenderId
	 * @param bidPrice
	 * @param evalPrice 
	 */
	@Override
	public Integer updateSupplierSumPrice(String tenderId, boolean bidPrice, boolean evalPrice) {
		return updateSupplierSumPrice(tenderId, null, bidPrice, evalPrice);
	}

	/** 
	 * @Description: 修改供应商的总价
	 * @author mazc
	 * @param tenderId
	 * @param supplierId
	 * @param bidPrice
	 * @param evalPrice 
	 */
	@Override
	public Integer updateSupplierSumPrice(String tenderId, String supplierId, boolean bidPrice, boolean evalPrice) {
		return supplierOpeningResultMapper.updateSupplierSumPrice(tenderId, supplierId, bidPrice, evalPrice);
	}

	/**
	 * @Description: 获取投标人开标结果
	 * @author wangxiang
	 * @param tenderId
	 * @param supplierId
	 * @return 
	 */
	@Override
	public SupplierOpeningResult getSupplierOpeningResult(String tenderId, String supplierId) {
		return supplierOpeningResultMapper.getSupplierOpeningResult(tenderId,supplierId);
	}

	/**
	 * @Description: 获得开标结果主键
	 * @author wangxiang
	 * @param tenderId
	 * @param supplierId
	 * @return 
	 */
	@Override
	public String getSupplierOpeningResultId(String tenderId, String supplierId) {
		return supplierOpeningResultMapper.getSupplierOpeningResultId(tenderId, supplierId);
	}


	@Override
	public SupplierOpeningResult saveOrUpdateNoEmptyReturn(SupplierOpeningResult supplierOpeningResult) {
		if(null == supplierOpeningResult ) {
			return null;
		}
		if(StringUtils.isNotEmpty(supplierOpeningResult.getId())) {
			int size = supplierOpeningResultMapper.updateById(supplierOpeningResult);
			if(size > 0) {
				return supplierOpeningResultMapper.selectById(supplierOpeningResult.getId());
			}
			return supplierOpeningResult;
		}else {
			supplierOpeningResultMapper.insert(supplierOpeningResult);
			return supplierOpeningResult;
		}
 	}


	@Override
	public List<SupplierOpeningResult> getSupplierOpeningResultListByTenderId(String tenderId) {
		QueryWrapper<SupplierOpeningResult> queryWrapper = new QueryWrapper<>();
		queryWrapper.lambda().eq(SupplierOpeningResult::getTenderId, tenderId);
 		return supplierOpeningResultMapper.selectList(queryWrapper);
	}


	@Override
	public List<SupplierOpeningResult> getSupplierOpeningResultListByTenderIdAndBidOpeningStatus(String tenderId,
			String bidOpeningStatus) {
		QueryWrapper<SupplierOpeningResult> queryWrapper = new QueryWrapper<>();
		queryWrapper.lambda().eq(SupplierOpeningResult::getTenderId, tenderId).eq(SupplierOpeningResult::getBidOpeningStatus, bidOpeningStatus);
 		return supplierOpeningResultMapper.selectList(queryWrapper);
	}

	/**
	 * @Description: 汇总并更新修正投标报价原因
	 * @param tenderId
	 * @param supplierId
	 * @return
	 */
	@Override
	public boolean updateSumModifyReason(String tenderId, String supplierId) {
		StringBuilder modifyReason = new StringBuilder();
		List<BidPriceResult> bidPriceResultList = bidPriceResultService.getSupplierBidPriceResultList(tenderId, supplierId);
		if (CollectionUtils.isNull(bidPriceResultList)) return false;
		Iterator<BidPriceResult> iterator = bidPriceResultList.iterator();
		modifyReason.append(ObjectUtils.getObjString(iterator.next().getModifyReason()));
		while (iterator.hasNext())
			modifyReason.append("\r\n").append(ObjectUtils.getObjString(iterator.next().getModifyReason()));
		UpdateWrapper<SupplierOpeningResult> wrapper = new UpdateWrapper<>();
		wrapper.lambda()
				.set(SupplierOpeningResult::getModifyReason, modifyReason.toString())
				.eq(SupplierOpeningResult::getTenderId, tenderId)
				.eq(SupplierOpeningResult::getSupplierId, supplierId);
		return update(wrapper);
	}

}
