package com.gx.obe.server.management.user.controller;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.gx.obe.server.common.base.BaseController;
import com.gx.obe.server.management.user.entity.AuthLoginRecordEntity;
import com.gx.obe.server.management.user.service.AuthLoginRecordService;

import io.swagger.annotations.Api;
import lombok.AllArgsConstructor;

 /**  
 * 
 * @author 
 * @Title: AuthLoginRecordController  
 * @Package com.gx.obe.server.management.user.controller   
 * @Description:   
 * @author: XiaoMa Pro - myzhichao@163.com
 * @date:   2019-05-08   
 * @version V1.0 
 * @Copyright: 2019 www.msypro.com Inc. All rights reserved. 
 *  
 */
@Api(tags = "登陆记录管理")
@RestController
@AllArgsConstructor
@RequestMapping("/loginRecord")
public class AuthLoginRecordController  extends BaseController<AuthLoginRecordService,AuthLoginRecordEntity>{
	@Autowired
	private	AuthLoginRecordService authLoginRecordService;
	
	 /**   
	 * @Title: getAccountByLoginInfo   
	 * @Description: 通过登陆户账号 登陆类型查询
	 * @param: @param userAccount
	 * @param: @param loginSysType
	 * @param: @return  登陆信息
	 * @return: AuthLoginRecordEntity      
	 * @throws   
	 */  
	@GetMapping("/accountbyinfo")
	public AuthLoginRecordEntity getAccountByLoginInfo(String userAccount , String loginSysType){
		QueryWrapper<AuthLoginRecordEntity> queryWrapper = new QueryWrapper<>();
		queryWrapper.lambda().eq(AuthLoginRecordEntity::getUserAccount, userAccount).eq(AuthLoginRecordEntity::getLoginSysType, loginSysType);
		return authLoginRecordService.getOne(queryWrapper);
	}
	
	
	 /**   
	 * @Title: deleteAccountByLoginInfo   
	 * @Description: 通过登陆户账号 登陆类型删除信息
	 * @param: @param userAccount
	 * @param: @param loginSysType      
	 * @return: void      
	 * @throws   
	 */  
	@GetMapping("/deletebyinfo")
	public void deleteAccountByLoginInfo(String userAccount , String loginSysType){
		authLoginRecordService.deleteAccountByLoginInfo(userAccount, loginSysType);
	}
	
	
	/**
	 * @Description: 根据项目id获取项目经理登录信息
	 * @author mazc
	 * @param tenderId
	 * @return 
	 */
 	@GetMapping("/getLoginRecordByTenderId")
	public AuthLoginRecordEntity getLoginRecordByTenderId(String tenderId){
		return authLoginRecordService.getLoginRecordByTenderId(tenderId);
	}
 	
 	/**
 	 * @Description: 注销登录
 	 * @author mazc
 	 * @param account
 	 * @param sysType
 	 * @param loginRecordId
 	 * @param applyUserId 
 	 */
 	@GetMapping("/logout")
 	public void logout(String account, String sysType, String loginRecordId,String applyUserId){
 		 authLoginRecordService.logout(account, sysType, loginRecordId, applyUserId);
 	}
 	
 	/**
 	 * @Description: 获得评标室已登录的专家记录
 	 * @author mazc
 	 * @param evaluationRoomId
 	 * @return 
 	 */
 	@GetMapping("/getLoginRecordByEvalBidRoomId")
 	public List<AuthLoginRecordEntity> getLoginRecordByEvalBidRoomId(String evaluationRoomId){
 		return authLoginRecordService.getLoginRecordByEvalBidRoomId(evaluationRoomId);
 	}
 	
 	
	/**
	 * @Description: 获得评标室已登录的专家记录
	 * @author mazc
	 * @param evaluationRoomId
	 * @return 
	 */
 	@GetMapping("/getLoginRecordAllList")
	public List<AuthLoginRecordEntity> getLoginRecordAllList(){
 		return authLoginRecordService.getLoginRecordAllList();
	}
 	
 	/**
 	 * @Description:  是否已被注销
 	 * @author mazc
 	 * @param account
 	 * @param sysType
 	 * @param loginRecordId
 	 * @return 
 	 */
 	@GetMapping("/isLogout")
 	public boolean isLogout(String account, String sysType ,String loginRecordId){
 		return authLoginRecordService.isLogout(account, sysType, loginRecordId);
 	}
 	
}