package com.gx.obe.server.management.user.controller;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;

import javax.servlet.http.HttpServletRequest;

import com.google.gson.JsonObject;
import com.google.gson.JsonParser;
import com.gx.obe.http.HttpUtil;
import com.gx.obe.http.json.JsonUtil;
import com.gx.obe.server.common.utils.DESUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.gx.obe.server.common.base.BaseController;
import com.gx.obe.server.common.enumeration.CommonEnum;
import com.gx.obe.server.common.enumeration.UserEnum;
import com.gx.obe.server.common.log.OperationType;
import com.gx.obe.server.common.log.SysLogAnnotation;
import com.gx.obe.server.common.utils.IDUtils;
import com.gx.obe.server.common.utils.NowTimeUtils;
import com.gx.obe.server.common.utils.StringUtils;
import com.gx.obe.server.common.vo.Result;
import com.gx.obe.server.config.token.util.TokenUtil;
import com.gx.obe.server.management.user.entity.AuthRoleEntity;
import com.gx.obe.server.management.user.entity.AuthUserEntity;
import com.gx.obe.server.management.user.service.AuthRoleService;
import com.gx.obe.server.management.user.service.AuthUserService;
import com.gx.obe.server.management.vo.AuthUserVo;
import com.gx.obe.server.management.vo.UpdateAssignPropertyVo;

import io.swagger.annotations.Api;

/**
 * 
 * @author
 * @Title: AuthUserController
 * @Package com.gx.obe.server.management.user.controller
 * @Description:
 * @author: XiaoMa Pro - myzhichao@163.com
 * @date: 2019-04-25
 * @version V1.0
 * @Copyright: 2019 www.msypro.com Inc. All rights reserved.
 * 
 */
@Slf4j
@Api(tags = "用户模块")
@RestController
@RequestMapping("/authUser")
public class AuthUserController  extends BaseController<AuthUserService,AuthUserEntity>{ 

	@Autowired
	public AuthUserService authUserService;
	@Autowired
	public AuthRoleService authRoleService;

	@Value("${platformVarifyURL}")
	private String varifyUrl;
	

	@PostMapping("/updateUser")
	public boolean  updateUserInfo(@RequestBody AuthUserEntity user) {
		if(null != user && user.getUserType().length() > 2) {
			user.setUserType(null);
		}
		UpdateWrapper<AuthUserEntity> updateWrapper = new UpdateWrapper<AuthUserEntity>();
		updateWrapper.lambda().eq(AuthUserEntity::getUserAccount, user.getUserAccount());
		return authUserService.update(user,updateWrapper);	
	}
	
	@GetMapping("/valUserAccount")
	public AuthUserEntity geUserInfo(String userAccount) {
		QueryWrapper<AuthUserEntity> queryWrapper = new QueryWrapper<>();
		queryWrapper.lambda().eq(AuthUserEntity::getUserAccount, userAccount);
		AuthUserEntity user = authUserService.getOne(queryWrapper);
		if(user != null) {
			return user;
		}
		return null;
	}
	
	@GetMapping("/getUserRoleList")
	public List<AuthRoleEntity> getUserRole(String terminalType, String userId, String ignoreRoleType){
		return authRoleService.getUserRoleList(terminalType, userId, ignoreRoleType);
	}
	
	
	@GetMapping("/findUserByIdCardLogin")
	public AuthUserEntity findUserByIdCardLogin(String idCard , String passWord){
		QueryWrapper<AuthUserEntity> queryWrapper = new QueryWrapper<>();
		queryWrapper.lambda().eq(AuthUserEntity::getUserIdcard, idCard).eq(AuthUserEntity::getUserPassword, passWord);
		return authUserService.getOne(queryWrapper);
		
	}

	@GetMapping("/updatePassword")
	@SysLogAnnotation(detail="修改密码",level = 0,operationType = OperationType.UPDATE)
	public boolean updatePassword(String userAccount , String userPassword,String code){

		Map<String, Object> paramMap = new HashMap<String, Object>();
		paramMap.put("checkUserLogin",code);

		try {
				String resultJson = HttpUtil.post(varifyUrl, paramMap,50000);
				JsonParser jsonParser=new JsonParser();
				JsonObject bodyObject = jsonParser.parse(resultJson).getAsJsonObject();
				JsonObject dataObject = bodyObject.getAsJsonObject("data");
				String data  = dataObject.get("data").getAsString();
				String decrypt = DESUtils.decrypt(data, "utf-8");
				Map<String,String> resultMap = JsonUtil.strToMap(decrypt);
				if(resultMap.get("ret").equals("0")) {
					UpdateWrapper<AuthUserEntity> updateWrapper = new UpdateWrapper<>();
					updateWrapper.lambda().set(AuthUserEntity::getUserPassword, userPassword).set(AuthUserEntity::getModifyTime, NowTimeUtils.getNowDate()).eq(AuthUserEntity::getUserAccount, userAccount);
					return authUserService.update(updateWrapper);
				}
		} catch (Exception e) {
			log.error(e.getMessage());
			return false;
		}

		return false;

	}
	
	/**
	 * @Description: 更新分配属性
	 * @author chenxw
	 * @param updateAssignPropertyVo
	 * @return 
	 */
	@PostMapping("/updateAssignProperty")
    public boolean updateAssignProperty(@RequestBody UpdateAssignPropertyVo<AuthUserEntity> updateAssignPropertyVo) {
        AuthUserEntity authUserEntity = updateAssignPropertyVo.getEntity();
        String[] attributes = updateAssignPropertyVo.getAttributes();
        return authUserService.updateAssignProperty(authUserEntity, attributes);
    }
	
	@GetMapping("/valLoginAuthority")
	public boolean valLoginAuthority(String userId, String terminal){
		return authUserService.valLoginAuthority(userId, terminal);
	}
	
	
	@GetMapping("/login")
	@SysLogAnnotation(detail="用户登陆",level = 0,operationType = OperationType.SELECT)
	public AuthUserVo showlogin(@RequestParam String USER_ACCOUNT, @RequestParam String USER_PASSWORD) {
		HttpServletRequest request = ((ServletRequestAttributes) Objects.requireNonNull(RequestContextHolder.getRequestAttributes())).getRequest();
		QueryWrapper<AuthUserEntity> queryWrapper = new QueryWrapper<>();
		queryWrapper.lambda().eq(AuthUserEntity::getUserAccount, USER_ACCOUNT);
		AuthUserVo vo = new AuthUserVo();
		AuthUserEntity user = authUserService.getOne(queryWrapper);
		vo.setAuthUserEntity(user);
		if(user != null) {
			if (USER_PASSWORD.equals(user.getUserPassword())) {
				String token= TokenUtil.sign(user,request);
				vo.setAtuokthen(token);
				return vo;
			}
		}
		return null;


	}
	
	@PostMapping("/loginRemote")
	@SysLogAnnotation(detail="用户远程登陆",level = 0,operationType = OperationType.SELECT)
	public AuthUserVo loginRemote(@RequestBody AuthUserEntity remoteUser) {
		HttpServletRequest request = ((ServletRequestAttributes) Objects.requireNonNull(RequestContextHolder.getRequestAttributes())).getRequest();
		QueryWrapper<AuthUserEntity> queryWrapper = new QueryWrapper<>();
		queryWrapper.lambda().eq(AuthUserEntity::getUserAccount, remoteUser.getUserAccount());
		AuthUserEntity user = authUserService.getOne(queryWrapper);
		String userId = null;
		if(user != null) {
			userId = user.getId();
			UpdateWrapper<AuthUserEntity> updateWrapper = new UpdateWrapper<>();
			updateWrapper.lambda().set(AuthUserEntity::getUserPassword, remoteUser.getUserPassword());
			updateWrapper.lambda().set(AuthUserEntity::getModifyTime, NowTimeUtils.getNowDate());
			if(!UserEnum.USER_SOURCE_REMOTE.equals(remoteUser.getSource())) {
				updateWrapper.lambda().set(AuthUserEntity::getSource, UserEnum.USER_SOURCE_REMOTE);
			}
			updateWrapper.lambda().eq(AuthUserEntity::getId, user.getId());
			
			authUserService.update(updateWrapper);
		}else {
			userId = null != remoteUser.getId() ? remoteUser.getId() : IDUtils.getId();
			remoteUser.setId(userId);
			remoteUser.setCreateTime(NowTimeUtils.getNowDate());
			remoteUser.setModifyTime(remoteUser.getCreateTime());;
			remoteUser.setUserPassword(remoteUser.getUserPassword());
			remoteUser.setUseStatus(CommonEnum.REAL_STATUS);
			remoteUser.setSource(UserEnum.USER_SOURCE_REMOTE);
			authUserService.save(remoteUser);
		}
		AuthUserEntity queryUser = authUserService.getById(userId);
		AuthUserVo vo = new AuthUserVo();
		vo.setAuthUserEntity(queryUser);
		String token= TokenUtil.sign(queryUser,request);
		vo.setAtuokthen(token);
		return vo;
	}
	
	@GetMapping("/caLogin")
	public AuthUserEntity caLogin(@RequestParam String caName, @RequestParam String certId) {
		return authUserService.caLogin(caName, certId);
	}
	
	@GetMapping("/getUserByIdCard")
	public AuthUserEntity getUserByIdCard(@RequestParam String idCard) {
		QueryWrapper<AuthUserEntity> queryWrapper = new QueryWrapper<>();
		queryWrapper.lambda().eq(AuthUserEntity::getUserIdcard, idCard);
		return authUserService.getOne(queryWrapper);
	}
	
	/**
	 * @Description: 判断用户身份证是否存在
	 * @author mazc
	 * @param userAccount
	 * @param id
	 * @param userIdcard
	 * @return 
	 */
 	@GetMapping("/valAccountAndIdCard")
	public int valAccountAndIdCard(String userAccount, String id, String userIdcard) {
		QueryWrapper<AuthUserEntity> queryWrapper = new QueryWrapper<AuthUserEntity>();
		LambdaQueryWrapper<AuthUserEntity> lambda = queryWrapper.lambda();
		if(StringUtils.isNotEmpty(userAccount)) {
			lambda.eq(AuthUserEntity::getUserAccount, userAccount);
		}
		if(StringUtils.isNotEmpty(id)){
			lambda.ne(AuthUserEntity::getId,id);
		}
		if(StringUtils.isNotEmpty(userIdcard)) {
			lambda.eq(AuthUserEntity::getUserIdcard, userIdcard);
		}
		return authUserService.list(queryWrapper).size();
	}
	
 	/**
 	 * @Description: 列出所有的指定状态的用户
 	 * @author mazc
 	 * @return 
 	 */
 	@GetMapping("/getUserAllByStatus")
	public List<AuthUserEntity> getUserAllByStatus(String useStatus ,String userType){
 		return authUserService.getUserAllByStatus(useStatus,userType);
	}
 	
 	@GetMapping("/getUserByName")
 	public AuthUserEntity getUserByName(String userName){
 		return authUserService.getUserByName(userName);
 	}
 	
	/**
	 * @Description: 添加或删除
	 * @author mazc
	 * @param authUserEntity
	 * @return 
	 */
	@PostMapping("/saveOrUpdateReturn")
	@SysLogAnnotation(detail="新增用户",level = 0,operationType = OperationType.INSERT)
	public AuthUserEntity saveOrUpdateReturn(@RequestBody AuthUserEntity authUserEntity){
		return authUserService.saveOrUpdateReturn(authUserEntity);
	}
	
	/**
	 * @Description: 通过用户account获取用户
	 * @author mazc
	 * @param userAccount
	 * @return 
	 */
	@GetMapping("/getUserByUserAccount")
	public AuthUserEntity getUserByUserAccount(String userAccount) {
		return authUserService.getUserByUserAccount(userAccount);
	}
	
	
	/**
	 * @Description: 通过用户account获取用户
	 * @author mazc
	 * @param userAccount
	 * @return 
	 */
	@GetMapping("/getUserAllByUserAccount")
	public List<AuthUserEntity> getUserAllByUserAccount(String userAccount) {
		return authUserService.getUserAllByUserAccount(userAccount);
	}
	
	
	/**
	 * @Description: 通过项目ID，查询当前项目下所有专家的用户信息
	 * @author mazc
	 * @param projectId
	 * @return 
	 */
	@GetMapping("/getExpertByProjectId")
	public List<AuthUserEntity> getExpertByProjectId(String projectId) {
		return authUserService.getExpertByProjectId(projectId);
	}
	
	/**
	 * @Description: 获取当前系统临时用户数量
	 * @author mazc
	 * @return 
	 */
	@GetMapping("/getTempUserNum")
	public int getTempUserNum(){
		return authUserService.getTempUserNum();
	}
	
	
	/**
	 * @Description: 通过账号或者身份证验证用户是否存在
	 * @author mazc
	 * @param userAccont
	 * @param idCard
	 * @return 
	 */
	@GetMapping("/getUserByIdOrAccont") 
	public AuthUserEntity getUserByIdOrAccont(String userAccont, String idCard){
		return authUserService.getUserByIdOrAccont(userAccont, idCard);
	}
	
	/**
	 * @Description: 根据用户名和用户类型，获得用户列表。
	 * @author mazc
	 * @param userType
	 * @param userName
	 * @return 
	 */
	@GetMapping("/getUserListByUserName") 
	public List<AuthUserEntity> getUserListByUserName(String userType, String userName) {
		return authUserService.getUserListByUserName(userType, userName);
	}
	
	/**
	 * @Description: 更新使用状态
	 * @author mazc
	 * @param id
	 * @param useStatus
	 * @return 
	 */
	@GetMapping("/updateUsestatus")
	public int updateUsestatus(String id ,String useStatus) {
		return authUserService.updateUsestatus(id, useStatus);
	}
	
	@GetMapping("/getUserPageList")
	public IPage<AuthUserEntity> getUserPageList(String account,String userName,String roleType, int pageNo, int pageSize){
		 Page<AuthUserEntity> page = new Page<AuthUserEntity>(pageNo,pageSize);
		 return authUserService.getUserPageList(page, account, userName, roleType);
	}
	
	
	/**
	 * @Description: 条件查找用户
	 * @author mazc
	 * @param userAccount
	 * @param userName
	 * @param userType
	 * @return 
	 */
	@GetMapping("/getUserAllByUserTypeUserNameOrUserAccount")
	public List<AuthUserEntity> getUserAllByUserTypeUserNameOrUserAccount(String userAccount ,String userName, String userType){
		return authUserService.getUserAllByUserTypeUserNameOrUserAccount(userAccount,userName,userType);
	}
	
	
	/**
	 * @Description: 通过
	 * @author mazc
	 * @param id
	 * @return 
	 */
	@GetMapping("/getGroupItemUserList")
	public List<AuthUserEntity> getGroupItemUserList(String id) {
		return authUserService.getGroupItemUserList(id);
	}
	
	
 	@GetMapping("/login/login")
//	@SysLogAnnotation(detail="用户登陆",level = 0,operationType = OperationType.SELECT)
	public  Result<AuthUserVo>  login(@RequestParam String USER_ACCOUNT, @RequestParam String USER_PASSWORD) {
		QueryWrapper<AuthUserEntity> queryWrapper = new QueryWrapper<>();
		queryWrapper.lambda().eq(AuthUserEntity::getUserAccount, USER_ACCOUNT);
		AuthUserEntity user = authUserService.getOne(queryWrapper);
		if(user != null) {
			if (USER_PASSWORD.equals(user.getUserPassword())) {
				HttpServletRequest request = ((ServletRequestAttributes) Objects.requireNonNull(RequestContextHolder.getRequestAttributes())).getRequest();
				AuthUserVo vo = new AuthUserVo();
 				vo.setAuthUserEntity(user);
				vo.setAtuokthen(TokenUtil.sign(user,request));
				return new Result<AuthUserVo>(vo, "登录陈功!", Result.SUCCESS);
			}else {
//				AuthUserVo vo = new AuthUserVo();
// 				vo.setAuthUserEntity(user);
//				return new Result<AuthUserVo>(vo, "密码错误",Result.FAIL);
				AuthUserVo vo = new AuthUserVo();
				AuthUserEntity AuthUserEntity = new AuthUserEntity();
				AuthUserEntity.setUserAccount(USER_ACCOUNT);
				vo.setAuthUserEntity(AuthUserEntity);
				return new Result<AuthUserVo>(vo, "密码错误",Result.FAIL);
			}
		}
		return new Result<AuthUserVo>(null, "用户不存在!",Result.FAIL);
 	}


	@PostMapping({"/updatePhone"})
	public String updatePhone(@RequestParam("id") String id, @RequestParam("oldPhone") String oldPhone, @RequestParam("newPhone") String newPhone) {
		return this.authUserService.updatePhone(id, oldPhone, newPhone);
	}
	
	
}