package com.gx.obe.server.management.user.service;

import java.util.List;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.IService;
import com.gx.obe.server.management.evaluation.entity.EvaluationStep;
import com.gx.obe.server.management.user.entity.AuthUserEntity;

/**  
 * 
 * @author 
 * @Title: AuthUserService
 * @Package com.gx.obe.server.management.user.service  
 * @Description:    服务类
 * @author: XiaoMa Pro - myzhichao@163.com
 * @date:   2019-04-25   
 * @version V1.0 
 * @Copyright: 2019 www.msypro.com Inc. All rights reserved. 
 *  
 */
public interface AuthUserService extends IService<AuthUserEntity> {
	
	
	/**
	 * @Description: 验证登录权限
	 * @author mazc
	 * @param userId
	 * @param terminal 终端
	 * @return 
	 */
	boolean valLoginAuthority(String userId, String terminal);
	
	
	/**
	 * @Description: CA 登陆
	 * @author mazc
	 * @param caName
	 * @param certId
	 * @return 
	 */
	AuthUserEntity caLogin( String caName,  String certId);
	
	/**
	 * @Description: 获得评标步骤的评标专家
	 * @author mazc
	 * @param tenderId
	 * @param evaluationStep
	 * @return 
	 */
	List<AuthUserEntity>findEvaluationStepUserList(String tenderId, EvaluationStep evaluationStep);
	
	
	
	/**
	 * @Description: 查询项目所对用的用户信息  
	 * @author mazc
	 * @param tenderId
	 * @return 
	 */
	AuthUserEntity findUserInfoByTenderId(String tenderId);
	
	
	/**
	 * @Description: 获取所有指定用户状态的用户信息
	 * @author mazc
	 * @param useStatus
	 * @return 
	 */
	List<AuthUserEntity> getUserAllByStatus(String useStatus, String userType);
	
	
	/**
	 * @Description: 通过name获取用户信息
	 * @author mazc
	 * @param userName
	 * @return 
	 */
	AuthUserEntity getUserByName(String userName);
	
	/**
	 * @Description: 更新或保存
	 * @author mazc
	 * @param authUserEntity
	 * @return 
	 */
	AuthUserEntity saveOrUpdateReturn(AuthUserEntity authUserEntity);
	
	
	 /**
	 * @Description: 通过用户account获取用户
	 * @author mazc
	 * @param userAccount
	 * @return 
	 */
	AuthUserEntity getUserByUserAccount(String userAccount);
	
	
	/**
	 * @Description:通过项目ID，查询当前项目下所有专家的用户信息 
	 * @author mazc
	 * @param projectId
	 * @return 
	 */
	List<AuthUserEntity> getExpertByProjectId(String projectId);
	
	/**
	 * @Description:获取当前系统临时用户数量 
	 * @author mazc
	 * @return 
	 */
	int getTempUserNum();

	 /**
	 * @Description: 通过账号或者身份证验证用户是否存在
	 * @author mazc
	 * @param userAccont
	 * @param idCard
	 * @return 
	 */
	AuthUserEntity getUserByIdOrAccont(String userAccont, String idCard);
	
	
	/**
	 * @Description: 根据用户名和用户类型，获得用户列表。
	 * @author mazc
	 * @param userType
	 * @param userName
	 * @return 
	 */
	List<AuthUserEntity> getUserListByUserName(String userType, String userName);
	
	
	/**
	 * @Description: 更新使用状态
	 * @author mazc
	 * @param id
	 * @param useStatus
	 * @return 
	 */
	int updateUsestatus(String id ,String useStatus); 
	
	/**
	 * @Description: 通过用户身份id 获取用户信息
	 * @author mazc
	 * @param idCard
	 * @return 
	 */
	AuthUserEntity getUserByIdCard( String idCard);
	
	/**
	 * @Description: 通过用户登陆名获取列表
	 * @author mazc
	 * @param userAccount
	 * @return 
	 */
	List<AuthUserEntity> getUserAllByUserAccount(String userAccount); 

	/**
	 * @Description: 
	 * @author mazc
	 * @param userAccount
	 * @param userName
	 * @param userType
	 * @return 
	 */
	List<AuthUserEntity> getUserAllByUserTypeUserNameOrUserAccount(String userAccount ,String userName, String userType);
	
	
	IPage<AuthUserEntity> getUserPageList(Page<AuthUserEntity> page,String account,String userName,String roleType);


    /**
     * @Description: 更新分配属性
     * @author chenxw
     * @param authUserEntity
     * @param attributes
     * @return 
     */
    boolean updateAssignProperty(AuthUserEntity authUserEntity, String[] attributes);
    
    
    /**
     * @Description: 
     * @author mazc
     * @param id
     * @return 
     */
    List<AuthUserEntity> getGroupItemUserList(String id);

	String updatePhone(String id, String oldPhone, String newPhone);
}

