package com.gx.obe.server.management.user.service.impl;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.gx.obe.server.management.user.dao.AuthRoleMapper;
import com.gx.obe.server.management.user.entity.AuthRoleEntity;
import com.gx.obe.server.management.user.service.AuthRoleService;
   /**  
 * 
 * @author 
 * @Title: AuthRoleService
 * @Package com.gx.obe.server.management.user.service  
 * @Description:     服务实现类
 * @author: XiaoMa Pro - myzhichao@163.com
 * @date:   2019-04-29   
 * @version V1.0 
 * @Copyright: 2019 www.msypro.com Inc. All rights reserved. 
 *  
 */
@Service
public class AuthRoleServiceImpl extends  ServiceImpl<AuthRoleMapper, AuthRoleEntity> implements AuthRoleService {
	@Autowired
	private AuthRoleMapper authRoleMapper;

	/** 
	 * @Description: 根据角色类型获取角色列表
	 * @author mazc
	 * @param roleType
	 * @return 
	 */
	@Override
	public List<AuthRoleEntity> getRoleListByRoleType(String roleType) {
		return authRoleMapper.getRoleListByRoleType(roleType);
	}
	
	@Override
	public List<AuthRoleEntity> getUserRoleList(String terminalType, String userId, String ignoreRoleType){
 		return authRoleMapper.getUserRoleList(terminalType, userId, ignoreRoleType);
	}


	/** 
	 * @Description: 根据加色名字查找角色列表
	 * @author mazc
	 * @param roleName
	 * @return 
	 */
	@Override
	public List<AuthRoleEntity> getQueryRoleListByRoleName(String roleName) {
		return authRoleMapper.getQueryRoleListByRoleName(roleName);
	}

	/** 
	 * @Description: 根据角色状态获取列表
	 * @author mazc
	 * @param useStatus
	 * @return 
	 */
	@Override
	public List<AuthRoleEntity> getRoleListByuseStatus(String useStatus) {
		QueryWrapper<AuthRoleEntity> queryWrapper = new QueryWrapper<AuthRoleEntity>();
		queryWrapper.lambda().eq(AuthRoleEntity::getUseStatus, useStatus);
		return authRoleMapper.selectList(queryWrapper);
	}

	/** 
	 * @Description: 更新角色状态
	 * @author mazc
	 * @param useStatus
	 * @param roleId
	 * @return 
	 */
	@Override
	public boolean updateRoleuseStatus(String useStatus,String roleId) {
		UpdateWrapper<AuthRoleEntity> updateWrapper = new UpdateWrapper<AuthRoleEntity>();
		updateWrapper.set("USE_STATUS", useStatus);
		updateWrapper.lambda().eq(AuthRoleEntity::getId, roleId);
		return authRoleMapper.update(null, updateWrapper) > 0;
 	}

	/** 
	 * @Description: 
	 * @author mazc
	 * @param roleCode
	 * @return 
	 */
	@Override
	public List<AuthRoleEntity> getRoleListByRoleCode(String[] roleCode) {
		QueryWrapper<AuthRoleEntity> queryWrapper = new QueryWrapper<AuthRoleEntity>();
		if(null != roleCode && roleCode.length == 1){
			queryWrapper.lambda().eq(AuthRoleEntity::getRoleCode, roleCode[0]);		
		}else {
			queryWrapper.lambda().in(AuthRoleEntity::getRoleCode, roleCode);
		}
		return authRoleMapper.selectList(queryWrapper);
	}
 
}
